//============================================================================
//
//      Bcs.h - Interface definition for Bcs.dll
//
//      Copyright  2007 Microvision, Inc.
//      All Rights Reserved.
//
//============================================================================

#ifndef __BCS_H__
#define __BCS_H__

#ifndef __oaidl_h__
#include "oaidl.h"					// VARIANT
#endif

//--------------------------- Version Info -----------------------------------

#define BCS_VER_MAJOR  0			// BCS_VER_MAJOR will be set to 1, and BCS_VER_MINOR set to 0 for the first release candidate.
#define BCS_VER_MINOR  7			// BCS_VER_MINOR will be incremented for each change to the interface before release.
#define BCS_VER_BUILD  8			// BCS_VER_BUILD will be incremented with each internal release and reset to 0 for each version number change.

#define BCS_VERSION_STRING( major, minor, build ) #major "." #minor ".0." #build
#define BCS_VERSION_EXPAND( major, minor, build ) BCS_VERSION_STRING( major, minor, build )

// This version information is stored as a resource in Bcs.dll which can be displayed
// using Windows Explorer or programmatically through GetFileVersionInfo, VerQueryValue, etc.

#define BCS_COMPANY_NAME			"Microvision, Inc."
#define BCS_FILE_DESCRIPTION		"Scanner API"
#define BCS_FILE_VERSION			BCS_VERSION_EXPAND( BCS_VER_MAJOR, BCS_VER_MINOR, BCS_VER_BUILD )
#define BCS_INTERNAL_NAME			"Bcs"
#define BCS_LEGAL_COPYRIGHT			"Copyright  2007 Microvision, Inc."	// "Copyright  2007 " BCS_COMPANY_NAME
#define BCS_ORIGINAL_FILENAME		"Bcs.dll"
#define BCS_PRODUCT_NAME			"Scanner SDK"
#define BCS_PRODUCT_VERSION			BCS_FILE_VERSION

//--------------------------- Return Codes -----------------------------------

// All Bcs functions return an HRESULT.  A return value of 0 (S_OK) indicates SUCCESS
// Negative values indicate that an error occurred.  Positive values are informational.

// Error return codes from SetupDi*, Bluetooth*, Winsock errors, etc. are returned from
// the Bcs functions as HRESULTs.  Descriptions for these errors can be found programmatically
// by calling FormatMessage.  Descriptions are also available in BcsMsgs and in <winerror.h> or
// via the Tools | Error Lookup menu and tooltips while debugging in Visual Studio.

// Other errors and informational messages are defined in BcsMsgs.h and their descriptions are 
// compiled into the Bcs DLL as a binary resource available through the FormatMessage function.

// Any errors specific the the Scanner SDK will have have the facility code set to 
// FACILITY_ITF.  For more information on facility codes see <winerror.h>

// Additional languages can be added to BscMsgs.mc.
// Use "mc -o BscMsgs.mc" to compile binary resources.

#include "BcsMsgs.h"

// The following typedef is used to interpret WM_DEVICECHANGE messages

typedef struct {							// _DEV_BROADCAST_USERDEFINED
	struct {								// _DEV_BROADCAST_HDR
		DWORD       dbch_size;
		DWORD       dbch_devicetype;
		DWORD       dbch_reserved;
	} dbud_dbh;
    char        dbud_szName[36];			// ASCIIZ name ("Microvision\<event>")
//  BYTE        dbud_rgbUserDefined[];		// User-defined contents
} BcsDevBroadcast;


//--------------------------- Function Interface -----------------------------

#ifdef __cplusplus
extern "C" {    // Assume C declarations for C++
#endif          // __cplusplus

#ifdef _LIB
#define BCS_API
#else
// The following ifdef block is the standard way of creating macros which make exporting 
// from a DLL simpler. All files within this DLL are compiled with the BCS_EXPORTS
// symbol defined on the command line. this symbol should not be defined on any project
// that uses this DLL. This way any other project whose source files include this file see 
// BCS_API functions as being imported from a DLL, whereas this DLL sees symbols

// defined with this macro as being exported.
#ifdef BCS_EXPORTS
#define BCS_API __declspec(dllexport)
#else
#ifndef _WIN32_WCE
#define BCS_API __declspec(dllimport)
#else
#define BCS_API
#endif
#endif
#endif

DECLARE_HANDLE(HBCS);								// Barcode Scanner Connection handle

// This structure holds either a Bluetooth address or a COM port number
typedef union {										// Identical to BLUETOOTH_ADDRESS
    ULONGLONG ullLong;								// Easier to compare against BLUETOOTH_NULL_ADDRESS
    BYTE    rgBytes[ 6 ];							// Easier to format
} BCS_CONNECTION_ADDRESS;

// This value is used in place of a Bluetooth "Class of Device" to indicate that a BCS_CONNECTION_INFO contains serial connection information
#define BCS_COD_SERIAL_PORT			(0xffffffff)	// Indicates that the BCS_CONNECTION_INFO structure is for a serial connection
// This value is the class of device reported by the Flic and Rov Bluetooth battery doors
#define BCS_COD_BLUETOOTH_DOOR		(0x00000500)	// Indicates that the BCS_CONNECTION_INFO structure is for a Bluetooth connection

// These values are used to subdivide the szName field of a BLUETOOTH_DEVICE_INFO structure to hold serial connection information
#define BCS_MAX_NAME_SIZE			128				// 127 Unicode characters + NUL (i.e. "Microvision Scanner USB Interface")
#define BCS_MAX_PORTNAME_SIZE		  8				//   7 Unicode characters + NUL (i.e. "COM20")
#define BCS_MAX_INSTANCEID_SIZE		112				// 111 Unicode characters + NUL (i.e. "HID\VID_041F&PID_0104\6&31EA2B5D&0&0000")

// Connection information structure that can contain information for either a Bluetooth or Serial connection
typedef struct {									// Interchangeable with BLUETOOTH_DEVICE_INFO
    DWORD   dwSize;									// Size, in bytes, of this structure - must be the sizeof BLUETOOTH_DEVICE_INFO

	BCS_CONNECTION_ADDRESS Address;					// Bluetooth address or COM port number

    ULONG   ulClassofDevice;						// Bluetooth "Class of Device" or BCS_COD_SERIAL_PORT

    BOOL    fConnected;								// Device connected/in use
    BOOL    fRemembered;							// Device remembered
    BOOL    fAuthenticated;							// Device authenticated/paired/bonded

    SYSTEMTIME  stLastSeen;							// Last time the device was seen
    SYSTEMTIME  stLastUsed;							// Last time the device was used for other than RNR, inquiry, or SDP

	WCHAR szName[		BCS_MAX_NAME_SIZE		];	// Name of the device.
	WCHAR szPortName[	BCS_MAX_PORTNAME_SIZE	];	// PortName of the serial device
	WCHAR szInstanceId[ BCS_MAX_INSTANCEID_SIZE	];	// Instance ID of the serial device

} BCS_CONNECTION_INFO;

// NOTE: The instance id field can be used to retrieve additional information about a serial device:
//       Use SetupDiCreateDeviceInfoList function to create an empty device information set.
//       Then use SetupDiOpenDeviceInfo to locate a device directly from its device instance id:
//       i.e. SetupDiOpenDeviceInfo( hDevInfo, szDeviceInstanceId, NULL, 0, &DeviceInfoData );

// BcsEnumConnections callback delaration
typedef HRESULT (CALLBACK *PFN_BCS_CONNECTION_CALLBACK)(void *pvParam, BCS_CONNECTION_INFO *pConnectionInfo);

typedef enum {
	eEventFlicBanner,								// Banner received from the scanner
	eEventDataReceived,								// Data received from the scanner
	eEventBarCodeDataReceived,						// Data identified as a bar code (not implemented)
	eEventCommandPacket,							// Packet that will be sent to the scanner
	eEventResponsePacket,							// Packet received from the scanner
	eEventBluetoothAttemptingToConnect,				// Attempting to connect with Bluetooth device
	eEventBluetoothConnectionLost,					// Connection with Bluetooth device lost
	eEventBluetoothDeviceConnected					// Bluetooth connection established
	// Additional event types will be added here.  Applications should return S_OK from the event callback function for any unrecognized event types.
} BCS_EVENT_TYPE;

#define BCS_MAX_DATA_RESPONSE_SIZE	16				// 16 bytes

// Data packet information structure received through the pEvent parameter of an event callback when the event type is eDataReceived
typedef struct {									// Used to recieve and reply to data from the scanner
    DWORD   dwSize;									// Size, in bytes, of this structure - must be the sizeof BCS_DATA

	BYTE   *pDataReceived;							// Pointer to the bytes received from the scanner
    int		cbDataReceived;							// [in] Count of bytes received from the scanner, and [out] number of bytes to discard

    BYTE    rgResponse[BCS_MAX_DATA_RESPONSE_SIZE];	// Response data
    int		cbResponse;								// Number of bytes to send in response
} BCS_DATA;

// These values are bitwise or'd together to describe how to proceed from an event callback
#define BCS_DATA_DISCARD			0x00			// Discard cbData bytes and continue processing (Same as S_OK)
#define BCS_DATA_MORE				0x01			// Do not discard any bytes and continue reading from port
#define BCS_DATA_REPLY				0x02			// Send cbResponse bytes from rgResponse as a response and continue processing
#define BCS_DATA_MASK				0x03			// Bit mask of valid BCS_DATA bits

typedef HRESULT (CALLBACK *PFN_BCS_EVENT_CALLBACK)(void *pvParam, BCS_EVENT_TYPE eEventType, void *pEvent);

// These values are bitwise or'd together to determine the behavior of the BcsEnumConnections function
#define BCS_ENUM_SERIAL				0x01			// Show potential serial connections
#define BCS_ENUM_BLUETOOTH			0x02			// Show Bluetooth devices with a Class of Device of 0x500 (COD of Flic)
#define BCS_ENUM_DISCOVER			0x04			// Issue an inquiry to discover new Bluetooth devices
#define BCS_ENUM_SELECT				0x08			// Call BluetoothSelectDevices or equivalent to discover Bluetooth using a common dialog
#define BCS_ENUM_APPEND_BDA			0x10			// Append Bluetooth Device Address (BDA) if device has default name (i.e. "Cordless")
#define BCS_ENUM_FRIENDLYNAME		0x20			// Return serial device names using "friendly" (SPDRP_FRIENDLYNAME) format (Appends " (COM#)")
#define BCS_ENUM_MASK				0x3f			// Bit mask of valid BCS_ENUM bits

// BcsOpenConnection (dwBaudRate and pdwBaudRate)
#define BCS_UNKNOWN_BAUDRATE		0				// Unknown (or not applicable) baud rate (sets BCS_OPEN_DETECT_BAUDRATE flag bit for serial connections)

// These values are bitwise or'd together to determine the behavior of the BcsOpenConnection function
#define BCS_OPEN_DETECT_SCANNER		0x01			// Detect the existence of a scanner
#define BCS_OPEN_DETECT_BAUDRATE	0x02			// Auto detect the baud rate. The dwBaudRate parameter is treated as a hint
#define BCS_OPEN_FAIL_NO_DETECT		0x04			// Fail to open the port and return an error code if BCS_OPEN_DETECT_SCANNER fails
#define BCS_OPEN_GET_BANNER			0x08			// Get the Microvision banner from the scanner and send it via the event callback function.
#define BCS_OPEN_PACKET_EVENTS		0x10			// Generate an event for each M2M packet.
#define BCS_OPEN_MASK				0x1f			// Bit mask of valid BCS_OPEN bits

//--------------------- Connection Management Functions ----------------------

// Calls a callback function for each potential connection
BCS_API HRESULT BcsEnumConnections(
    CONST GUID *pGuidClass,							// IN  Reserved, must be NULL
    HWND        hwndParent,							// IN  Window handle to be used as the parent window for any dialogs displayed by this function
    PFN_BCS_CONNECTION_CALLBACK pfnConnectionCallback,	// IN A callback that will be called for each Connection.
													//	If the callback returns >= 0, the enumeration will continue.
													//	If the callback returns < 0, the enumeration will stop.
    void *pvParam,									// IN  Parameter to be passed to pfnConnectionCallback as the pvParam
    DWORD dwFlags									// IN  Bitwise or'd combination of BCS_ENUM bits
    );

// Invokes the Control Panel's device information property sheet
BCS_API HRESULT BcsDisplayConnectionProperties(
    HWND hwndParent,								// IN  Window handle to be used as the parent window for dialog displayed by this function
    BCS_CONNECTION_INFO *pConnectionInfo,			// IN  A connection information structure (usually retrieved via BcsEnumConnections)
    DWORD dwFlags									// IN  Reserved, must be 0
    );

// Removes authentication between a Bluetooth device and the computer, purging any cached information about the device
BCS_API HRESULT BcsRemoveConnection(
    BCS_CONNECTION_INFO *pConnectionInfo,			// IN  A connection information structure (usually retrieved via BcsEnumConnections)
    DWORD dwFlags									// IN  Reserved, must be 0
    );

// Opens a connection using a connection information structure
BCS_API HRESULT BcsOpenConnection(
	HBCS *phBcs,									// OUT A connection handle for this connection
    BCS_CONNECTION_INFO *pConnectionInfo,			// IN  A connection information structure (usually retrieved via BcsEnumConnections)
	DWORD dwBaudRate,								// IN  A specific baud rate (4800|9600|19200|38400|57600|115200) or BCS_UNKNOWN_BAUDRATE
	DWORD *pdwBaudRate,								// OUT Baud rate of the connection.  Can be NULL.
    PFN_BCS_EVENT_CALLBACK pfnEventCallback,		// IN  A callback that will be called for each event.
    void *pvParam,									// IN  Parameter to be passed to pfnEventCallback as the pvParam
    DWORD dwFlags									// IN  Bitwise or'd combination of BCS_OPEN bits or 0
    );

// Creates a connection using an ASCII string
BCS_API HRESULT BcsOpenPortA(
	HBCS *phBcs,									// OUT A connection handle for this connection
    const char *pszPort,							// IN  An ASCII string containing containing a port name or Bluetooth address hexadecimal string
    ULONG ulClassofDevice,							// IN  Bluetooth "Class of Device" or BCS_COD_SERIAL_PORT
	DWORD dwBaudRate,								// IN  A specific baud rate (4800|9600|19200|38400|57600|115200) or BCS_UNKNOWN_BAUDRATE
	DWORD *pdwBaudRate,								// OUT Baud rate of the connection.  Can be NULL.
    PFN_BCS_EVENT_CALLBACK pfnEventCallback,		// IN  A callback that will be called for each event.
    void *pvParam,									// IN  Parameter to be passed to pfnEventCallback as the pvParam
    DWORD dwFlags									// IN  Bitwise or'd combination of BCS_OPEN bits or 0
    );

// Creates a connection using a WCHAR string
BCS_API HRESULT BcsOpenPortW(
	HBCS *phBcs,									// OUT A connection handle for this connection
    const WCHAR *pszPort,							// IN  A WCHAR string containing containing a port name or Bluetooth address hexadecimal string
    ULONG ulClassofDevice,							// IN  Bluetooth "Class of Device" or BCS_COD_SERIAL_PORT
	DWORD dwBaudRate,								// IN  A specific baud rate (4800|9600|19200|38400|57600|115200) or BCS_UNKNOWN_BAUDRATE
	DWORD *pdwBaudRate,								// OUT Baud rate of the connection.  Can be NULL.
    PFN_BCS_EVENT_CALLBACK pfnEventCallback,		// IN  A callback that will be called for each event.
    void *pvParam,									// IN  Parameter to be passed to pfnEventCallback as the pvParam
    DWORD dwFlags									// IN  Bitwise or'd combination of BCS_OPEN bits or 0
    );

#ifdef UNICODE
#define BcsOpenPort		BcsOpenPortW
#else
#define BcsOpenPort		BcsOpenPortA
#endif // !UNICODE

// Returns the connection status: S_OK - connected, S_FALSE - disconnected (Scanner turned off Bluetooth radio, scanner unplugged, etc.)
BCS_API HRESULT BcsIsConnected(
	HBCS hBcs,										// IN  The connection handle provided by BcsOpenConnection
    DWORD dwFlags									// IN  Reserved, must be 0
    );

// Close a connection
BCS_API HRESULT BcsCloseConnection(
	HBCS hBcs										// IN  The connection handle provided by BcsOpenConnection
    );

//------------------------ Data Management Functions -------------------------

#define BCS_MAX_BARCODE_SIZE		96				// Maximum length of a bar code

// Get the number of available bar codes
BCS_API HRESULT BcsGetBarCodeCount(
	HBCS hBcs,										// IN  The connection handle provided by BcsOpenConnection
	int *pCount										// OUT Number of bar codes available to be retreived
	);

// ASCII Bar code download information structure
typedef struct {									// Parsed data element attributes
    DWORD		 dwSize;							// Size, in bytes, of this structure
    const char * pszDeviceSn;						// 'dev'	Device Serial Number
	int			 nPacketId;							// 'id'		Download packet identifier
	int			 nBarCodeCount;						// 'bcc'	Number of bar codees stored in the scanner at the time the download data packet was sent
    const char * pszDownloadTimestamp;				// 'dt'		Value of scanners internal clock at time of download (19 chars + NUL).  Only valid if Timestamp = True.
	VARIANT		 varDownloadTimestamp;				// 'dt'		Value of scanners internal clock at time of download expressed as a VARIANT.  DATE type if conversion possible.
	int			 nBatteryEventCount;				// 'ec'		Battery event count (e# for next bar code).  Missing value indicated by -1.
} BCS_DATA_ATTRIBUTESA;

typedef struct {									// Parsed bc element attributes
    DWORD		 dwSize;							// Size, in bytes, of this structure
	const char * pszSymIdType;						// 'si'		Symbology identifier type ("Off", "AIM", or "Custom")
	const char * pszTimestamp;						// 'ts'		Timestamp
	const char * pszTimeType;						// 'tt'		Time type (OK, or eN - where N = number of power cycles since "Set Time")
} BCS_BC_ATTRIBUTESA;

typedef struct {
    DWORD					dwSize;					// Size, in bytes, of this structure
	BCS_DATA_ATTRIBUTESA *	pDataAttribs;			// Pointer to data element attributes structure
	BCS_BC_ATTRIBUTESA *	pBcAttribs;				// Pointer to bc element attributes structure
	const void *			pReserved;				// Reserved for internal use
} BCS_DOWNLOAD_INFOA;

// BcsGetBarCodesA callback delaration
typedef HRESULT (CALLBACK *PFN_BCS_BARCODE_CALLBACKA)(
    void *		pvParam,							// IN  Parameter passed into BcsGetBarCodesA as the pvParam
	const char *pszSymbology,						// IN  Symbol Identifier (This value is dependent on the setting of the SymIdType property)
	const char *pszTimestamp,						// IN  Timestamp (This pointer value is only valid if Timestamp = True, otherwize the the pointer value is NULL)
	const char *pszTimeType,						// IN  Time Type (This pointer value is only valid if Timestamp = True, otherwize the the pointer value is NULL)
													//	   Possible Values: "ok" = Actual Time, or e1, e2, ..., e<N> = Estimated time after N battery events)
	const char *pszBarCode,							// IN  Bar code data
	int         cchBarCode,							// IN  Length of the bar code data in bytes

	BCS_DOWNLOAD_INFOA *pDownloadInfo				// IN  Supplemental information in download packet
	);

// WCHAR Bar code download information structures
typedef struct {									// Parsed data element attributes
    DWORD		 dwSize;							// Size, in bytes, of this structure
    const WCHAR *pszDeviceSn;						// 'dev'	Device Serial Number
	int			 nPacketId;							// 'id'		Download packet identifier
	int			 nBarCodeCount;						// 'bcc'	Number of bar codees stored in the scanner at the time the download data packet was sent
    const WCHAR *pszDownloadTimestamp;				// 'dt'		Value of scanners internal clock at time of download (19 chars + NUL).  Only valid if Timestamp = True.
	VARIANT		 varDownloadTimestamp;				// 'dt'		Value of scanners internal clock at time of download expressed as a VARIANT.  DATE type if conversion possible.
	int			 nBatteryEventCount;				// 'ec'		Battery event count (e# for next bar code).  Missing value indicated by -1.
} BCS_DATA_ATTRIBUTESW;

typedef struct {									// Parsed bc element attributes
    DWORD		 dwSize;							// Size, in bytes, of this structure
	const WCHAR *pszSymIdType;						// 'si'		Symbology identifier type ("Off", "AIM", or "Custom")
	const WCHAR *pszTimestamp;						// 'ts'		Timestamp
	const WCHAR *pszTimeType;						// 'tt'		Time type (OK, or eN - where N = number of power cycles since "Set Time")
} BCS_BC_ATTRIBUTESW;

typedef struct {
    DWORD					dwSize;					// Size, in bytes, of this structure
	BCS_DATA_ATTRIBUTESW *	pDataAttribs;			// Pointer to data element attributes structure
	BCS_BC_ATTRIBUTESW *	pBcAttribs;				// Pointer to bc element attributes structure
	const void *			pReserved;				// Reserved for internal use
} BCS_DOWNLOAD_INFOW;

// BcsGetBarCodesW callback delaration
typedef HRESULT (CALLBACK *PFN_BCS_BARCODE_CALLBACKW)(
    void *		 pvParam,							// IN  Parameter passed into BcsGetBarCodesA as the pvParam
	const WCHAR *pszSymbology,						// IN  Symbol Identifier (This value is dependent on the setting of the SymIdType property)
	const WCHAR *pszTimeStamp,						// IN  Timestamp (This pointer value is only valid if Timestamp = True, otherwize the the pointer value is NULL)
	const WCHAR *pszTimeType,						// IN  Time Type (This pointer value is only valid if Timestamp = True, otherwize the the pointer value is NULL)
													//	   Possible Values: "ok" = Actual Time, or e1, e2, ..., e<N> = Estimated time after N battery events)
	const WCHAR *pszBarCode,						// IN  Bar code data
	int			 cchBarCode,						// IN  Length of the bar code data in characters
	BCS_DOWNLOAD_INFOW *pDownloadInfo				// IN  Supplemental information in download packet
	);

// Pass this value for the nDelPacketId parameter of BcsGetBarCodes to indicate that there is no packet to delete
#define BCS_BC_NO_PACKET_ID		-1

// Pass this value for the nMaxBarCodes parameter of BcsGetBarCodes to get all available bar codes via the callback function
#define BCS_BC_GET_ALL			-1

// These values are bitwise or'd together determine the behavior of the BcsGetBarCodes function
#define BCS_BC_AUTO_DELETE			0x01			// Automatically delete all bar codes if BcsGetBarCodes does not return an error.
#define BCS_BC_ADJUST_TIMESTAMPS	0x02			// Adjust certain timestamp values (pszTimeStamp) and change pszTypeType accordingly.  See documentation for details.
#define BCS_BC_MASK					0x03			// Bit mask of valid BCS_BC bits

///--------------------------------------------------------------------------------------
///
/// <summary>
/// Send a download request to the scanner. If the scanner contains bar codes then the
/// bar codes are sent in a batch and a callback is activated for each bar code received.
/// This function returns after the callback function has been called for all bar codes
/// in the current batch.
/// </summary>
///
/// <param name=              "hBcs">
/// [IN ] Connection handle provided by BcsOpenConnection.</param>
/// <param name=       nDelPacketId">
/// [IN ] Packet identifier of packet to delete (use BCS_BC_NO_PACKET_ID to NOT delete).
/// <param name=       nMaxBarCodes">
/// [IN ] Maximum number of bar codes to retrieve or BCS_BC_GET_ALL to get all available
/// bar codes.
/// <param name= pfnBarCodeCallback">
/// [IN ] A callback that is called for each bar code downloaded in this batch.
/// If the callback returns >= 0, the enumeration continues.
/// If the callback returns < 0, the enumeration stops and the value is returned.
/// <param name=           *pvParam">
/// [IN ] Parameter to be passed to pfnBarCodeCallback as the pvParam.
/// <param name=      *pDelPacketId">
/// [OUT] The packet identifier of the current packet. This is often passed as the
/// nDelPacketId parameter on the next call to BcsGetBarCodes (to delete the previous
/// packet).
/// <param name=*pRemainingBarCodes">
/// [OUT] Pointer to the number of bar codes remaining in the scanner after the host
/// receives this batch of bar codes.
/// <param name=            dwFlags">
/// [IN ] 0 or BCS_BC_AUTO_DELETE. If zero then the received bar codes are retained in
/// scanner memory until the host specifically deletes them. With BCS_BC_AUTO_DELETE the
/// entire batch of bar codes is deleted after each has been successfully received.
///
///--------------------------------------------------------------------------------------
BCS_API HRESULT BcsGetBarCodes(
	HBCS						hBcs,
	int							nDelPacketId,
	int							nMaxBarCodes,
    PFN_BCS_BARCODE_CALLBACKA	pfnBarCodeCallback,
	void *						pvParam,
	int  *						pDelPacketId,
	int  *						pRemainingBarCodes,
    DWORD						dwFlags);
#undef BcsGetBarCodes

BCS_API HRESULT BcsGetBarCodesA(					// Calls a callback function for each bar code
	HBCS hBcs,										// IN  The connection handle provided by BcsOpenConnection
	int  nDelPacketId,								// IN  Packet identifier of packet to delete or BCS_BC_NO_PACKET_ID to skip the delete
	int  nMaxBarCodes,								// IN  Maximum number of bar codes to retrieve or BCS_BC_GET_ALL to get all available bar codes
    PFN_BCS_BARCODE_CALLBACKA pfnBarCodeCallback,	// IN  A callback that will be called for each Connection.
													//	If the callback returns >= 0, the enumeration will continue.
													//	If the callback returns < 0, the enumeration will stop and the value will be returned.
    void *pvParam,									// IN  Parameter to be passed to pfnBarCodeCallback as the pvParam
	int  *pDelPacketId,								// OUT Pointer to the packet identifier for use in deleting the bar codes
	int  *pRemainingBarCodes,						// OUT Number of bar codes remaining after download (and delete if BCS_BC_AUTO_DELETE)
    DWORD dwFlags									// IN  Bitwise or'd combination of BCS_BC_AUTO_DELETE and BCS_BC_ADJUST_TIMESTAMPS or 0
	);

BCS_API HRESULT BcsGetBarCodesW(					// Calls a callback function for each bar code
	HBCS hBcs,										// IN  The connection handle provided by BcsOpenConnection
	int  nDelPacketId,								// IN  Packet identifier of packet to delete (-1 indicates don't delete)
	int  nMaxBarCodes,								// IN  Maximum number of bar codes to retrieve or BCS_BC_GET_ALL to get all available bar codes
    PFN_BCS_BARCODE_CALLBACKW pfnBarCodeCallback,	// IN A callback that will be called for each Connection.
													//	If the callback returns >= 0, the enumeration will continue.
													//	If the callback returns < 0, the enumeration will stop and the value will be returned.
    void *pvParam,									// IN  Parameter to be passed to pfnBarCodeCallback as the pvParam
	int  *pDelPacketId,								// OUT Pointer to the packet identifier for use in deleting the bar codes
	int  *pRemainingBarCodes,						// OUT Pointer to the number of bar codes remaining after download (and delete if BCS_BC_AUTO_DELETE)
    DWORD dwFlags									// IN  Bitwise or'd combination of BCS_BC_AUTO_DELETE and BCS_BC_ADJUST_TIMESTAMPS or 0 
	);

#ifdef UNICODE
#define BCS_DOWNLOAD_INFO			BCS_DOWNLOAD_INFOW
#define PFN_BCS_BARCODE_CALLBACK	PFN_BCS_BARCODE_CALLBACKW
#define BcsGetBarCodes				BcsGetBarCodesW
#else
#define BCS_DOWNLOAD_INFO			BCS_DOWNLOAD_INFOA
#define PFN_BCS_BARCODE_CALLBACK	PFN_BCS_BARCODE_CALLBACKA
#define BcsGetBarCodes				BcsGetBarCodesA
#endif // !UNICODE

//---------------------- Scanner Management Functions ------------------------

#define MAX_XSP_COMMAND	224		// Currently limited by Bluetooth packet size and packet overhead

// These values are bitwise or'd together determine the behavior of the Bcs Scanner Management functions (NOTE:  May not support BCS_XSP_BSTR in release code)
// NOTE:  Actually, the only flags defined so far are all mutually exclusive
#define BCS_XSP_XML		0x01	// Return the raw XSP XML packet as a BSTR		(No XML parsing)
#define BCS_XSP_BSTR	0x02	// Return the raw XSP reply element as a BSTR	(No type conversion)
#define BCS_XSP_MASK	0x03	// Bit mask of valid BCS_XSP bits

// Input arguments and return values are parsed and converted to the appropriate data type automatically.
// There are a number of support functions and wrapper classes for VARIANTs, allowing them to be easily created, converted, formatted, etc.
// Scripting engines such as VBScript include native support for VARIANTs.

///--------------------------------------------------------------------------------------
///
/// <summary>
/// Use BcsCommand to send operational commands to the scanner. The pszCommand parameter
/// is the XSP command string that is passed through to the XSP layer.  Use BcsCommand to
/// issue any XSP command including download commands, clear commands, or commands to get
/// or set scanner properties (also handled by BcsGetProp and BcsSetProp). See the XSP
/// manual for a complete list of scanner properties.
/// </summary>
///
/// <param name=       "hBcs">[IN ] Connection handle provided by BcsOpenConnection.</param>
/// <param name="*pszCommand">[IN ] XSP command string.</param>
/// <param name=       "lcid">[IN ] Locale used to both interpret pVarArg and format pVarResult.</param>
/// <param name=   "*pVarArg">[IN ] Command argument.</param>
/// <param name="*pVarResult">[OUT] Data returned from the command.</param>
/// <param name=    "dwFlags">[IN ] Accepts 0, BCS_XSP_XML or BCS_XSP_BSTR.</param>
///
///--------------------------------------------------------------------------------------
BCS_API HRESULT BcsCommand(	// Send a Command to the Scanner
	HBCS		hBcs,
    const char *pszCommand,
	LCID		lcid,
	VARIANT		*pVarArg,
    VARIANT		*pVarResult,
    DWORD		dwFlags);
#undef BcsCommand

BCS_API HRESULT BcsCommandA(	// Send a Command to the Scanner
	HBCS hBcs,					// IN  The connection handle provided by BcsOpenConnection
    const char *pszCommand,		// IN  ASCII version of the command
	LCID lcid,					// IN  Locale used to both interpret pVarArg and format pVarResult
	VARIANT *pVarArg,			// IN  Command argument
    VARIANT *pVarResult,		// OUT Data returned from Command
    DWORD dwFlags				// IN  Accepts 0, BCS_XSP_XML or BCS_XSP_BSTR
    );

BCS_API HRESULT BcsCommandW(	// Send a Command to the Scanner
	HBCS hBcs,					// IN  The connection handle provided by BcsOpenConnection
    const WCHAR *pszCommand,	// IN  WCHAR version of the command
	LCID lcid,					// IN  Locale used to both interpret pVarArg and format pVarResult
	VARIANT *pVarArg,			// IN  Command argument
    VARIANT *pVarResult,		// OUT Data returned from Command
    DWORD dwFlags				// IN  Accepts 0, BCS_XSP_XML or BCS_XSP_BSTR
    );

///--------------------------------------------------------------------------------------
///
/// <summary>
/// Use BcsSetProp to set the value of a configuration property in the scanner. The
/// property parameter must match the XSP Get Command Strings and are passed through to
/// the XSP layer. See the XSP manual for a complete list of scanner properties.
/// </summary>
///
/// <param name=        hBcs">[IN] Connection handle provided by BcsOpenConnection.</param>
/// <param name=*pszPropName">[IN] String version of property name.</param>
/// <param name=        lcid">[IN] Locale used to interpret pVarValue.</param>
/// <param name=  *pVarValue">[IN] New propery value.</param>
/// <param name=     dwFlags">[IN] Reserved, must be 0 .</param>
///
///--------------------------------------------------------------------------------------
BCS_API HRESULT BcsSetProp(   // Set a Scanner Configuration Property
      HBCS        hBcs,
      const TCHAR *pszPropName,
      LCID        lcid,
      VARIANT           *pVarValue,
    DWORD         dwFlags);
#undef BcsSetProp


BCS_API HRESULT BcsSetPropA(	// Set a Scanner Configuration Property
	HBCS hBcs,					// IN  The connection handle provided by BcsOpenConnection
	const char *pszPropName,	// IN  ASCII version of property name
	LCID lcid,					// IN  Locale used to interpret pVarValue
	VARIANT *pVarValue,			// IN  New propery value
    DWORD dwFlags				// IN  Reserved, must be 0
	);

BCS_API HRESULT BcsSetPropW(	// Set a Scanner Configuration Property
	HBCS hBcs,					// IN  The connection handle provided by BcsOpenConnection
	const WCHAR *pszPropName,	// IN  WCHAR version of property name
	LCID lcid,					// IN  Locale used to interpret pVarValue
	VARIANT *pVarValue,			// IN  New propery value
    DWORD dwFlags				// IN  Reserved, must be 0 
	);

///--------------------------------------------------------------------------------------
///
/// <summary>
/// Use BcsGetProp to read the value of a configuration property in the scanner. The
/// property parameter must match the XSP Get Command Strings and are passed through to
/// the XSP layer. See the XSP manual for a complete list of scanner properties.
/// </summary>
///
/// <param name=       "hBcs">[IN ] Connection handle provided by BcsOpenConnection.</param>
/// <param name="pszPropName">[IN ] Property name string.</param>
/// <param name=       "lcid">[IN ] Locale used to format pVarResult.</param>
/// <param name="*pVarResult">[OUT] Returned propery value.</param>
/// <param name=    "dwFlags">[IN ] Accepts 0, BCS_XSP_XML, or BCS_XSP_BSTR.</param>
///
///--------------------------------------------------------------------------------------
BCS_API HRESULT BcsGetProp(
      HBCS        hBcs,
      const TCHAR *pszPropName,
      LCID        lcid,
      VARIANT           *pVarResult,
    DWORD         dwFlags);
#undef BcsGetProp

BCS_API HRESULT BcsGetPropA(	// Get a Scanner Configuration Property
	HBCS hBcs,					// IN  The connection handle provided by BcsOpenConnection
	const char *pszPropName,	// IN  ASCII version of property name
	LCID lcid,					// IN  Locale used to format pVarResult
	VARIANT *pVarResult,		// OUT Returned propery value
    DWORD dwFlags				// IN  Accepts 0, BCS_XSP_XML or BCS_XSP_BSTR
	);

BCS_API HRESULT BcsGetPropW(	// Get a Scanner Configuration Property
	HBCS hBcs,					// IN  The connection handle provided by BcsOpenConnection
	const WCHAR *pszPropName,	// IN  WCHAR version of property name
	LCID lcid,					// IN  Locale used to format pVarResult
	VARIANT *pVarResult,		// OUT Returned propery value
    DWORD dwFlags				// IN  Accepts 0, BCS_XSP_XML or BCS_XSP_BSTR
	);

#ifdef UNICODE
#define BcsCommand	BcsCommandW
#define BcsSetProp	BcsSetPropW
#define BcsGetProp	BcsGetPropW
#else
#define BcsCommand  BcsCommandA
#define BcsSetProp	BcsSetPropA
#define BcsGetProp	BcsGetPropA
#endif // !UNICODE

//---------------------------- Signal Function -------------------------------

BCS_API HRESULT _cdecl BcsSignal(
	HBCS hBcs,					// IN  The connection handle provided by BcsOpenConnection
	int nParamSets,				// IN  Number of parameter sets (1 through 8)
	int nFrequency,				// IN  Frequency in kHz, If freq = 0, beeper is silent
	int nDuration,				// IN  Duration in 0.5 ms intervals
	BOOL bLed0,					// IN  Led0 is On (1) or Off (0) during the signal
	BOOL bLed1,					// IN  Led1 is On (1) or Off (0) during the signal
	...
	);

#ifdef __cplusplus
}
#endif  // __cplusplus
#endif // __BCS_H__
