 /*

 BcsMsgs.mc : Microvision Barcode Scanner SDK messages

      Copyright 2007 Microvision, Inc.
      All Rights Reserved.

 This file contains the message definitions for the Scanner SDK and
 overrides of Winsock and Win32 error messages to make them SDK specific
 and appropriate for display in the Scanner Wedge status bar.

-------------------------------------------------------------------------
 HEADER SECTION

 The header section defines names and language identifiers for use
 by the message definitions later in this file. The MessageIdTypedef,
 SeverityNames, FacilityNames, and LanguageNames keywords are
 optional and not required.


 MessageIdTypedef = HRESULT

 The MessageIdTypedef keyword gives a typedef name that is used in a
 type cast for each message code in the generated include file. Each
 message code appears in the include file with the format: #define
 name ((type) 0xnnnnnnnn) The default value for type is empty, and no
 type cast is generated. It is the programmer's responsibility to
 specify a typedef statement in the application source code to define
 the type. The type used in the typedef must be large enough to
 accomodate the entire 32-bit message code.

 The SeverityNames keyword defines the set of names that are allowed
 as the value of the Severity keyword in the message definition. The
 set is delimited by left and right parentheses. Associated with each
 severity name is a number that, when shifted left by 30, gives the
 bit pattern to logical-OR with the Facility value and MessageId
 value to form the full 32-bit message code. The default value of
 this keyword is:

 SeverityNames=(
   Success			= 0x0
   Informational	= 0x1
   Warning			= 0x2
   Error			= 0x3
   )

 Severity values occupy the high two bits of a 32-bit message code.
 Any severity value that does not fit in two bits is an error. The
 severity codes can be given symbolic names by following each value
 with :name

 The default severity values map the values used by Windows NT.
 The Barcode Scanner SDK uses HRESULTS which use only the high bit.
 The FacilityNames keyword defines the set of names that are allowed
 as the value of the Facility keyword in the message definition. The
 set is delimited by left and right parentheses. Associated with each
 facility name is a number that, when shift it left by 16 bits, gives
 the bit pattern to logical-OR with the Severity value and MessageId
 value to form the full 32-bit message code. The default value of
 this keyword is:

 FacilityNames=(
   System=0x0FF
   Application=0xFFF
   )

 Facility codes occupy the low order 12 bits of the high order
 16-bits of a 32-bit message code. Any facility code that does not
 fit in 12 bits is an error. This allows for 4,096 facility codes.
 The first 256 codes are reserved for use by the system software. The
 facility codes can be given symbolic names by following each value
 with :name


 The LanguageNames keyword defines the set of names that are allowed
 as the value of the Language keyword in the message definition. The
 set is delimited by left and right parentheses. Associated with each
 language name is a number and a file name that are used to name the
 generated resource file that contains the messages for that
 language. The number corresponds to the language identifier to use
 in the resource table. The number is separated from the file name
 with a colon.


 Any new names in the source file which don't override the built-in
 names are added to the list of valid languages. This allows an
 application to support private languages with descriptive names.


-------------------------------------------------------------------------
 MESSAGE DEFINITION SECTION

 Following the header section is the body of the Message Compiler
 source file. The body consists of zero or more message definitions.
 Each message definition begins with one or more of the following
 statements:

 MessageId = [number|+number]
 Severity = severity_name
 Facility = facility_name
 SymbolicName = name

 The MessageId statement marks the beginning of the message
 definition. A MessageID statement is required for each message,
 although the value is optional. If no value is specified, the value
 used is the previous value for the facility plus one. If the value
 is specified as +number then the value used is the previous value
 for the facility, plus the number after the plus sign. Otherwise, if
 a numeric value is given, that value is used. Any MessageId value
 that does not fit in 16 bits is an error.

 The Severity and Facility statements are optional. These statements
 specify additional bits to OR into the final 32-bit message code. If
 not specified they default to the value last specified for a message
 definition. The initial values prior to processing the first message
 definition are:

 Severity=Success
 Facility=Application

 The value associated with Severity and Facility must match one of
 the names given in the FacilityNames and SeverityNames statements in
 the header section. The SymbolicName statement allows you to
 associate a C/C++ symbolic constant with the final 32-bit message
 code.
 */

// Overrides of Win32 error messages
//
//  Values are 32 bit values layed out as follows:
//
//   3 3 2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1
//   1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
//  +-+-+-+-+-+---------------------+-------------------------------+
//  |S|R|C|N|r|    Facility         |               Code            |
//  +-+-+-+-+-+---------------------+-------------------------------+
//
//  where
//
//      S - Severity - indicates success/fail
//
//          0 - Success
//          1 - Fail (COERROR)
//
//      R - reserved portion of the facility code, corresponds to NT's
//              second severity bit.
//
//      C - reserved portion of the facility code, corresponds to NT's
//              C field.
//
//      N - reserved portion of the facility code. Used to indicate a
//              mapped NT status value.
//
//      r - reserved portion of the facility code. Reserved for internal
//              use. Used to indicate HRESULT values that are not status
//              values, but are instead message ids for display strings.
//
//      Facility - is the facility code
//
//      Code - is the facility's status code
//
//
// Define the facility codes
//


//
// Define the severity codes
//


//
// MessageId: BCS_E_ACCESS_DENIED
//
// MessageText:
//
//  Access is denied
//
#define BCS_E_ACCESS_DENIED              0x80070005L

//
// MessageId: BCS_E_NOT_ENOUGH_MEMORY
//
// MessageText:
//
//  Not enough storage is available to process this command
//
#define BCS_E_NOT_ENOUGH_MEMORY          0x80070008L

//
// MessageId: BCS_E_NOT_READY
//
// MessageText:
//
//  The device is not ready
//
#define BCS_E_NOT_READY                  0x80070015L

//
// MessageId: BCS_E_GEN_FAILURE
//
// MessageText:
//
//  A device attached to the system is not functioning
//
#define BCS_E_GEN_FAILURE                0x8007001FL

//
// MessageId: BCS_E_REQ_NOT_ACCEP
//
// MessageText:
//
//  No more connections can be made at this time
//
#define BCS_E_REQ_NOT_ACCEP              0x80070047L

//
// MessageId: BCS_E_VC_DISCONNECTED
//
// MessageText:
//
//  The session was canceled
//
#define BCS_E_VC_DISCONNECTED            0x800700F0L

//
// MessageId: BCS_E_WAIT_TIMEOUT
//
// MessageText:
//
//  The wait operation timed out
//
#define BCS_E_WAIT_TIMEOUT               0x80070102L

//
// MessageId: BCS_E_DEVICE_NOT_CONNECTED
//
// MessageText:
//
//  The device is not connected
//
#define BCS_E_DEVICE_NOT_CONNECTED       0x8007048FL

//
// MessageId: BCS_E_CANCELED
//
// MessageText:
//
//  The operation was canceled by the user
//
#define BCS_E_CANCELED                   0x800704C7L

//
// MessageId: BCS_E_NOT_AUTHENTICATED
//
// MessageText:
//
//  The user has not been authenticated
//
#define BCS_E_NOT_AUTHENTICATED          0x800704DCL

// Overrides of Winsock error messages
// Private (base at FACILITY_ITF + 0xBC00) informational messages
//
// MessageId: BCS_MSG_READY
//
// MessageText:
//
//  Ready
//
#define BCS_MSG_READY                    0x00000000L

//
// MessageId: BCS_MSG_NO_BLUETOOTH
//
// MessageText:
//
//  Bluetooth not installed
//
#define BCS_MSG_NO_BLUETOOTH             0x0004BC00L

//
// MessageId: BCS_MSG_NO_RADIOS
//
// MessageText:
//
//  No Bluetooth radios found
//
#define BCS_MSG_NO_RADIOS                0x0004BC01L

//
// MessageId: BCS_MSG_SEARCHING
//
// MessageText:
//
//  Searching...
//
#define BCS_MSG_SEARCHING                0x0004BC02L

//
// MessageId: BCS_MSG_DISCOVER_CANCELED
//
// MessageText:
//
//  Bluetooth device discovery canceled
//
#define BCS_MSG_DISCOVER_CANCELED        0x0004BC03L

//
// MessageId: BCS_MSG_NO_CONNECTIONS
//
// MessageText:
//
//  No scanner connections found
//
#define BCS_MSG_NO_CONNECTIONS           0x0004BC04L

//
// MessageId: BCS_MSG_NO_DEVICES
//
// MessageText:
//
//  No Bluetooth devices found
//
#define BCS_MSG_NO_DEVICES               0x0004BC05L

//
// MessageId: BCS_MSG_NO_SCANNERS
//
// MessageText:
//
//  No scanners found
//
#define BCS_MSG_NO_SCANNERS              0x0004BC06L

//
// MessageId: BCS_MSG_SCANNER_NOT_DETECTED
//
// MessageText:
//
//  Scanner not detected
//
#define BCS_MSG_SCANNER_NOT_DETECTED     0x0004BC07L

//
// MessageId: BCS_MSG_CONNECTING
//
// MessageText:
//
//  Connecting to scanner
//
#define BCS_MSG_CONNECTING               0x0004BC08L

//
// MessageId: BCS_MSG_AUTHENTICATING
//
// MessageText:
//
//  Authenticating scanner
//
#define BCS_MSG_AUTHENTICATING           0x0004BC09L

//
// MessageId: BCS_MSG_INVALID_PASSKEY
//
// MessageText:
//
//  The passkey entered does not match the passkey of the scanner
//
#define BCS_MSG_INVALID_PASSKEY          0x0004BC0AL

//
// MessageId: BCS_MSG_CONNECTED
//
// MessageText:
//
//  Connected
//
#define BCS_MSG_CONNECTED                0x0004BC0BL

//
// MessageId: BCS_MSG_ATTEMPTING_RECONNECT
//
// MessageText:
//
//  Attempting to reconnect scanner
//
#define BCS_MSG_ATTEMPTING_RECONNECT     0x0004BC0CL

//
// MessageId: BCS_MSG_SCANNER_NOT_RESPONDING
//
// MessageText:
//
//  Scanner inactive or out of range
//
#define BCS_MSG_SCANNER_NOT_RESPONDING   0x0004BC0DL

// Private (base at FACILITY_ITF + 0xBC00) error messages

#define BCS_E_BASE                       0x8000BC00L

//
// MessageId: BCS_E_MALFORMED_PACKET
//
// MessageText:
//
//  Flic error malformed plus packet
//
#define BCS_E_MALFORMED_PACKET           0x8004BC01L

//
// MessageId: BCS_E_READ_OVERFLOW
//
// MessageText:
//
//  Flic error read overflow
//
#define BCS_E_READ_OVERFLOW              0x8004BC02L

//
// MessageId: BCS_E_NOT_PLUS_PACKET
//
// MessageText:
//
//  Flic error not a plus packet
//
#define BCS_E_NOT_PLUS_PACKET            0x8004BC03L

//
// MessageId: BCS_E_NO_PACKET
//
// MessageText:
//
//  Flic no packet recieved
//
#define BCS_E_NO_PACKET                  0x8004BC04L

//
// MessageId: BCS_E_NAK
//
// MessageText:
//
//  Flic NAK received
//
#define BCS_E_NAK                        0x8004BC05L

//
// MessageId: BCS_E_UNEXPECTED_PACKET
//
// MessageText:
//
//  Flic error: ununexpected packet
//
#define BCS_E_UNEXPECTED_PACKET          0x8004BC07L

//
// MessageId: BCS_E_INVALID_OPTION
//
// MessageText:
//
//  Flic error: unknown option requested
//
#define BCS_E_INVALID_OPTION             0x8004BC08L

//
// MessageId: BCS_E_INVALID_HANDLE
//
// MessageText:
//
//  Flic error: serial port not open or valid
//
#define BCS_E_INVALID_HANDLE             0x8004BC09L

//
// MessageId: BCS_E_INVALID_CRC
//
// MessageText:
//
//  Flic error: packet with invalid CRC received from HOST
//
#define BCS_E_INVALID_CRC                0x8004BC0AL

//
// MessageId: BCS_E_NO_BARCODE
//
// MessageText:
//
//  Flic error: no barcode available
//
#define BCS_E_NO_BARCODE                 0x8004BC0BL

//
// MessageId: BCS_E_NOT_CONNECTED
//
// MessageText:
//
//  Flic error: Flic not connected
//
#define BCS_E_NOT_CONNECTED              0x8004BC0CL

//
// MessageId: BCS_E_BLUETOOTH_STACK_ERROR
//
// MessageText:
//
//  Error: Bluetooth stack error
//
#define BCS_E_BLUETOOTH_STACK_ERROR      0x8004BC0DL

//
// MessageId: BCS_E_FLIC_NOT_FOUND
//
// MessageText:
//
//  Error: No Flic detected
//
#define BCS_E_FLIC_NOT_FOUND             0x8004BC0EL

//
// MessageId: BCS_E_DUPLICATE_PACKET
//
// MessageText:
//
//  Flic error: Duplicate sequence number received
//
#define BCS_E_DUPLICATE_PACKET           0x8004BC0FL

//
// MessageId: BCS_E_INVALID_SEQUENCE_NUMBER
//
// MessageText:
//
//  Flic error: Sequence number out of sequence
//
#define BCS_E_INVALID_SEQUENCE_NUMBER    0x8004BC10L

