; /*
;
; BcsMsgs.mc : Microvision Barcode Scanner SDK messages
;
;      Copyright 2007 Microvision, Inc.
;      All Rights Reserved.
;
; This file contains the message definitions for the Scanner SDK and
; overrides of Winsock and Win32 error messages to make them SDK specific
; and appropriate for display in the Scanner Wedge status bar.
;
;-------------------------------------------------------------------------
; HEADER SECTION
;
; The header section defines names and language identifiers for use
; by the message definitions later in this file. The MessageIdTypedef,
; SeverityNames, FacilityNames, and LanguageNames keywords are
; optional and not required.
;
;
; MessageIdTypedef = HRESULT
;
; The MessageIdTypedef keyword gives a typedef name that is used in a
; type cast for each message code in the generated include file. Each
; message code appears in the include file with the format: #define
; name ((type) 0xnnnnnnnn) The default value for type is empty, and no
; type cast is generated. It is the programmer's responsibility to
; specify a typedef statement in the application source code to define
; the type. The type used in the typedef must be large enough to
; accomodate the entire 32-bit message code.
;

SeverityNames = (
  Success		= 0x0
  Informational	= 0x0
  Warning		= 0x2
  Error			= 0x2
  )

; The SeverityNames keyword defines the set of names that are allowed
; as the value of the Severity keyword in the message definition. The
; set is delimited by left and right parentheses. Associated with each
; severity name is a number that, when shifted left by 30, gives the
; bit pattern to logical-OR with the Facility value and MessageId
; value to form the full 32-bit message code. The default value of
; this keyword is:
;
; SeverityNames=(
;   Success			= 0x0
;   Informational	= 0x1
;   Warning			= 0x2
;   Error			= 0x3
;   )
;
; Severity values occupy the high two bits of a 32-bit message code.
; Any severity value that does not fit in two bits is an error. The
; severity codes can be given symbolic names by following each value
; with :name
;
; The default severity values map the values used by Windows NT.
; The Barcode Scanner SDK uses HRESULTS which use only the high bit.

FacilityNames = (
	Null      = 0
	Dispatch  = 2
	Bcs       = 4
	Win32     = 7
	)

; The FacilityNames keyword defines the set of names that are allowed
; as the value of the Facility keyword in the message definition. The
; set is delimited by left and right parentheses. Associated with each
; facility name is a number that, when shift it left by 16 bits, gives
; the bit pattern to logical-OR with the Severity value and MessageId
; value to form the full 32-bit message code. The default value of
; this keyword is:
;
; FacilityNames=(
;   System=0x0FF
;   Application=0xFFF
;   )
;
; Facility codes occupy the low order 12 bits of the high order
; 16-bits of a 32-bit message code. Any facility code that does not
; fit in 12 bits is an error. This allows for 4,096 facility codes.
; The first 256 codes are reserved for use by the system software. The
; facility codes can be given symbolic names by following each value
; with :name
;
;
; The LanguageNames keyword defines the set of names that are allowed
; as the value of the Language keyword in the message definition. The
; set is delimited by left and right parentheses. Associated with each
; language name is a number and a file name that are used to name the
; generated resource file that contains the messages for that
; language. The number corresponds to the language identifier to use
; in the resource table. The number is separated from the file name
; with a colon.
;
LanguageNames=(English	= 0x409:BCS00409)
;
; Any new names in the source file which don't override the built-in
; names are added to the list of valid languages. This allows an
; application to support private languages with descriptive names.
;
;
;-------------------------------------------------------------------------
; MESSAGE DEFINITION SECTION
;
; Following the header section is the body of the Message Compiler
; source file. The body consists of zero or more message definitions.
; Each message definition begins with one or more of the following
; statements:
;
; MessageId = [number|+number]
; Severity = severity_name
; Facility = facility_name
; SymbolicName = name
;
; The MessageId statement marks the beginning of the message
; definition. A MessageID statement is required for each message,
; although the value is optional. If no value is specified, the value
; used is the previous value for the facility plus one. If the value
; is specified as +number then the value used is the previous value
; for the facility, plus the number after the plus sign. Otherwise, if
; a numeric value is given, that value is used. Any MessageId value
; that does not fit in 16 bits is an error.
;
; The Severity and Facility statements are optional. These statements
; specify additional bits to OR into the final 32-bit message code. If
; not specified they default to the value last specified for a message
; definition. The initial values prior to processing the first message
; definition are:
;
; Severity=Success
; Facility=Application
;
; The value associated with Severity and Facility must match one of
; the names given in the FacilityNames and SeverityNames statements in
; the header section. The SymbolicName statement allows you to
; associate a C/C++ symbolic constant with the final 32-bit message
; code.
; */
;

;// Overrides of Win32 error messages

MessageId = 5
Severity = Error
Facility = Win32
SymbolicName = BCS_E_ACCESS_DENIED
Language = English
Access is denied
.

MessageId = 8
SymbolicName = BCS_E_NOT_ENOUGH_MEMORY
Language = English
Not enough storage is available to process this command
.

MessageId = 21
SymbolicName = BCS_E_NOT_READY
Language = English
The device is not ready
.

MessageId = 31
SymbolicName = BCS_E_GEN_FAILURE
Language = English
A device attached to the system is not functioning
.

MessageId = 71
SymbolicName = BCS_E_REQ_NOT_ACCEP
Language = English
No more connections can be made at this time
.

MessageId = 240
SymbolicName = BCS_E_VC_DISCONNECTED
Language = English
The session was canceled
.

MessageId = 258
SymbolicName = BCS_E_WAIT_TIMEOUT
Language = English
The wait operation timed out
.

MessageId = 1167
SymbolicName = BCS_E_DEVICE_NOT_CONNECTED
Language = English
The device is not connected
.

MessageId = 1223
SymbolicName = BCS_E_CANCELED
Language = English
The operation was canceled by the user
.

MessageId = 1244
SymbolicName = BCS_E_NOT_AUTHENTICATED
Language = English
The user has not been authenticated
.

;// Overrides of Winsock error messages

;// Private (base at FACILITY_ITF + 0xBC00) informational messages

MessageId=0
Severity=Success
Facility=Null
SymbolicName=BCS_MSG_READY
Language=English
Ready
.

MessageId=0x0000BC00
Severity=Success
Facility=Bcs
SymbolicName=BCS_MSG_NO_BLUETOOTH
Language=English
Bluetooth not installed
.

MessageId=
SymbolicName=BCS_MSG_NO_RADIOS
Language=English
No Bluetooth radios found
.

MessageId=
SymbolicName=BCS_MSG_SEARCHING
Language=English
Searching...
.

MessageId=
SymbolicName=BCS_MSG_DISCOVER_CANCELED
Language=English
Bluetooth device discovery canceled
.

MessageId=
SymbolicName=BCS_MSG_NO_CONNECTIONS
Language=English
No scanner connections found
.

MessageId=
SymbolicName=BCS_MSG_NO_DEVICES
Language=English
No Bluetooth devices found
.

MessageId=
SymbolicName=BCS_MSG_NO_SCANNERS
Language=English
No scanners found
.

MessageId=
SymbolicName=BCS_MSG_SCANNER_NOT_DETECTED
Language=English
Scanner not detected
.

MessageId=
SymbolicName=BCS_MSG_CONNECTING
Language=English
Connecting to scanner
.

MessageId=
SymbolicName=BCS_MSG_AUTHENTICATING
Language=English
Authenticating scanner
.

MessageId=
SymbolicName=BCS_MSG_INVALID_PASSKEY
Language=English
The passkey entered does not match the passkey of the scanner
.

MessageId=
SymbolicName=BCS_MSG_CONNECTED
Language=English
Connected
.

MessageId=
SymbolicName=BCS_MSG_ATTEMPTING_RECONNECT
Language=English
Attempting to reconnect scanner
.

MessageId=
SymbolicName=BCS_MSG_SCANNER_NOT_RESPONDING
Language=English
Scanner inactive or out of range
.

;// Private (base at FACILITY_ITF + 0xBC00) error messages
;
;#define BCS_E_BASE                       0x8000BC00L
;

MessageId=0x0000BC01
Severity=Error
Facility=Bcs
SymbolicName=BCS_E_MALFORMED_PACKET
Language=English
Flic error malformed plus packet
.

MessageId=
SymbolicName=BCS_E_READ_OVERFLOW
Language=English
Flic error read overflow
.

MessageId=
SymbolicName=BCS_E_NOT_PLUS_PACKET
Language=English
Flic error not a plus packet
.

MessageId=
SymbolicName=BCS_E_NO_PACKET
Language=English
Flic no packet recieved
.

MessageId=
SymbolicName=BCS_E_NAK
Language=English
Flic NAK received
.

MessageId=0x0000BC07
SymbolicName=BCS_E_UNEXPECTED_PACKET
Language=English
Flic error: ununexpected packet
.

MessageId=
SymbolicName=BCS_E_INVALID_OPTION
Language=English
Flic error: unknown option requested
.

MessageId=
SymbolicName=BCS_E_INVALID_HANDLE
Language=English
Flic error: serial port not open or valid
.

MessageId=
SymbolicName=BCS_E_INVALID_CRC
Language=English
Flic error: packet with invalid CRC received from HOST
.

MessageId=
SymbolicName=BCS_E_NO_BARCODE
Language=English
Flic error: no barcode available
.

MessageId=
SymbolicName=BCS_E_NOT_CONNECTED
Language=English
Flic error: Flic not connected
.

MessageId=
SymbolicName=BCS_E_BLUETOOTH_STACK_ERROR
Language=English
Error: Bluetooth stack error
.

MessageId=
SymbolicName=BCS_E_FLIC_NOT_FOUND
Language=English
Error: No Flic detected
.

MessageId=
SymbolicName=BCS_E_DUPLICATE_PACKET
Language=English
Flic error: Duplicate sequence number received
.

MessageId=
SymbolicName=BCS_E_INVALID_SEQUENCE_NUMBER
Language=English
Flic error: Sequence number out of sequence
.
