'***************************************************************************************
' 
' This application demonstrates how to use the Microvision ActiveX Control to create a
' scanner interface.
' 
' Copyright  2007 Microvision, Inc.
' All Rights Reserved.
'
'---------------------------------------------------------------------------------------
'
' Date			Author			Description
' ----------	--------------	--------------------------------------------------------
' 2007-11-18	T Cataldo		Added this comment block.
'
'***************************************************************************************


Option Strict Off
Option Explicit On


Friend Class BcsActiveX
	Inherits System.Windows.Forms.Form


	'--------------------------------------------------------------------------------
	'
	' Use this function when you want to search for Bluetooth scanners. The Bluetooth
	' scanners found will be displayed in the control. In order for a scanner to be
	' found it must not be paired to another computer and it must be in Discoverable
	' mode. Place the scanner in Discoverable mode by pressing the scan button. The
	' scanner should repeatedly double-blink the blue LED when it is Discoverable.
	'
	'--------------------------------------------------------------------------------
	Private Sub ButtonAdd_Click(ByVal sender As System.Object, ByVal e As System.EventArgs)
		On Error GoTo ErrorExit
		BcsConn1.AddBluetoothDevice()
		Exit Sub

ErrorExit:
		MsgBox(Err.Description, MsgBoxStyle.Information, "Bcs Error")
	End Sub



	'--------------------------------------------------------------------------------
	'
	' Configure the scanner by setting it to its factory default state and then
	' setting the configuration parameters required by this application. In a "real"
	' application you could also set the bar code parameters, symbology IDs, etc.
	'
	'--------------------------------------------------------------------------------
	Private Sub ButtonConfigure_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonConfigure.Click
		On Error GoTo ErrorExit
		BcsConn1.Command("RestoreFactorySettings", "")		' Set scanner to a known configuration.
		BcsConn1.SetProp("AutoDownload", True)				' Send ATTN packet upon connection.
		BcsConn1.SetProp("BcDataAck", True)					' M2M with data acknowledgements (required).
		BcsConn1.SetProp("Time", System.DateTime.Now)		' Set scanner date/time to current value.
		MessageBox.Show("Scanner Configured!", "Configure", MessageBoxButtons.OK, MessageBoxIcon.Information)
		Return

ErrorExit:
		MsgBox(Err.Description, MsgBoxStyle.Information, "Bcs Error")
	End Sub


	'--------------------------------------------------------------------------------
	'
	' This function display the properties of the scanner connection. The properties
	' are shown in a standard Windows property dialog. This can be very helpful when
	' trying to troubleshoot connection problems. This function works even if you do
	' not have an active scanner connection.
	'
	'--------------------------------------------------------------------------------
	Private Sub ButtonProperties_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonProperties.Click
		On Error GoTo ErrorExit
		BcsConn1.ShowDeviceProperties()
		Exit Sub

ErrorExit:
		MsgBox(Err.Description, MsgBoxStyle.Information, "Bcs Error")
	End Sub


	'--------------------------------------------------------------------------------
	'
	' This function removes a selected connection line from the scanner control list.
	' This is most often used to remove a Bluetooth scanner that is no longer being
	' used by this host.
	'
	'--------------------------------------------------------------------------------
	Private Sub ButtonRemove_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonRemove.Click
		On Error GoTo ErrorExit
		BcsConn1.RemoveBluetoothDevice()
		Exit Sub

ErrorExit:
		MsgBox(Err.Description, MsgBoxStyle.Information, "Bcs Error")
	End Sub


	'--------------------------------------------------------------------------------
	'
	' This function opens and closes a scanner connection and enables/disables the
	' dialog controls as appropriate.
	'
	'--------------------------------------------------------------------------------
	Private Sub ButtonStart_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonStart.Click
		On Error GoTo ErrorExit
		If ButtonStart.Text = "Activate" Then
			' Open a connection to the scanner and 0 auto-selects the baud rate.
			BcsConn1.Open(0)
			' Disable the scanner control to prevent click and other events.
			BcsConn1.Enabled = False
			' Update the dialog controls.
			ButtonStart.Text = "Deactivate"
			ButtonConfigure.Enabled = True
			CommandGetProp.Enabled = True
			CommandSetProp.Enabled = True

			' If the scanner is connected when the application starts then this command
			' causes the scanner to send its stored bar codes.
			BcsConn1.GetBarCodes()
		Else
			BcsConn1.Close()						' Close the open scanner connection.
			BcsConn1.Enabled = True					' Enable the scanner list box control.
			ButtonStart.Text = "Activate"			' Update the dialog controls.
			ButtonConfigure.Enabled = False
			CommandGetProp.Enabled = False
			CommandSetProp.Enabled = False
		End If
		Exit Sub

ErrorExit:
		MsgBox(Err.Description, MsgBoxStyle.Information, "Bcs Error")
	End Sub


	'--------------------------------------------------------------------------------
	'
	' This is the callback function that is called when the host detects data from
	' the scanner. You never call this function directly -- the system calls it in
	' response to a DataReceived event. Use this function to process the incoming
	' bar code data. For this example we write the data to the screen but you could
	' write the data to a file, add it to a database, etc. When adding this callback
	' to an existing application select the BcsConn1 object from the "Object" drop-
	' list above the IDE code window (list on the left). After selecting the BcsConn1
	' object from the left then select the "BarCode" event in the right drop-list.
	' The IDE will then create a function shell for your callback.
	'
	'--------------------------------------------------------------------------------
	Private Sub BcsConn1_BarCode( _
		ByVal sender As System.Object, _
		ByVal eventArgs As AxBcsCtlLib._IBcsConnEvents_BarCodeEvent) Handles BcsConn1.BarCode
		' For this example we display the bar code and the bar code details.
		OutputWindow.SelectedText = _
			EventArgs.barCode & vbTab & _
			"(si: " & EventArgs.symbology & _
			", ts: " & EventArgs.timeStamp & _
			", tt: " & EventArgs.timeType & ")" & vbCrLf
	End Sub


	'--------------------------------------------------------------------------------
	'
	' Use this function to read the specified property value from the scanner. In
	' the dialog box select the property from the list and press the "Get" button.
	' The program then displays the value for that scanner property. This function
	' is very useful for verifying the scanner configuration and for verifying the
	' scanner connection.
	'
	'--------------------------------------------------------------------------------
	Private Sub CommandGetProp_Click(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles CommandGetProp.Click
		On Error GoTo ErrorExit
		' Get the value of the property displayed in the combo box on the main dialog.
		edPropValue.Text = BcsConn1.GetProp(cbPropName.Text)
		Exit Sub

ErrorExit:
		edPropValue.Text = ""
		MsgBox(Err.Description, MsgBoxStyle.Information, "Bcs Error")
	End Sub


	'--------------------------------------------------------------------------------
	'
	' Set the value of the specified property. Select the property from the combo box
	' on the bottom left in the main dialog. Enter the property value in the edit
	' field on the bottom right of the dialog. When you press the "Set" button then
	' the "SetProp" command is sent to the scanner. Note that this application does
	' not validate the values that you specify. For example, if you set BcDataAck
	' then valid values are "true" and "false". If you enter an inappropriate value
	' then the scanner will reject the command.
	'
	'--------------------------------------------------------------------------------
	Private Sub CommandSetProp_Click(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles CommandSetProp.Click
		On Error GoTo ErrorExit
		' Send the command specified in the combo box and the value from the edit field.
		BcsConn1.SetProp(cbPropName.Text, edPropValue.Text)
		Exit Sub

ErrorExit:
		MsgBox(Err.Description, MsgBoxStyle.Information, "Bcs Error")
	End Sub


End Class