//***************************************************************************************
// 
// Simple C# Application Using Microvision .NET API
// 
// Copyright  2007 Microvision, Inc.
// All Rights Reserved.
//
//***************************************************************************************

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.IO;                                        // For ASCII file output.

using System.Text;                                      // For ClassscannerApi
using System.Threading;									// For .Sleep

using Microvision.BarCodeScanner;
using System.Runtime.Remoting.Messaging;


namespace BcsBarCode
{

    /// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class frmMain : System.Windows.Forms.Form
	{

		#region Class Constants
		
		/// <summary>
		/// Maximum number of characters in the scanner UserID field.
		/// </summary>
		const int LEN_DEVICE_ID = 16;

        /// <summary>
        /// Represents the title of messages to be shown to user.
        /// </summary>
        const string MSG_TITLE = "BarCode.C#";

		/// <summary>
		/// These are the possible states of the application. These values are used by the
		/// UpdateControls function to set the state of the controls on the main dialog.
		/// </summary>
		private enum DISPLAY_STATE
		{
			INITIAL,
			READY
		}


        private enum DOWNLOAD
		{
			AUTOMATIC,
			MANUAL
		}

#if false
        private enum MODE
		{
			ERROR		= -1,
			BATCH		= 0,
			BLUETOOTH	= 1,
			TETHERED	= 2
		}		// end of MODE

#endif
		#endregion


        #region Class Variables
        private Scanner m_bcsApi = new Scanner();
		DOWNLOAD        mDownload;
	//	FlicApi         mFlic = new FlicApi ();
//		MODE			m_mode;
		#endregion
	
		internal System.Windows.Forms.GroupBox grpSave;
		internal System.Windows.Forms.Button btnSave;
		internal System.Windows.Forms.TextBox editFilename;
		internal System.Windows.Forms.GroupBox grpUserId;
		internal System.Windows.Forms.Button btnUserId;
		internal System.Windows.Forms.TextBox editUserId;
		internal System.Windows.Forms.Label lblSdk;
		internal System.Windows.Forms.Label lblSdkVer;
		internal System.Windows.Forms.Label lblMessage;
		internal System.Windows.Forms.Label lblDeviceIdVal;
		internal System.Windows.Forms.Label lblDeviceId;
		internal System.Windows.Forms.Label lblFw;
		internal System.Windows.Forms.Label lblFwVer;
		internal System.Windows.Forms.Label lblHwVer;
		internal System.Windows.Forms.Label lblHw;
		internal System.Windows.Forms.Button btnFind;
		internal System.Windows.Forms.Button btnClearList;
		internal System.Windows.Forms.Button btnClear;
		internal System.Windows.Forms.Button btnGetCount;
		internal System.Windows.Forms.Button btnDownload;
        internal System.Windows.Forms.ListBox lstOutput;
		internal System.Windows.Forms.Button btnActivate;
		internal System.Windows.Forms.GroupBox grpScannerInfo;
		internal System.Windows.Forms.CheckBox CkDeviceId;
		internal System.Windows.Forms.Button btnGetInfo;
		internal System.Windows.Forms.Button btnRemove;
		internal System.Windows.Forms.GroupBox grpConnection;
		internal System.Windows.Forms.ListBox lstConnection;
		internal System.Windows.Forms.GroupBox grpDownload;
		internal System.Windows.Forms.RadioButton radAuto;
		internal System.Windows.Forms.RadioButton radManual;
		internal System.Windows.Forms.Label lblTime;
		internal System.Windows.Forms.ComboBox cboTime;
		internal System.Windows.Forms.Label lblDate;
		internal System.Windows.Forms.ComboBox cboDate;
        internal System.Windows.Forms.Button btnInitScanner;
		private System.ComponentModel.IContainer components;

        /// <summary>
        /// Main form.
        /// </summary>
        public frmMain()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			//
			// TODO: Add any constructor code after InitializeComponent call
			//
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.grpSave = new System.Windows.Forms.GroupBox();
            this.btnSave = new System.Windows.Forms.Button();
            this.editFilename = new System.Windows.Forms.TextBox();
            this.grpUserId = new System.Windows.Forms.GroupBox();
            this.btnUserId = new System.Windows.Forms.Button();
            this.editUserId = new System.Windows.Forms.TextBox();
            this.lblSdk = new System.Windows.Forms.Label();
            this.lblSdkVer = new System.Windows.Forms.Label();
            this.lblMessage = new System.Windows.Forms.Label();
            this.lblDeviceIdVal = new System.Windows.Forms.Label();
            this.lblDeviceId = new System.Windows.Forms.Label();
            this.lblFw = new System.Windows.Forms.Label();
            this.lblFwVer = new System.Windows.Forms.Label();
            this.lblHwVer = new System.Windows.Forms.Label();
            this.lblHw = new System.Windows.Forms.Label();
            this.btnFind = new System.Windows.Forms.Button();
            this.btnClearList = new System.Windows.Forms.Button();
            this.btnClear = new System.Windows.Forms.Button();
            this.btnGetCount = new System.Windows.Forms.Button();
            this.btnDownload = new System.Windows.Forms.Button();
            this.lstOutput = new System.Windows.Forms.ListBox();
            this.btnActivate = new System.Windows.Forms.Button();
            this.grpScannerInfo = new System.Windows.Forms.GroupBox();
            this.CkDeviceId = new System.Windows.Forms.CheckBox();
            this.btnGetInfo = new System.Windows.Forms.Button();
            this.btnRemove = new System.Windows.Forms.Button();
            this.grpConnection = new System.Windows.Forms.GroupBox();
            this.lstConnection = new System.Windows.Forms.ListBox();
            this.grpDownload = new System.Windows.Forms.GroupBox();
            this.radAuto = new System.Windows.Forms.RadioButton();
            this.radManual = new System.Windows.Forms.RadioButton();
            this.lblTime = new System.Windows.Forms.Label();
            this.cboTime = new System.Windows.Forms.ComboBox();
            this.lblDate = new System.Windows.Forms.Label();
            this.cboDate = new System.Windows.Forms.ComboBox();
            this.btnInitScanner = new System.Windows.Forms.Button();
            this.grpSave.SuspendLayout();
            this.grpUserId.SuspendLayout();
            this.grpScannerInfo.SuspendLayout();
            this.grpConnection.SuspendLayout();
            this.grpDownload.SuspendLayout();
            this.SuspendLayout();
            // 
            // grpSave
            // 
            this.grpSave.Controls.Add(this.btnSave);
            this.grpSave.Controls.Add(this.editFilename);
            this.grpSave.Location = new System.Drawing.Point(512, 258);
            this.grpSave.Name = "grpSave";
            this.grpSave.Size = new System.Drawing.Size(160, 150);
            this.grpSave.TabIndex = 61;
            this.grpSave.TabStop = false;
            this.grpSave.Text = "Save Data";
            // 
            // btnSave
            // 
            this.btnSave.Location = new System.Drawing.Point(28, 116);
            this.btnSave.Name = "btnSave";
            this.btnSave.Size = new System.Drawing.Size(104, 24);
            this.btnSave.TabIndex = 23;
            this.btnSave.Text = "Save to File";
            this.btnSave.Click += new System.EventHandler(this.btnSave_Click);
            // 
            // editFilename
            // 
            this.editFilename.Location = new System.Drawing.Point(8, 24);
            this.editFilename.MaxLength = 50;
            this.editFilename.Name = "editFilename";
            this.editFilename.Size = new System.Drawing.Size(144, 20);
            this.editFilename.TabIndex = 22;
            this.editFilename.Text = "editFilename";
            this.editFilename.TextChanged += new System.EventHandler(this.editFilename_TextChanged);
            // 
            // grpUserId
            // 
            this.grpUserId.Controls.Add(this.btnUserId);
            this.grpUserId.Controls.Add(this.editUserId);
            this.grpUserId.Location = new System.Drawing.Point(356, 320);
            this.grpUserId.Name = "grpUserId";
            this.grpUserId.Size = new System.Drawing.Size(135, 88);
            this.grpUserId.TabIndex = 60;
            this.grpUserId.TabStop = false;
            this.grpUserId.Text = "User ID";
            // 
            // btnUserId
            // 
            this.btnUserId.Location = new System.Drawing.Point(20, 56);
            this.btnUserId.Name = "btnUserId";
            this.btnUserId.Size = new System.Drawing.Size(94, 24);
            this.btnUserId.TabIndex = 26;
            this.btnUserId.Text = "Set";
            this.btnUserId.Click += new System.EventHandler(this.btnUserId_Click);
            // 
            // editUserId
            // 
            this.editUserId.Location = new System.Drawing.Point(6, 24);
            this.editUserId.MaxLength = 8;
            this.editUserId.Name = "editUserId";
            this.editUserId.Size = new System.Drawing.Size(123, 20);
            this.editUserId.TabIndex = 22;
            this.editUserId.Text = "editUserId";
            this.editUserId.TextChanged += new System.EventHandler(this.editUserId_TextChanged);
            // 
            // lblSdk
            // 
            this.lblSdk.Location = new System.Drawing.Point(75, 101);
            this.lblSdk.Name = "lblSdk";
            this.lblSdk.Size = new System.Drawing.Size(48, 14);
            this.lblSdk.TabIndex = 59;
            this.lblSdk.Text = "lblSdk";
            // 
            // lblSdkVer
            // 
            this.lblSdkVer.ImageAlign = System.Drawing.ContentAlignment.MiddleRight;
            this.lblSdkVer.Location = new System.Drawing.Point(8, 101);
            this.lblSdkVer.Name = "lblSdkVer";
            this.lblSdkVer.Size = new System.Drawing.Size(56, 16);
            this.lblSdkVer.TabIndex = 58;
            this.lblSdkVer.Text = "SDK Ver";
            this.lblSdkVer.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
            // 
            // lblMessage
            // 
            this.lblMessage.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
            this.lblMessage.Location = new System.Drawing.Point(20, 130);
            this.lblMessage.Name = "lblMessage";
            this.lblMessage.Size = new System.Drawing.Size(304, 24);
            this.lblMessage.TabIndex = 57;
            this.lblMessage.Text = "lblMessage";
            this.lblMessage.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
            // 
            // lblDeviceIdVal
            // 
            this.lblDeviceIdVal.Location = new System.Drawing.Point(587, 149);
            this.lblDeviceIdVal.Name = "lblDeviceIdVal";
            this.lblDeviceIdVal.Size = new System.Drawing.Size(77, 16);
            this.lblDeviceIdVal.TabIndex = 56;
            this.lblDeviceIdVal.Text = "lblDeviceIdVal";
            // 
            // lblDeviceId
            // 
            this.lblDeviceId.ImageAlign = System.Drawing.ContentAlignment.MiddleRight;
            this.lblDeviceId.Location = new System.Drawing.Point(8, 76);
            this.lblDeviceId.Name = "lblDeviceId";
            this.lblDeviceId.Size = new System.Drawing.Size(56, 16);
            this.lblDeviceId.TabIndex = 55;
            this.lblDeviceId.Text = "Device ID";
            this.lblDeviceId.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
            // 
            // lblFw
            // 
            this.lblFw.Location = new System.Drawing.Point(587, 123);
            this.lblFw.Name = "lblFw";
            this.lblFw.Size = new System.Drawing.Size(67, 16);
            this.lblFw.TabIndex = 53;
            this.lblFw.Text = "lblFw";
            // 
            // lblFwVer
            // 
            this.lblFwVer.ImageAlign = System.Drawing.ContentAlignment.MiddleRight;
            this.lblFwVer.Location = new System.Drawing.Point(6, 50);
            this.lblFwVer.Name = "lblFwVer";
            this.lblFwVer.Size = new System.Drawing.Size(56, 16);
            this.lblFwVer.TabIndex = 52;
            this.lblFwVer.Text = "F/W Ver";
            this.lblFwVer.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
            // 
            // lblHwVer
            // 
            this.lblHwVer.ImageAlign = System.Drawing.ContentAlignment.MiddleRight;
            this.lblHwVer.Location = new System.Drawing.Point(8, 23);
            this.lblHwVer.Name = "lblHwVer";
            this.lblHwVer.Size = new System.Drawing.Size(56, 16);
            this.lblHwVer.TabIndex = 51;
            this.lblHwVer.Text = "H/W Ver";
            this.lblHwVer.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
            // 
            // lblHw
            // 
            this.lblHw.Location = new System.Drawing.Point(587, 96);
            this.lblHw.Name = "lblHw";
            this.lblHw.Size = new System.Drawing.Size(77, 16);
            this.lblHw.TabIndex = 50;
            this.lblHw.Text = "lblHw";
            // 
            // btnFind
            // 
            this.btnFind.Location = new System.Drawing.Point(360, 14);
            this.btnFind.Name = "btnFind";
            this.btnFind.Size = new System.Drawing.Size(80, 24);
            this.btnFind.TabIndex = 38;
            this.btnFind.Text = "Add...";
            this.btnFind.Click += new System.EventHandler(this.btnFind_Click);
            // 
            // btnClearList
            // 
            this.btnClearList.Location = new System.Drawing.Point(352, 243);
            this.btnClearList.Name = "btnClearList";
            this.btnClearList.Size = new System.Drawing.Size(139, 31);
            this.btnClearList.TabIndex = 49;
            this.btnClearList.Text = "Clear List";
            this.btnClearList.Click += new System.EventHandler(this.btnClearList_Click);
            // 
            // btnClear
            // 
            this.btnClear.Location = new System.Drawing.Point(352, 159);
            this.btnClear.Name = "btnClear";
            this.btnClear.Size = new System.Drawing.Size(139, 31);
            this.btnClear.TabIndex = 48;
            this.btnClear.Text = "Clear Bar Codes";
            this.btnClear.Click += new System.EventHandler(this.btnClear_Click);
            // 
            // btnGetCount
            // 
            this.btnGetCount.Location = new System.Drawing.Point(352, 194);
            this.btnGetCount.Name = "btnGetCount";
            this.btnGetCount.Size = new System.Drawing.Size(139, 31);
            this.btnGetCount.TabIndex = 47;
            this.btnGetCount.Text = "Get Bar Code Count";
            this.btnGetCount.Click += new System.EventHandler(this.btnGetCount_Click);
            // 
            // btnDownload
            // 
            this.btnDownload.Location = new System.Drawing.Point(352, 125);
            this.btnDownload.Name = "btnDownload";
            this.btnDownload.Size = new System.Drawing.Size(139, 31);
            this.btnDownload.TabIndex = 46;
            this.btnDownload.Text = "Download";
            this.btnDownload.Click += new System.EventHandler(this.btnDownload_Click);
            // 
            // lstOutput
            // 
            this.lstOutput.Location = new System.Drawing.Point(20, 170);
            this.lstOutput.Name = "lstOutput";
            this.lstOutput.Size = new System.Drawing.Size(304, 238);
            this.lstOutput.TabIndex = 45;
            // 
            // btnActivate
            // 
            this.btnActivate.Location = new System.Drawing.Point(360, 74);
            this.btnActivate.Name = "btnActivate";
            this.btnActivate.Size = new System.Drawing.Size(80, 24);
            this.btnActivate.TabIndex = 40;
            this.btnActivate.Text = "Activate";
            this.btnActivate.Click += new System.EventHandler(this.btnActivate_Click);
            // 
            // grpScannerInfo
            // 
            this.grpScannerInfo.Controls.Add(this.CkDeviceId);
            this.grpScannerInfo.Controls.Add(this.btnGetInfo);
            this.grpScannerInfo.Controls.Add(this.lblSdk);
            this.grpScannerInfo.Controls.Add(this.lblSdkVer);
            this.grpScannerInfo.Controls.Add(this.lblDeviceId);
            this.grpScannerInfo.Controls.Add(this.lblFwVer);
            this.grpScannerInfo.Controls.Add(this.lblHwVer);
            this.grpScannerInfo.Location = new System.Drawing.Point(512, 73);
            this.grpScannerInfo.Name = "grpScannerInfo";
            this.grpScannerInfo.Size = new System.Drawing.Size(160, 179);
            this.grpScannerInfo.TabIndex = 54;
            this.grpScannerInfo.TabStop = false;
            this.grpScannerInfo.Text = "Scanner Info";
            // 
            // CkDeviceId
            // 
            this.CkDeviceId.Location = new System.Drawing.Point(11, 126);
            this.CkDeviceId.Name = "CkDeviceId";
            this.CkDeviceId.Size = new System.Drawing.Size(112, 16);
            this.CkDeviceId.TabIndex = 25;
            this.CkDeviceId.Text = "Device ID w/data";
            // 
            // btnGetInfo
            // 
            this.btnGetInfo.Location = new System.Drawing.Point(42, 149);
            this.btnGetInfo.Name = "btnGetInfo";
            this.btnGetInfo.Size = new System.Drawing.Size(72, 24);
            this.btnGetInfo.TabIndex = 24;
            this.btnGetInfo.Text = "Get Info";
            this.btnGetInfo.Click += new System.EventHandler(this.btnGetInfo_Click);
            // 
            // btnRemove
            // 
            this.btnRemove.Location = new System.Drawing.Point(360, 44);
            this.btnRemove.Name = "btnRemove";
            this.btnRemove.Size = new System.Drawing.Size(80, 24);
            this.btnRemove.TabIndex = 39;
            this.btnRemove.Text = "Remove";
            this.btnRemove.Click += new System.EventHandler(this.btnRemove_Click);
            // 
            // grpConnection
            // 
            this.grpConnection.Controls.Add(this.lstConnection);
            this.grpConnection.Controls.Add(this.btnActivate);
            this.grpConnection.Controls.Add(this.btnFind);
            this.grpConnection.Controls.Add(this.btnRemove);
            this.grpConnection.Location = new System.Drawing.Point(20, 6);
            this.grpConnection.Name = "grpConnection";
            this.grpConnection.Size = new System.Drawing.Size(471, 106);
            this.grpConnection.TabIndex = 42;
            this.grpConnection.TabStop = false;
            this.grpConnection.Text = "Connection";
            // 
            // lstConnection
            // 
            this.lstConnection.Location = new System.Drawing.Point(16, 22);
            this.lstConnection.Name = "lstConnection";
            this.lstConnection.Size = new System.Drawing.Size(288, 69);
            this.lstConnection.TabIndex = 37;
            this.lstConnection.SelectedIndexChanged += new System.EventHandler(this.lstConnection_SelectedIndexChanged);
            // 
            // grpDownload
            // 
            this.grpDownload.Controls.Add(this.radAuto);
            this.grpDownload.Controls.Add(this.radManual);
            this.grpDownload.Location = new System.Drawing.Point(512, 10);
            this.grpDownload.Name = "grpDownload";
            this.grpDownload.Size = new System.Drawing.Size(160, 57);
            this.grpDownload.TabIndex = 43;
            this.grpDownload.TabStop = false;
            this.grpDownload.Text = "Download";
            // 
            // radAuto
            // 
            this.radAuto.Location = new System.Drawing.Point(78, 26);
            this.radAuto.Name = "radAuto";
            this.radAuto.Size = new System.Drawing.Size(76, 16);
            this.radAuto.TabIndex = 9;
            this.radAuto.TabStop = true;
            this.radAuto.Text = "Automatic";
            this.radAuto.CheckedChanged += new System.EventHandler(this.radAuto_CheckedChanged);
            // 
            // radManual
            // 
            this.radManual.Location = new System.Drawing.Point(6, 26);
            this.radManual.Name = "radManual";
            this.radManual.Size = new System.Drawing.Size(66, 16);
            this.radManual.TabIndex = 8;
            this.radManual.Text = "Manual";
            this.radManual.CheckedChanged += new System.EventHandler(this.radManual_CheckedChanged);
            // 
            // lblTime
            // 
            this.lblTime.Location = new System.Drawing.Point(518, 344);
            this.lblTime.Name = "lblTime";
            this.lblTime.Size = new System.Drawing.Size(32, 20);
            this.lblTime.TabIndex = 65;
            this.lblTime.Text = "Time";
            this.lblTime.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
            // 
            // cboTime
            // 
            this.cboTime.Location = new System.Drawing.Point(556, 340);
            this.cboTime.Name = "cboTime";
            this.cboTime.Size = new System.Drawing.Size(108, 21);
            this.cboTime.TabIndex = 64;
            this.cboTime.Text = "cboTime";
            // 
            // lblDate
            // 
            this.lblDate.Location = new System.Drawing.Point(518, 316);
            this.lblDate.Name = "lblDate";
            this.lblDate.Size = new System.Drawing.Size(32, 17);
            this.lblDate.TabIndex = 63;
            this.lblDate.Text = "Date";
            this.lblDate.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
            // 
            // cboDate
            // 
            this.cboDate.Location = new System.Drawing.Point(556, 312);
            this.cboDate.Name = "cboDate";
            this.cboDate.Size = new System.Drawing.Size(108, 21);
            this.cboDate.TabIndex = 62;
            this.cboDate.Text = "cboDate";
            // 
            // btnInitScanner
            // 
            this.btnInitScanner.Location = new System.Drawing.Point(352, 278);
            this.btnInitScanner.Name = "btnInitScanner";
            this.btnInitScanner.Size = new System.Drawing.Size(139, 31);
            this.btnInitScanner.TabIndex = 66;
            this.btnInitScanner.Text = "Initialize Scanner";
            this.btnInitScanner.Click += new System.EventHandler(this.btnInitialize_Click);
            // 
            // frmMain
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(680, 423);
            this.Controls.Add(this.btnInitScanner);
            this.Controls.Add(this.lblTime);
            this.Controls.Add(this.cboTime);
            this.Controls.Add(this.lblDate);
            this.Controls.Add(this.cboDate);
            this.Controls.Add(this.grpUserId);
            this.Controls.Add(this.lblMessage);
            this.Controls.Add(this.lblDeviceIdVal);
            this.Controls.Add(this.lblFw);
            this.Controls.Add(this.lblHw);
            this.Controls.Add(this.btnClearList);
            this.Controls.Add(this.btnClear);
            this.Controls.Add(this.btnGetCount);
            this.Controls.Add(this.btnDownload);
            this.Controls.Add(this.lstOutput);
            this.Controls.Add(this.grpScannerInfo);
            this.Controls.Add(this.grpConnection);
            this.Controls.Add(this.grpDownload);
            this.Controls.Add(this.grpSave);
            this.Name = "frmMain";
            this.Text = "BarCode.C#";
            this.Load += new System.EventHandler(this.frmMain_Load);
            this.grpSave.ResumeLayout(false);
            this.grpSave.PerformLayout();
            this.grpUserId.ResumeLayout(false);
            this.grpUserId.PerformLayout();
            this.grpScannerInfo.ResumeLayout(false);
            this.grpConnection.ResumeLayout(false);
            this.grpDownload.ResumeLayout(false);
            this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new frmMain());
		}

		//-----------------------------------------------------------------------------------------
		/// <summary>
		/// When using auto-download the scanner could contain bar codes and the user may have connected
		/// the scanner before the application was active. This function starts a download to handle
		/// this initial case. If a download starts then the event handler generates a data event that 
		/// will trigger the application to read the bar codes.
		/// NOTE: The connection to the scanner MUST be open before calling this function.
		/// </summary>
		/// <returns>
		/// true = scanner downloaded data, false = no scanner connected or not data in the scanner.
		/// </returns>
		//-----------------------------------------------------------------------------------------
		private bool ReadBarCodeData()
		{
			// short count;
			bool dataWasDownloaded = false;

			// &&&&& Send a download request to the scanner.
			return dataWasDownloaded;
		}		// end of ReadBarCodeData


        private delegate uint DataInterruptHandler(IntPtr pvParam, Scanner.EventType eventType, IntPtr pEvent);

        //-----------------------------------------------------------------------------------------
        /// <summary>
        /// Callback function that handles scanner connection events
        /// </summary>
        /// <param name="pvParam"></param>
        /// <param name="eventType"></param>
        /// <param name="pEvent"></param>
        /// <returns></returns>
        //-----------------------------------------------------------------------------------------

        uint MyEventCallback(
            IntPtr pvParam,
            Scanner.EventType eventType,
            IntPtr pEvent)
        {
            //if (radAuto.Checked)
            //    this.BeginInvoke(new EventHandler(DataEventReceived));

            //code modified here for bug fix

            if (this.InvokeRequired)
            {
                this.BeginInvoke(new DataInterruptHandler(MyEventCallback), pvParam, eventType, pEvent);
            }
            else
            {
                if (radAuto.Checked)
                {
                    switch (eventType)
                    {
                        case Scanner.EventType.FlicBanner:
                            break;

                        case Scanner.EventType.DataReceived:
                            //BcsEventHandler_DataReceived();
                            this.BeginInvoke(new EventHandler(DataEventReceived));
                            break;

                        case Scanner.EventType.BarCodeDataReceived:
                            // Data identified as a bar code (not implemented) implemented)";
                            break;

                        case Scanner.EventType.CommandPacket:
                            // Packet that will be sent to the scanner
                            break;

                        case Scanner.EventType.ResponsePacket:
                            break;

                        case Scanner.EventType.BluetoothAttemptingToConnect:
                            // Attempting to connect with Bluetooth device
                            break;

                        case Scanner.EventType.BluetoothConnectionLost:
                            // Connection with Bluetooth device lost
                            break;

                        case Scanner.EventType.BluetoothDeviceConnected:
                            // Bluetooth connection established
                            break;

                        default:
                            break;
                    }
                }
            }
            return 0;

        }


        private void DataEventReceived(object s, EventArgs e)
        {
            lblMessage.Text = "Getting Bar Codes";
            lblMessage.Refresh();

            int remainingBarCodes;
            Scanner.BarCodeCallback barCodeCallback = new Scanner.BarCodeCallback(MyBarCodeCallback);

            try
            {
                if (radManual.Checked)
                {
                    m_bcsApi.GetBarCodes(
                        Scanner.BC_NO_PACKET_ID,
                        Scanner.BC_GET_ALL,
                        barCodeCallback,
                        out remainingBarCodes,
                        0);
                }
                //Checks if automatic download is selected then, bar codes are deleted after downloading.
                else if (radAuto.Checked)
                {
                    m_bcsApi.GetBarCodes(
                       Scanner.BC_NO_PACKET_ID,
                       Scanner.BC_GET_ALL,
                       barCodeCallback,
                       out remainingBarCodes,
                       Scanner.BC_AUTO_DELETE);
                }
                lblMessage.Text = "GetBarCodes succeeded";
            }
            catch (Exception ex)
            {
                lblMessage.Text = "GetBarCodes failed";
                MessageBox.Show(ex.Message, MSG_TITLE, MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        private Microvision.BarCodeScanner.EventCallback eventCallback;

		//-----------------------------------------------------------------------------------------
		/// <summary>
		/// This function creates a connection to the scanner. The scanner must be connected
		/// to the host or be in range with the Bluetooth radio on. Normally you enumerate a list
        /// of possible scanners and then select the desired scanner from the list. You can also
        /// connect to the first scanner found or use some other algorithm of your choosing to
        /// connect to a scanner.
        /// 
        /// After connecting you can either monitor system events (bar code events) or you can manually
        /// request a data download.
        /// </summary>
		/// <param name="connectPrompt">TRUE prompts the user to plug in the scanner, FALSE displays nothing.</param>
		/// <returns>
		/// TRUE if the scanner was found and the connection was openned.
		/// </returns>
		//-----------------------------------------------------------------------------------------
		private bool OpenConnection(bool connectPrompt)
		{
			uint baudRate;
            eventCallback = new Microvision.BarCodeScanner.EventCallback(MyEventCallback);
            Scanner.ConnectionInfo connectionInfo = (Scanner.ConnectionInfo)lstConnection.SelectedItem;

			// If we are searching for the scanner then prompt the user to connect the scanner.
			if (connectPrompt) {
                if (connectionInfo.classofDevice == Scanner.COD_SERIAL_PORT)
				    MessageBox.Show("Connect scanner", MSG_TITLE, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                else {
			        if (DialogResult.OK != MessageBox.Show("Put scanner in discoverable mode",
					        "Discoverable",
					        MessageBoxButtons.OKCancel,
					        MessageBoxIcon.Exclamation,
					        MessageBoxDefaultButton.Button1))
			        {
				        return false;
			        }
                }
            }

			// This next section opens a connection to the scanner. Note that this only opens a
			// connection and it does NOT configure the scanner. Your application should always
			// include some means of configuring the scanner. The scanner configuration is stored
			// in non-volatile memory but you must configure each new scanner once and it is also
			// possible for users to tamper with the scanner configuration. Double-click the
			// "Initialize Scanner" button on the main form to view the initialization function.
			try
			{
                baudRate = m_bcsApi.OpenConnection(
                    connectionInfo,
                    Scanner.UNKNOWN_BAUDRATE,
                    eventCallback,
                    0);

                lblMessage.Text = "Scanner connection established";
                return true;
			}
			catch (Exception ex)
			{
				lblMessage.Text = "Scanner connection failed";
                MessageBox.Show(ex.Message, MSG_TITLE, MessageBoxButtons.OK, MessageBoxIcon.Information);
			}
			return false;
		}


		//-----------------------------------------------------------------------------------------
		/// <summary>
		/// This function enables and disables the controls that are appropriate for the current state of
		/// the application. Handling the controls is critical because many of functions do not contain error
		/// checking to insure that the control was activated at an appropriate time. The application assumes
		/// that controls that should be selected are not enabled.
		/// </summary>
		/// <param name="displayState">Set controls as appropriate for this operating state.</param>
		//-----------------------------------------------------------------------------------------
		private void UpdateControls(DISPLAY_STATE displayState)
		{
            bool bConnectionSelected = lstConnection.SelectedIndex >= 0;
            bool bIsConnected = m_bcsApi.IsConnected();

            lstConnection.Enabled = !bIsConnected;

            btnFind.Enabled     = !bIsConnected;
            btnRemove.Enabled   = !bIsConnected && bConnectionSelected && ((Scanner.ConnectionInfo)lstConnection.SelectedItem).classofDevice == Scanner.COD_BLUETOOTH_DOOR;
            btnActivate.Enabled = bConnectionSelected;

            radAuto.Enabled     = true;
            radManual.Enabled   = true;

			btnSave.Enabled     = (editFilename.Text.Length > 0  &&  lstOutput.Items.Count > 0);

            btnDownload.Enabled     = bIsConnected && radManual.Checked;
            btnClear.Enabled        = bIsConnected && radManual.Checked;
            btnGetCount.Enabled     = bIsConnected && radManual.Checked;

            btnInitScanner.Enabled  = bIsConnected;

            grpScannerInfo.Enabled  = bIsConnected;
            grpDownload.Enabled     = bIsConnected;
            grpUserId.Enabled       = bIsConnected;

			switch (displayState)
			{
				case DISPLAY_STATE.INITIAL:
					lblDeviceIdVal.Text	= "";
					lblFw.Text			= "";
					lblHw.Text			= "";
                    lblSdk.Text         = "";
					break;

				case DISPLAY_STATE.READY:
					btnUserId.Enabled	= (editUserId.Text.Length > 0);
					break;
			}

            btnActivate.Text = bIsConnected ? "Deactivate" : "Activate";
		}


		//-----------------------------------------------------------------------------------------
		/// <summary>
		/// Format the bar code, date stamp, and time stamp (if selected). Use the settings in the
		/// combo boxes for the date/time format.
		/// </summary>
		/// <param name="aBarcode">Bar code data read from the scanner.</param>
        /// <param name="timeString">Represents timestamp for the bar code data
        /// read from the scanner.</param>
		/// <returns>
		/// The formatted string with bar code, date, and time.
		/// </returns>
		//-----------------------------------------------------------------------------------------
		private StringBuilder FormatOutputLine(string aBarcode, string timeString)
		{
            StringBuilder formatted = new StringBuilder();
            formatted.Append(aBarcode);

            if(!string.IsNullOrEmpty(timeString)) 
            {
                DateTime timeStamp = DateTime.Parse(timeString);                  	
			       				
			    switch (cboDate.Text)
			    {
				    case "mm/dd/yy":
					    formatted.Append(',');
					    formatted.Append(timeStamp.ToString("MM/dd/yy"));
					    break;
				    case "mm/dd/yyyy":
					    formatted.Append(',');
					    formatted.Append(timeStamp.ToString("MM/dd/yyyy"));
					    break;
				    case "dd/mm/yy":
					    formatted.Append(',');
					    formatted.Append(timeStamp.ToString("dd/MM/yy"));
					    break;
				    case "dd/mm/yyyy":
					    formatted.Append(',');
					    formatted.Append(timeStamp.ToString("dd/MM/yyyy"));
					    break;
			    }		// end of switch
    				
			    switch (cboTime.Text)
			    {
				    case "hh:mm":
					    formatted.Append(',');
					    formatted.Append(timeStamp.ToString("hh:mm"));
					    break;
				    case "hh:mm:ss":
					    formatted.Append(',');
					    formatted.Append(timeStamp.ToString("hh:mm:ss"));
					    break;
			    }		// end of switch
            }
				
			return formatted;
		}		// end of FormatOutputLine


        //-----------------------------------------------------------------------------------------
        /// <summary>
        /// This function ...
        /// </summary>
        //-----------------------------------------------------------------------------------------
        private void RefreshConnectionList(DISPLAY_STATE displayState, bool discoverDevicesDialog)
        {
            try
            {
                uint enumFlags = Scanner.ENUM_SERIAL | Scanner.ENUM_BLUETOOTH | Scanner.ENUM_APPEND_BDA | Scanner.ENUM_FRIENDLYNAME;

                if (discoverDevicesDialog)
                    enumFlags |= Scanner.ENUM_SELECT;   // Select (Discover) Bluetooth devices dialog

                lstConnection.Items.Clear();            // The list box will be re-populated by MyConnectionCallback

                Scanner.ConnectionCallback connectionCallback = new Scanner.ConnectionCallback(MyConnectionCallback);

                // Because BcsEnum is intelligent about potential connections, there usually will be very few
                // scanners to choose from. All possible scanner connections should be enumerated after
                // returning from the BcsEnumConnections function. The callback should have been called for
                // each possible connection.

                m_bcsApi.EnumConnections(
                    this.Handle,						// (IN) Handle to parent window.
                    connectionCallback,					// (IN) callback called for each connection.
                    enumFlags);		                    // (IN) OR'ed ENUM bits.

                if (lstConnection.Items.Count > 0)      // One or more connections
                {
                    lstConnection.SetSelected(0, true);
                    lblMessage.Text = "Select connection and press \"Activate\"";
                }
                else
                {
                    lblMessage.Text = "No connections found";
                }
            }
            catch (Exception ex)
            {
                lblMessage.Text = ex.Message;
            }

            UpdateControls(displayState);
        }


		//-----------------------------------------------------------------------------------------
		/// <summary>
		/// This function removes a Bluetooth connection from the connection list.
		/// </summary>
		//-----------------------------------------------------------------------------------------
		private void btnRemove_Click(object sender, System.EventArgs e)
		{
            try
            {
                m_bcsApi.RemoveConnection((Scanner.ConnectionInfo)lstConnection.SelectedItem);
                RefreshConnectionList(DISPLAY_STATE.READY, false);
            }
            catch (Exception ex)
            {
                lblMessage.Text = ex.Message;
            }
		}


		//-----------------------------------------------------------------------------------------
		/// <summary>
		/// Initialize all the object on the main dialog. This function is called prior to painting
		/// the dialog.
		/// </summary>
		//-----------------------------------------------------------------------------------------
		private void frmMain_Load(object sender, System.EventArgs e)
		{
//			lblSdk.Text = "v" + mFlic.ApiVersion();		//&&&&& fix this later.

			cboDate.Items.Clear ();
			cboDate.Items.Add ("mm/dd/yy");
			cboDate.Items.Add ("mm/dd/yyyy");
			cboDate.Items.Add ("dd/mm/yy");
			cboDate.Items.Add ("dd/mm/yyyy");
			cboDate.Items.Add ("--none--");
			cboDate.SelectedIndex = 0;

			cboTime.Items.Clear();
			cboTime.Items.Add ("hh:mm:ss");
			cboTime.Items.Add ("hh:mm");
			cboTime.Items.Add ("--none--");
			cboTime.SelectedIndex = 0;

			editFilename.Clear();						// Save data to this file.
			editUserId.MaxLength= LEN_DEVICE_ID;	    // Limit user input to max User ID field length.
			editUserId.Clear();
			mDownload = DOWNLOAD.MANUAL;                // Save in a file later.

			radAuto.Checked     = (mDownload == DOWNLOAD.AUTOMATIC);
			radManual.Checked   = (mDownload == DOWNLOAD.MANUAL);

            lblSdk.Text = "";

            // Initialize the connection list
            RefreshConnectionList(DISPLAY_STATE.INITIAL, false);
        }


		//-----------------------------------------------------------------------------------------
		/// <summary>
		/// This function opens the connection to the scanner using the parameters specified by
		/// the other controls on the dialog.
        /// 
        /// This function opens a scanner connection and does not require the scanner to be
        /// connected to the host.
        /// 
        /// There was an existing connection and we openned it. Be sure to note that we do not know
        /// what mode the scanner is in. If the scanner was configured incorrectly (or not configured
        /// at all) then subsequent commands may not work as intended. Therefore, the final application
        /// should have some means for configuring a new scanner to the proper settings.
        /// </summary>
		//-----------------------------------------------------------------------------------------
		private void btnActivate_Click(object sender, System.EventArgs e)
		{
            if (m_bcsApi.IsConnected())     // Deactivate
            {
                try
                {
                    m_bcsApi.CloseConnection();
                    lblMessage.Text = "Scanner connection closed";
                    UpdateControls(DISPLAY_STATE.INITIAL);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, MSG_TITLE, MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
            }
            else
            {
                lblMessage.Text = "Activating connection...";

                OpenConnection(true);       // Connect to the port

                if (m_bcsApi.IsConnected())
                {
                    UpdateControls(DISPLAY_STATE.READY);
                    // The scanner was found and the connection was created. Now initialize the scanner. It is 
                    // assumed that the scanner does not contain any data at this point. If the scanner does contain
                    // data then the time values will not be correct since the initial time is reset.
                    // lblMessage.Text = "Setting download mode";

                    if (radAuto.Checked)
                    {
                        // If we are in auto-download mode then force a download in case the scanner contains
                        // data and was connected prior to the application being active. We don't have to
                        // explicitly read the data because the timer will handle that for us.
                        ReadBarCodeData();
                    }
                }
                else
                {
                    lblMessage.Text = "Connection did not open!";
                }
            }
		}


        //-----------------------------------------------------------------------------------------
        /// <summary>
        /// 
        /// </summary>
        /// <param name="pvParam"></param>
        /// <param name="symbology"></param>
        /// <param name="timeStamp"></param>
        /// <param name="timeType"></param>
        /// <param name="barCode"></param>
        /// <param name="cchBarCode"></param>
        /// <param name="downloadInfo"></param>
        /// <returns></returns>
        //-----------------------------------------------------------------------------------------
        uint MyBarCodeCallback(
			IntPtr	pvParam,				            // Parameter passed into BcsGetBarCodesA as the pvParam
			string	symbology,					        // Symbol Identifier (This value is dependent on the setting of the SymIdType property)
			string	timeStamp,					        // Timestamp (This pointer value is only valid if Timestamp = True, otherwize the the pointer value is NULL)
			string	timeType,					        // Time Type (This pointer value is only valid if Timestamp = True, otherwize the the pointer value is NULL)
			                                            // Possible Values: "ok" = Actual Time, or e1, e2, ..., e<N> = Estimated time after N battery events)
			string	barCode,					        // Bar code data
			int		cchBarCode,					        // Length of the bar code data in characters
			//ref BCS_DOWNLOAD_INFO pDownloadInfo);	    // Supplemental information in download packet
            int     downloadInfo)
        {
            lstOutput.Items.Add(FormatOutputLine(barCode, timeStamp).ToString());
            return 0;
        }

		//-----------------------------------------------------------------------------------------

		private void btnDownload_Click (object sender, System.EventArgs e)
		{
            int	remainingBarCodes;
            Scanner.BarCodeCallback barCodeCallback = new Scanner.BarCodeCallback(MyBarCodeCallback);

			try
			{
                if (radManual.Checked)
                {
                    m_bcsApi.GetBarCodes(
                        Scanner.BC_NO_PACKET_ID,
                        Scanner.BC_GET_ALL,
                        barCodeCallback,
                        out remainingBarCodes,
                        0);
                }
                else if (radAuto.Checked)
                {
                    m_bcsApi.GetBarCodes(
                       Scanner.BC_NO_PACKET_ID,
                       Scanner.BC_GET_ALL,
                       barCodeCallback,
                       out remainingBarCodes,
                       Scanner.BC_AUTO_DELETE);
                }
                lblMessage.Text = "GetBarCodes succeeded";
            }
            catch (Exception ex)
            {
                lblMessage.Text = "GetBarCodes failed";
                MessageBox.Show(ex.Message, MSG_TITLE, MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }


		//-------------------------------------------------------------------------------------------------

		private void btnClearList_Click (object sender, System.EventArgs e)
		{
			lstOutput.Items.Clear();
			btnSave.Enabled = false;
			lblMessage.Text = "List cleared";
		}


		//-------------------------------------------------------------------------------------------------
		/// <summary>
		/// This function sends a command instructing the scanner to clear any bar codes buffered in the
		/// memory of the scanner. This command is normally not used in a live application since most
		/// applications read the data in blocks and delete each block after it is received and verified.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		//-------------------------------------------------------------------------------------------------
		private void btnClear_Click(object sender, System.EventArgs e)
		{
			try
			{			
				m_bcsApi.Command(
					"ClearBarCodes",					// XSP command.
					null,								// Command argument -- no arg for Clear command.
					0);									// API flag field.
				// After clearing the bar codes it is a very good idea to synchronize the initial time
				// value in the scanner with the host computer's clock.
				m_bcsApi.SetProp("Time", System.DateTime.Now);
				lblMessage.Text = "Bar codes cleared!";
			}
			catch (Exception ex)
			{
				lblMessage.Text = "Error clearing bar codes!";
                MessageBox.Show(ex.Message, MSG_TITLE, MessageBoxButtons.OK, MessageBoxIcon.Information);
			}
		}


		//-----------------------------------------------------------------------------------------
		/// <summary>
		/// This function reads the information strings from the scanner and displays them. The SDK sends an
		/// inquiry request to the scanner for each call. Therefore, each call is checked to ensure that the
		/// scanner is connected and communicating properly.
		/// </summary>
		//-----------------------------------------------------------------------------------------
		private void btnGetInfo_Click(object sender, System.EventArgs e)
		{
			Cursor.Current = Cursors.WaitCursor;
			// Set the fields to an initial value and then overwrite these as we read them from
			// the scanner. If we get these values in the application then that indicates that the
			// call has an error.
			lblDeviceIdVal.Text= "???";
			lblFw.Text		= "???";
			lblHw.Text		= "???";
            lblSdk.Text = "???";
			editUserId.Clear();
			
			try
			{
				// Read the device ID string from the scanner and store it in a field on the form. If
				// the returned string is empty then there was an error communicating with the scanner.
                lblDeviceIdVal.Text = m_bcsApi.GetProp("SystemSN").ToString();
                lblFw.Text          = m_bcsApi.GetProp("SwVersion").ToString();
                lblHw.Text          = m_bcsApi.GetProp("HwVersion").ToString();
                lblSdk.Text         = m_bcsApi.BCS_SDK_VERSION;
                editUserId.Text     = m_bcsApi.GetProp("UserId").ToString();

			}
			catch (Exception ex)
			{
                MessageBox.Show(ex.Message, MSG_TITLE, MessageBoxButtons.OK, MessageBoxIcon.Information);
			}
			Cursor.Current = Cursors.Default;
		}


		//-------------------------------------------------------------------------------------------------

		private void btnGetCount_Click (object sender, System.EventArgs e)
		{
			try
			{		    
				int count = m_bcsApi.GetBarCodeCount();
				lblMessage.Text = count + " Barcodes in the scanner";
			}
			catch (Exception ex)
			{
                lblMessage.Text = "Get barcode count failed";
                MessageBox.Show(ex.Message, MSG_TITLE, MessageBoxButtons.OK, MessageBoxIcon.Information);
			}
		}


		//-----------------------------------------------------------------------------------------
		/// <summary>
		/// This function saves the downloaded scanner data in the output list box to the specified file. The
		/// SAVE button should not be enabled unless there is data in the filename edit box. After writing
		/// the data to the file clear the output list box.
		/// </summary>
		//-----------------------------------------------------------------------------------------
		private void btnSave_Click(object sender, System.EventArgs e)
		{
			try
			{
                if (lstOutput.Items.Count > 0)
                {
                    StreamWriter outFile = new StreamWriter(editFilename.Text, true, Encoding.ASCII);

                    for (int i = 0; i < lstOutput.Items.Count; i++)
                    {
                        lstOutput.SelectedIndex = i;
                        outFile.WriteLine(lstOutput.SelectedItem);
                    }

                    outFile.WriteLine();
                    outFile.WriteLine();

                    outFile.Close();
                    lblMessage.Text = "List data saved to file";
                    lstOutput.Items.Clear();
                }
			}
			catch (Exception ex)
			{
				MessageBox.Show("File Error: " + ex.ToString(), MSG_TITLE, MessageBoxButtons.OK, MessageBoxIcon.Information);
			}
		}


		//-----------------------------------------------------------------------------------------
		/// <summary>
		/// 
		/// </summary>
		//-----------------------------------------------------------------------------------------
		private void editFilename_TextChanged(object sender, System.EventArgs e)
		{
			btnSave.Enabled = (editFilename.Text.Length > 0 && lstOutput.Items.Count > 0);
		}

		///----------------------------------------------------------------------------------------
		///
		/// <summary>
		/// This is the enumeration callback function. When the Add function is called then this
		/// function is called for each port (serial port or USB VCOMM port). It only recognizes
		/// VCOMM ports when the USB cable is connected. The COM ports detected may or may not
		/// have a scanner attached. This callback populates the connection list box and a member
		/// connection array. Activate uses the connection structure stored in the connection list
		/// to open a connection to a specific scanner.
		/// </summary>
		/// <param name="param">[in/out] Generic parameter used to communicate with the function.
        /// This is not used and a member array is used to save the connection information.</param>
		/// <param name="connectionInfo">[in] Is loaded with connection details when this
		/// function is called.</param>
		/// <returns>
		/// A value >= 0 if enumeration is to continue; less than 0 if enumeration should stop.
		/// </returns>
		///
		///----------------------------------------------------------------------------------------
		int MyConnectionCallback(
			IntPtr					    param,
            ref Scanner.ConnectionInfo  connectionInfo)
		{
            lstConnection.Items.Add( connectionInfo );
			return 0;
		}


		///----------------------------------------------------------------------------------------
		/// <summary>
		/// This function finds all the possible scanners and stores the details for each one in
		/// the lstConnection list box. You can then display a list of all possible scanners,
		/// find a desired scanner, etc. Note that the enumeration functions enumerates connections
		/// that appear to have a scanner. It enumerates connections to any previously paired
		/// scanners and serial ports that have a USB scanner cable connected to them. If you want
		/// to insure that a scanner is connected then you must open a connection on that port
		/// using the BCS_OPEN_DETECT_SCANNER flag.
		/// </summary>
		///----------------------------------------------------------------------------------------
		private void btnFind_Click(object sender, System.EventArgs e)
		{
			lblMessage.Text = "Searching for Bluetooth scanners";
            RefreshConnectionList(DISPLAY_STATE.READY, true);
		}


		//-----------------------------------------------------------------------------------------
		/// <summary>
		/// The User ID button should only be enabled if there is data in the User ID field. When
		/// the user presses this button then write the User ID text to the scanner.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		//-----------------------------------------------------------------------------------------
		private void btnUserId_Click(object sender, System.EventArgs e)
		{
			try
			{
				m_bcsApi.SetProp("UserId", editUserId.Text);
				lblMessage.Text = "User ID set";
			}
			catch (Exception ex)
			{
				lblMessage.Text = "User ID not set";
                MessageBox.Show(ex.Message, MSG_TITLE, MessageBoxButtons.OK, MessageBoxIcon.Information);
			}
		}


		//-----------------------------------------------------------------------------------------
		/// <summary>
		/// If the user enters data in the User ID field then enable the SET button. If the field
		/// is cleared then disable the button.
		/// </summary>
		//-----------------------------------------------------------------------------------------
		private void editUserId_TextChanged(object sender, System.EventArgs e)
		{
			btnUserId.Enabled = (editUserId.Text.Length > 0);
		}


		//-----------------------------------------------------------------------------------------
		/// <summary>
		/// Your application should always have a way to initialize a scanner. If the user receives a new
		/// scanner from the factory it must be configured once. This function calls scannerConnectionConfigure
		/// to open the port and force a scanner initialization. The scanner MUST be connected in order for
		/// this to work.
		/// </summary>
		//-----------------------------------------------------------------------------------------
		private void btnInitialize_Click(object sender, System.EventArgs e)
		{
			try
			{
				// The initialize button should not be enabled unless there is a valid connection
				// but check the connection anyway (the error box below should never be called).
				if (!m_bcsApi.IsConnected())
				{
					MessageBox.Show ("Must open a connection to the scanner!", MSG_TITLE, 
                                            MessageBoxButtons.OK, MessageBoxIcon.Warning);
					return;
				}

				// Your application should have a way to configure the scanner and your
				// configuration should always start by setting the scanner to the factory
				// defaults. After the scanner is in the factory state then set the scanner
				// properties required for your application.
				m_bcsApi.Command("RestoreFactorySettings", null, 0);
                m_bcsApi.SetProp("BCDataAck", true);
                m_bcsApi.SetProp("Autodownload", radAuto.Checked);

				lblMessage.Text = "Scanner configured";
			}
			catch (Exception ex)
			{
				lblMessage.Text = "Error configuring scanner!";
                MessageBox.Show(ex.Message, MSG_TITLE, MessageBoxButtons.OK, MessageBoxIcon.Information);
			}
		}

        //-----------------------------------------------------------------------------------------
        /// <summary>
        /// Reset the display state whenever the connection list selection changes
        /// </summary>
        //-----------------------------------------------------------------------------------------
        private void lstConnection_SelectedIndexChanged(object sender, EventArgs e)
        {
            UpdateControls(DISPLAY_STATE.READY);
        }

        private void radManual_CheckedChanged(object sender, EventArgs e)
        {
            UpdateControls(DISPLAY_STATE.READY);
        }

        private void radAuto_CheckedChanged(object sender, EventArgs e)
        {
            UpdateControls(DISPLAY_STATE.READY);
        }


	//-----------------------------------------------------------------------------------------

	}       // end of class frmMain
}
