// Scanner Class Library - SCL version 0.01
// Copyright (C) Microvision, Inc. All rights reserved.

#ifndef __SCLCONN_H__
#define __SCLCONN_H__

#pragma once

#ifndef __cplusplus
  #error FTL requires C++ compilation (use a .cpp suffix)
#endif

#include "..\..\..\BcsSdk\Bcs.h"				// Checks whether _LIB is defined

#if 0
#ifdef _LIB									// Link with the static SDK library
#ifdef _DEBUG								// Link with the debug static SDK library
#pragma comment(lib, "..\\..\\..\\..\\BCSSD.lib")
#else
#pragma comment(lib, "..\\..\\..\\..\\BCSS.lib")
#endif
#else										// Link with the SDK DLL
#ifdef _DEBUG								// Link with the debug SDK DLL
#pragma comment(lib, "..\\..\\..\\..\\BCSD.lib")
#else										// Link with the SDK DLL
#pragma comment(lib, "..\\..\\..\\..\\BCS.lib")
#endif
#endif
#endif

#ifndef _WIN32_WCE

#include <ws2bth.h>							// SDP_TYPE, ...
#include <BluetoothAPIs.h>					// BLUETOOTH_DEVICE_INFO

#endif // _WIN32_WCE

//#include "sclkeys.h"

///////////////////////////////////////////////////////////////////////////////
// Classes in this file:
//
// CConnectionInfo
// CConnection

/// Wraps the BCS_CONNECTION_INFO structure.

class CConnectionInfo : public BCS_CONNECTION_INFO
{
	// Public interface
public:
	CConnectionInfo() {
//		memset( this, 0, sizeof(BCS_CONNECTION_INFO) );

		dwSize = sizeof(BCS_CONNECTION_INFO);		//  size, in bytes, of this structure - must be the sizeof(BLUETOOTH_DEVICE_INFO)
#ifndef _WIN32_WCE
		ATLASSERT( dwSize == sizeof(BLUETOOTH_DEVICE_INFO) );
#endif
		memset( &Address, 0, sizeof(Address) );

		ulClassofDevice = 0;						//  Bluetooth "Class of Device" or BCS_COD_SERIAL_PORT

		fConnected = false;                         //  Device connected/in use
		fRemembered = false;                        //  Device remembered
		fAuthenticated = false;                     //  Device authenticated/paired/bonded

		memset( &stLastSeen, 0, sizeof(stLastSeen) );
		memset( &stLastUsed, 0, sizeof(stLastUsed) );

		szName[0] = '\0';			//  Name of the device
		szPortName[0]	= '\0';		//  Port name
		szInstanceId[0] = '\0';		//  Instance ID of the device
	}

	bool IsSerialPort()	{ return ulClassofDevice == BCS_COD_SERIAL_PORT; }

#ifdef _WIN32_WCE
#ifndef _UNICODE
#define _UNICODE
#endif
#endif

#ifdef _UNICODE
	PCTSTR  Name()						{ return szName;										}
	PCTSTR PortName()					{ return IsSerialPort() ? szPortName : L"Bluetooth";	}
#else
	PCTSTR  Name()
	{
		static char szNameA[ BLUETOOTH_MAX_NAME_SIZE ];
		LRESULT lRes = WideCharToMultiByte( CP_ACP, 0, szName, -1, szNameA, sizeof(szNameA), NULL, NULL );
		ATLASSERT( SUCCEEDED(lRes) );
		return szNameA;
	}
	PCTSTR PortName()
	{
		if (!IsSerialPort())
			return "Bluetooth";

		static char szPortNameA[ BCS_MAX_PORTNAME_SIZE ];
		LRESULT lRes = WideCharToMultiByte( CP_ACP, 0, szPortName, -1, szPortNameA, sizeof(szPortNameA), NULL, NULL );
		ATLASSERT( SUCCEEDED(lRes) );
		return szPortNameA;
	}
#endif
};

/// Encapsulates a connection to a bar code scanner.

class CConnection
{
public:
	// Private member variables
	CConnectionInfo		m_ConnectionInfo;
	HWND				m_hWndEvents;
	HKEY				m_hkRegistryHive;
	HANDLE				m_hEvent;
	HBCS				m_hBcs;
	DWORD				m_dwBaudRate;

	// Public interface
public:
	CConnection( HWND hWndEvents, HKEY hkRegistryHive = HKEY_LOCAL_MACHINE ) :
		m_hWndEvents(		hWndEvents					),
		m_hkRegistryHive(	hkRegistryHive				),
		m_hEvent(			NULL						),
		m_hBcs(				(HBCS)INVALID_HANDLE_VALUE	),
		m_dwBaudRate(		BCS_UNKNOWN_BAUDRATE		)
	{
	}
	~CConnection() {
		if (IsConnected())					// Disconnect before deleting
			Disconnect();
	}

//#ifdef _WIN32_WCE

	HRESULT DisplayProperties( HWND hWnd )	{ return BcsDisplayConnectionProperties( hWnd, &m_ConnectionInfo, 0 ); }

	bool IsSerialPort()		{ return m_ConnectionInfo.IsSerialPort(); }
	UINT MaxConnectTime()	{ return (IsSerialPort() && wcsncmp(m_ConnectionInfo.szInstanceId, L"HID", 3)) ? 500 : 10; }

	HRESULT	Connect( PFN_BCS_EVENT_CALLBACK pfnEventCallback )
	{
		// Ideally, we would remember the baud rate for connection and pass it in as a hint to speed up connection

		HRESULT hr = BcsOpenConnection(
			&m_hBcs,						// OUT A connection handle for this connection
			&m_ConnectionInfo,				// IN  A connection information structure (usually retrieved via BcsEnumConnections)
			m_dwBaudRate,					// IN  A specific baud rate (4800|9600|19200|38400|57600|115200) or BCS_UNKNOWN_BAUDRATE
			&m_dwBaudRate,					// OUT Baud rate of the connection.  Can be NULL.
			pfnEventCallback,				// IN  A callback that will be called for each event.
			this,							// IN  Parameter to be passed to pfnEventCallback as the pvParam
#ifdef _WIN32_WCE
			0		// IN  Bitwise or'd combination of BCS_OPEN bits or 0
#else
			BCS_OPEN_DETECT_BAUDRATE | BCS_OPEN_GET_BANNER | BCS_OPEN_PACKET_EVENTS		// IN  Bitwise or'd combination of BCS_OPEN bits or 0
#endif
		);

#if SIGNAL_CONNECT_DISCONNECT
		BcsSignal(
			m_hBcs,							// IN  The connection handle provided by BcsOpenConnection
			3,								// IN  Number of parameter sets (1 through 8)
			1000, 222, 0, 0,				// IN  Frequency in kHz, Duration in 0.5 ms intervals, Led0 is On (1) or Off (0), Led1
			1500, 222, 0, 0,				// IN  Frequency in kHz, Duration in 0.5 ms intervals, Led0 is On (1) or Off (0), Led1
			2000, 222, 0, 0					// IN  Frequency in kHz, Duration in 0.5 ms intervals, Led0 is On (1) or Off (0), Led1
		);
#endif

		return hr;
	}

	HBCS	GetHandle()		{ return m_hBcs; }
	bool	IsConnected()	{ return m_hBcs != INVALID_HANDLE_VALUE; }
	DWORD	GetBaudRate()
	{
		if (IsConnected())					// 2400 is the estimated Bluetooth baud rate
			return IsSerialPort() ? m_dwBaudRate : 2400;

		return BCS_UNKNOWN_BAUDRATE;		// Really means not applicable in this case
	}

	HRESULT Disconnect()
	{
		HRESULT hr = S_OK;

		if (IsConnected()) {

#if SIGNAL_CONNECT_DISCONNECT
			BcsSignal(
				m_hBcs,						// IN  The connection handle provided by BcsOpenConnection
				3,							// IN  Number of parameter sets (1 through 8)
				2000, 222, 0, 0,			// IN  Frequency in kHz, Duration in 0.5 ms intervals, Led0 is On (1) or Off (0), Led1
				1500, 222, 0, 0,			// IN  Frequency in kHz, Duration in 0.5 ms intervals, Led0 is On (1) or Off (0), Led1
				1000, 222, 0, 0				// IN  Frequency in kHz, Duration in 0.5 ms intervals, Led0 is On (1) or Off (0), Led1
			);
#endif

			hr = BcsCloseConnection( m_hBcs );
			ATLASSERT( SUCCEEDED(hr) );
		}
		m_hBcs = (HBCS)INVALID_HANDLE_VALUE;

		return hr;
	}

	HRESULT GetBarCodeCount( int *pnCount ) { return BcsGetBarCodeCount( m_hBcs, pnCount ); }

	HRESULT SetProp(
			const TCHAR *pszPropName,		// IN  TCHAR version of property name
			VARIANT *pVarValue,				// IN  New propery value
			DWORD dwFlags = 0				// IN  Currently accepts 0, BCS_XSP_XML or BCS_XSP_BSTR
		)
	{
		return BcsSetProp(					// Get a Scanner Configuration Property
			m_hBcs,							// IN  The connection handle provided by BcsOpenConnection
			pszPropName,					// IN  TCHAR version of property name
			LOCALE_SYSTEM_DEFAULT,			// IN  Locale used to interpret pVarValue
			pVarValue,						// IN  New propery value
			dwFlags							// IN  Currently accepts 0, BCS_XSP_XML or BCS_XSP_BSTR
			);
	}

	HRESULT GetProp(
			const TCHAR *pszPropName,		// IN  TCHAR version of property name
			VARIANT *pVarResult,			// OUT Returned propery value
			DWORD dwFlags = 0				// IN  Currently accepts 0, BCS_XSP_XML or BCS_XSP_BSTR
		)
	{
		return BcsGetProp(					// Get a Scanner Configuration Property
			m_hBcs,							// IN  The connection handle provided by BcsOpenConnection
			pszPropName,					// IN  TCHAR version of property name
			LOCALE_SYSTEM_DEFAULT,			// IN  Locale used to format pVarResult
			pVarResult,						// OUT Returned propery value
			dwFlags							// IN  Currently accepts 0, BCS_XSP_XML or BCS_XSP_BSTR
			);
	}
};

#endif // __SCLCONN_H__
