// Scanner Class Library - SCL version 0.01
// Copyright (C) Microvision, Inc. All rights reserved.

#ifndef __SCLCTRLS_H__
#define __SCLCTRLS_H__

#pragma once

#ifndef __cplusplus
	#error FTL requires C++ compilation (use a .cpp suffix)
#endif

#ifndef __ATLCTRLS_H__
	// #error sclctrls.h requires atlctrls.h to be included first
	#include <atlctrls.h>					// CListBox, etc.
#endif

#ifndef __ATLCRACK_H__
	// #error sclctrls.h requires atlcrack.h to be included first
	#include <atlcrack.h>					// BEGIN_MSG_MAP_EX, MSG_WM_CREATE, MESSAGE_RANGE_HANDLER_EX, etc.
#endif

#ifndef __ATLMISC_H__
	// #error sclctrls.h requires atlmisc.h to be included first
	#include <atlmisc.h>					// CPoint, etc.
#endif

#ifndef __ATLFRAME_H__
	// #error sclctrls.h requires atlframe.h to be included first
	#include <atlframe.h>					// COwnerDraw
#endif // __ATLFRAME_H__

#ifndef _WIN32_WCE
#ifndef __ATLTHEME_H__
	// #error sclctrls.h requires atltheme.h to be included first
	#include <atltheme.h>					// CTheme
#endif
#endif

#ifndef __SCLCONN_H__
	// #error sclctrls.h requires sclconn.h to be included first
	#include "sclconn.h"					// Scanner SDK Connection class
#endif

//#ifndef _WIN32_WCE
#ifndef __SCLKEYS_H__
	// #error sclctrls.h requires sclkeys.h to be included first
	#include "sclkeys.h"					// Scanner SDK Keystrokes class
#endif
//#endif

#ifndef _DBT_H
	// #error sclctrls.h requires dbt.h to be included first
	#include <dbt.h>						// DBT_*
#endif

#ifndef _STRSAFE_H_INCLUDED_
	// #error sclctrls.h requires strsafe.h to be included first
	#include <strsafe.h>					// StringCchPrintf, etc.
#endif

// {95c7a0a0-3094-11d7-a202-00508b9d7d5a}
static const GUID GUID_DEVCLASS_BTW				= { 0x95c7a0a0L, 0x3094, 0x11d7, { 0xa2, 0x02, 0x00, 0x50, 0x8b, 0x9d, 0x7d, 0x5a } };

#ifndef _WIN32_WCE
#include <devguid.h>						// GUID_DEVCLASS_BLUETOOTH
#endif

// {4D1E55B2-F16F-11CF-88CB-00-1111000030}
static const GUID GUID_DEVINTERFACE_HID			= { 0x4D1E55B2L, 0xF16F, 0x11CF, { 0x88, 0xCB, 0x00, 0x11, 0x11, 0x00, 0x00, 0x30} };

#ifndef _WIN32_WCE
#include <setupapi.h>						// SetupDiGetClassDevs, ...
#pragma comment(lib, "setupapi.lib")
#endif

enum										// Connection control messages
{
	SCM_MESSAGEIDCHANGE = 0xBC0,			// 0x7BC0
	SCM_CONNECTIONSTATECHANGE,				// 0x7BC1
	SCM_BARCODE								// 0x7BC2
};

#define SHOW_LOOPBACK_PORTS					FALSE
#define BCS_ENUM_DISPLAY_DEFAULTS			BCS_ENUM_SERIAL | BCS_ENUM_BLUETOOTH | BCS_ENUM_APPEND_BDA | BCS_ENUM_FRIENDLYNAME


///////////////////////////////////////////////////////////////////////////////
// Classes in this file:
//
// CConnectionListImpl<T, TBase, TWinTraits>
// CConnectionListCtrl
// CConnectionComboCtrl
//
// CKeystrokes
//
// CTrimImpl<T, TBase, TWinTraits>
// CTrimCtrl
//
// CKeyEditImpl<T, TBase, TWinTraits>
// CKeyEditCtrl
//
// CKeyImpl<T, TBase, TWinTraits>
// CKeyCtrl

//static const TCHAR *pcszBluetooth	= _T("Bluetooth");

///////////////////////////////////////////////////////////////////////////////
// CConnectionListCtrl/CConnectionComboCtrl - connection list implementation

#ifdef USE_LISTVIEW
template< class T, class TBase = CListViewCtrl, class TWinTraits = CControlWinTraits >
#else
template< class T, class TBase = CListBox, class TWinTraits = CControlWinTraits >
#endif
class ATL_NO_VTABLE CConnectionListImpl :
	public CWindowImpl< T, TBase, TWinTraits >
#ifndef USE_LISTVIEW
	, public COwnerDraw< T >
#endif
{
	DECLARE_WND_SUPERCLASS(NULL, TBase::GetWndClassName())

	// Member variables
	DWORD					m_dwEnumFlags;			// Flags for BcsEnumConnections
#ifndef _WIN32_WCE
    HDEVNOTIFY				m_hDevNotifyBthPort;	// Notification of change in Bluetooth ports
    HDEVNOTIFY				m_hDevNotifyHID;		// Notification of change in HID devices
#ifdef USE_LISTVIEW
	SP_CLASSIMAGELIST_DATA	m_ImageListData;
#endif
#endif
	int						m_nSerialPortIconIndex;
	int						m_nBluetoothIconIndex;
//public:
//	int nPortFilter;
	HRESULT		m_hr;					// HRESULT set by SetMessageId

#if 0
	// This function may be used in the future by OnDeviceChange in order to determine if there has been
	// any change in the number of available Bluetooth radios

	HRESULT GetNumBluetoothRadios( const GUID *pClassGuid )
	{
		// Open the device using device interface registered by the driver
		// Get the interface device information set that contains all devices of event class.

		HDEVINFO hDevInfo = SetupDiGetClassDevs( pClassGuid, NULL, NULL, DIGCF_PRESENT );

		if (hDevInfo == INVALID_HANDLE_VALUE)
			return HRESULT_FROM_WIN32( GetLastError() );

		//  Enumerate all the devices

		HRESULT hr = S_OK;
		int nRadios = 0;

		for (DWORD index = 0; index < 64; index++) {
			SP_DEVINFO_DATA deviceInfoData = {sizeof(SP_DEVINFO_DATA)};

			BOOL bStatus = SetupDiEnumDeviceInfo( hDevInfo, index, &deviceInfoData );

			if ( bStatus == FALSE ) {
				hr = HRESULT_FROM_WIN32( GetLastError() );
				break;                         // Free resources and return
			}

			DWORD   bufferSize = 256;
			BYTE    buffer[256];

			bStatus = SetupDiGetDeviceRegistryProperty(
						hDevInfo,
						&deviceInfoData,
						SPDRP_COMPATIBLEIDS,
						NULL,
						buffer,
						bufferSize,
						&bufferSize);

			// The Bluetooth HCI USB Transport Layer document specifies that all USB dongles have: 
			// class: 0xe0  Wireless Controller
			// subclass:0x01  RF Controller
			// prot: 0x01  Bluetooth programming

			if ( bStatus && _tcscmp((PCTSTR)buffer, _T("USB\\Class_e0&SubClass_01&Prot_01")) == 0 )
				nRadios++;
		}

		SetupDiDestroyDeviceInfoList( hDevInfo );

		return nRadios ? nRadios : hr;
	}
#endif

#if 0
	bool FindBluetoothRadio()
	{
#if 0
		// First check Bluetooth radios using the GUID defined by Microsoft
		if (GetNumBluetoothRadios(&GUID_DEVCLASS_BLUETOOTH ) > 0)
			return true;

		// Check Bluetooth radios using the GUID defined by Widcomm (Broadcomm)
		return GetNumBluetoothRadios(&GUID_DEVCLASS_BTW) > 0;
#else
		return true;
#endif
	}
#endif

// Message map and handlers
	BEGIN_MSG_MAP_EX(CConnectionListImpl)
		MSG_WM_CREATE(			OnCreate		)		// 0x0001
		MSG_WM_DESTROY(			OnDestroy		)		// 0x0002
		MSG_WM_DEVICECHANGE(	OnDeviceChange	)		// 0x0219

		CHAIN_MSG_MAP_ALT( COwnerDraw< T >, 1 )
		ALT_MSG_MAP(1)
	END_MSG_MAP()

	LRESULT OnCreate(CREATESTRUCT *pCreateStruct)
	{
		LRESULT lRes = DefWindowProc();
		ATLASSERT( ::IsWindow(m_hWnd) );
		return lRes;
	}

	void OnDestroy()
	{
		// Disconnect and delete any connection objects we have allocated.
		// Each list box item contains a pointer to the associated connection object.

		DeleteAllConnections();

		SetMsgHandled( false );				// Use default handling to destroy window
	}

#if 0
	bool IsBthModemComDevice( PCTSTR pszPortName )
	{
		TCHAR szTargetPath[256];
		DWORD dwRes	= ::QueryDosDevice(	pszPortName, szTargetPath, sizeof(szTargetPath)/sizeof(TCHAR) );
		if (dwRes == 0)
			return false;

		if (_tcsncmp(szTargetPath, _T("\\Device\\BthModem"), 16))
			return false;

		PCTSTR pszDevice = szTargetPath	+ 14;

		while (*pszDevice) {
			if (!isdigit(*pszDevice++))
				return false;				// Not a digit
		}

		return true;						// Matches the pattern \\Device\\BthModem#
	}
#endif

	bool IsInvalidPort( PCTSTR pszPortName )
	{
#ifndef _WIN32_WCE
		TCHAR szTargetPath[256];
		DWORD dwRes	= ::QueryDosDevice(	pszPortName, szTargetPath, sizeof(szTargetPath)/sizeof(TCHAR) );
		if (dwRes == 0)
			return true;

		if (_tcsncmp(szTargetPath, _T("\\Device\\BthModem"), 16))
			return false;

		PCTSTR pszDevice = szTargetPath	+ 16;

		while (*pszDevice) {
			if (!isdigit(*pszDevice++))
				return false;				// Not a digit
		}
#endif
		return true;						// Matches the pattern \\Device\\BthModem#
	}

#if 0
	unsigned int GetPort( PCTSTR pszPort, int nLen = -1 ) {
		ATLASSERT( nLen == -1 || nLen == (int)_tcslen(pszPort) );

		if (nLen == -1)
			nLen = _tcslen( pszPort );

		if (nLen < 4 || nLen > 7)			// COMn - COMnnn
			return 0;

		// First 3 characters must be 'C' 'O' 'M'
		if (!(*pszPort++ == 'C' && *pszPort++ == 'O' && *pszPort++ == 'M'))
			return 0;						// Doesn't start with "COM"

		int nPort = 0;

		// The rest of the string must be digits
		while (*pszPort) {
			if (!isdigit(*pszPort))
				return 0;					// Not a digit

			nPort *= 10;
			nPort += *pszPort++ - '0';
		}

		ATLASSERT( nPort > 0 && nPort <= 256 );

		return nPort;						// Should be valid port number
	}

	int	AddPortName( PCTSTR	pszPortName, int nLen =	-1 ) {
		int	nSel = LB_ERR;

		// Get the port	number and add the port	name to	the	combo box in numerical order
		unsigned int nPort = GetPort( pszPortName, nLen	);

		if (nPort) {
			// Find	the	insertion point
			int	n =	0;	//GetCount();
//			while (n-- > 0 && nPort	< GetItemData(n))
//				;
			// Insert the string and set its item data
//			nSel = InsertString( n + 1,	pszPortName	);
//			SetItemData( nSel, nPort );
			nPortFilter = nPort;
			EnumConnections( m_dwEnumFlags );
		}

		return nSel;
	}
#endif

	LRESULT	OnDeviceChange(	UINT uEvent, DWORD dwData )
	{
		// The WM_DEVICECHANGE device message notifies an application or device	driver of a	change to the hardware configuration of	a device or	the	computer.

		// uEvent =	(UINT) wParam;
		// dwData =	(DWORD)	lParam;

		switch (uEvent)	{
		case DBT_DEVNODES_CHANGED:
			// Sent	when configmg finishes a process tree batch. Some devnodes may have	been added or removed. This	is used
			// by ring3	applications which need	to be refreshed	whenever any devnode changes occur (i.e. device	manager).

			// We currently don't need to respond to this event because we register and handle more specific events.  We may need
			// too respond to this event in the future to determine if a USB Bluetooth dongle (or PCMCIA, etc.) has been added or
			// removed.  Unfortunately this event does not provide any information and gets called multiple times when a Bluetooth
			// radio is plugged in or unplugged, resulting in the list of devices flickering as it is redrawn multiple times.
#if 0
			if (m_hWnd && m_bSupportBluetooth) {
				int	nSel = FindStringExact(	-1,	pcszBluetooth );

				if (FindBluetoothRadio()) {		// Should have Bluetooth list entry
					if (nSel ==	LB_ERR)	{		// Bluetooth list entry	not	found, add it
						nSel = InsertString( 0,	pcszBluetooth );
						if (nSel !=	LB_ERR)	{
							SetItemData( nSel, 0 );
							SetCurSel( nSel	);	// Set the default selection

							if (GetCount() == 1)
								EnableWindow( true );
						}
					}
				}
				else {							// Should not have Bluetooth list entry
					if (nSel >=	0) {			// Bluetooth list entry	found, remove it
						int	nRemaining = DeleteString( nSel	);
						if (nRemaining == 0)
							EnableWindow( false	);
						else
							SetCurSel( 0 );	// Set the default selection
					}
				}
			}
#else
//			EnumConnections( m_dwEnumFlags );
#endif
			break;

		case DBT_DEVICEARRIVAL:
			// The system sends	the	DBT_DEVICEARRIVAL device message when a	device has been	inserted and becomes available.
			// The system sets the lParam to the address of	a DEV_BROADCAST_HDR	structure identifying the device inserted.
 
			// Parameters
			// dwData -	Address	of a DEV_BROADCAST_HDR structure identifying the device	inserted. 
			// dbch_devicetype - Type of device. Can be	one	of these values: 

			// DBT_DEVTYP_OEM				   0x00000000  // oem-defined device type
			// DBT_DEVTYP_DEVNODE			   0x00000001  // devnode number
			// DBT_DEVTYP_VOLUME			   0x00000002  // logical volume
			// DBT_DEVTYP_PORT				   0x00000003  // serial, parallel
			// DBT_DEVTYP_NET				   0x00000004  // network resource

#if 1
			if (m_hWnd)	{
				DEV_BROADCAST_HDR *pDbh	= (DEV_BROADCAST_HDR *)dwData;
				
				// If a new port has just been added, we try to determine if it can be ruled out.  If not,
				// then we just redraw the list from scratch letting the SDK decide which ports are valid.

				if (pDbh->dbch_devicetype == DBT_DEVTYP_PORT) {
					DEV_BROADCAST_PORT *pDbp = (DEV_BROADCAST_PORT *)pDbh;
					ATLTRACE( _T("DBT_DEVICEARRIVAL	- Port name: %s"), pDbp->dbcp_name );
					if (!IsInvalidPort(pDbp->dbcp_name)) {
#if 0
						int	nSel = AddPortName(	pDbp->dbcp_name	);
#if 0
						if (nSel !=	LB_ERR)	{
							SetCurSel( nSel	);	// Set the default selection

							if (GetCount() == 1)
								EnableWindow( true );
						}
#endif
#else
						EnumConnections( m_dwEnumFlags );	// More sophisticated way would be to just add the new port to the list (see AddPortName above)
#endif
					}
				}

				// We will get a device interface event if there is a change in the device set using either interface
				// that we registered.  We registered the HID interface solely for watching for changes in the Microvision
				// USB interface cable.  We search the device name for the vendor ID (Vid) and product ID (Pid) of our cable.

				// The other device interface that we register alerts us to any changes in the number of Bluetooth radios
				// present.  This will only work for Microsoft Bluetooth radios, although there is likely to be a different
				// device interface for other Bluetooth stacks.  If not, we can use the DBT_DEVNODES_CHANGED event.

				else if (pDbh->dbch_devicetype == DBT_DEVTYP_DEVICEINTERFACE) {
					DEV_BROADCAST_DEVICEINTERFACE *pDbdi = (DEV_BROADCAST_DEVICEINTERFACE *)pDbh;
					ATLTRACE( _T("DBT_DEVICEARRIVAL - Interface name:	%s"), pDbdi->dbcc_name );
					if (IsEqualGUID(pDbdi->dbcc_classguid, GUID_DEVINTERFACE_HID)) {
						//TCHAR *pVid_041f_Pid_0104 = _tcsstr( pDbdi->dbcc_name, _T("Vid_041f&Pid_0104") );
						if(_tcsstr( pDbdi->dbcc_name, _T("Vid_041f&Pid_0104")))	// Microvision Flic USB Interface
							EnumConnections( m_dwEnumFlags );
					}
					else	// GUID_BTHPORT_DEVICE_INTERFACE
						EnumConnections( m_dwEnumFlags );
				}
				else {
					ATLTRACE( _T("DBT_DEVICEARRIVAL - dbch_devicetype: %d"),	pDbh->dbch_devicetype );
				}
			}
#else
			EnumConnections( m_dwEnumFlags );
#endif
			break;

		case DBT_DEVICEREMOVECOMPLETE:
			// The system sends	the	DBT_DEVICEREMOVECOMPLETE device	message	when a device has been removed.

#if 1
			if (m_hWnd)	{
				DEV_BROADCAST_HDR *pDbh	= (DEV_BROADCAST_HDR *)dwData;
				if (pDbh->dbch_devicetype == DBT_DEVTYP_PORT) {
					DEV_BROADCAST_PORT *pDbp = (DEV_BROADCAST_PORT *)pDbh;
					ATLTRACE( _T("DBT_DEVICEREMOVECOMPLETE - Port name:	%s"), pDbp->dbcp_name );
#if 0
					int	nSel = GetCurSel();
					int	nRemove	= FindStringExact( -1, pDbp->dbcp_name );
					if (nRemove	>= 0) {				// COM port	list entry found, remove it
						int	nRemaining = DeleteString( nRemove );
						if (nRemaining == 0)
							EnableWindow( false	);
						else if	(nSel == nRemove)	// Need	to set the default selection
							SetCurSel( nSel	< nRemaining ? nSel	: 0	);
					}
#else
					EnumConnections( m_dwEnumFlags );		// Could be smarter about this and only delete the one entry from the list (if it exists).
#endif
				}
				else if (pDbh->dbch_devicetype == DBT_DEVTYP_DEVICEINTERFACE) {
					DEV_BROADCAST_DEVICEINTERFACE *pDbdi = (DEV_BROADCAST_DEVICEINTERFACE *)pDbh;
					ATLTRACE( _T("DBT_DEVICEREMOVECOMPLETE - Interface name:	%s"), pDbdi->dbcc_name );
					if (IsEqualGUID(pDbdi->dbcc_classguid, GUID_DEVINTERFACE_HID)) {
						if(_tcsstr( pDbdi->dbcc_name, _T("Vid_041f&Pid_0104")))	// Microvision Flic USB Interface
							EnumConnections( m_dwEnumFlags );
					}
					else	// GUID_BTHPORT_DEVICE_INTERFACE
						EnumConnections( m_dwEnumFlags );
				}
				else {
					ATLTRACE( _T("DBT_DEVICEREMOVECOMPLETE - dbch_devicetype: %d"),	pDbh->dbch_devicetype );
				}
			}
#else
			EnumConnections( m_dwEnumFlags );
#endif
			break;
		}

		// Return Values:
		// Return TRUE to grant	a requested	action.
		// Return BROADCAST_QUERY_DENY to deny a requested action.

		return TRUE;
	}

public:
	BOOL SubclassWindow(HWND hWnd)
	{
		ATLASSERT( m_hWnd == NULL );
		ATLASSERT( ::IsWindow(hWnd) );
		return CWindowImpl< T, TBase, TWinTraits >::SubclassWindow( hWnd );
	}

public:
// Constructors
	CConnectionListImpl() :
#ifndef _WIN32_WCE
		m_hDevNotifyBthPort(	NULL ),
		m_hDevNotifyHID(		NULL ),
#endif
//		nPortFilter(			0	 ),
		m_hr(					0	 )
	{
#ifndef _WIN32_WCE
		// GUIDs are the same, but I think this function is looking for class GUIDs
		BOOL b = SetupDiGetClassBitmapIndex( /*&GUID_DEVINTERFACE_SERENUM_BUS_ENUMERATOR*/&GUID_DEVCLASS_PORTS, &m_nSerialPortIconIndex );
		ATLASSERT( b );
		// SetupDiGetClassBitmapIndex fails to get Bluetooth icon until SetupDiGetClassImageList has been called
		//m_nBluetoothIconIndex = m_nSerialPortIconIndex;
		b = SetupDiLoadClassIcon( &GUID_DEVCLASS_BLUETOOTH, NULL, &m_nBluetoothIconIndex );
#endif
	}

	~CConnectionListImpl()
	{
#ifndef _WIN32_WCE
		BOOL b = UnregisterDeviceNotification( m_hDevNotifyHID );
			 b = UnregisterDeviceNotification( m_hDevNotifyBthPort );
#endif

#ifdef USE_LISTVIEW
		SetupDiDestroyClassImageList( &m_ImageListData );
#endif
	}


// Operations
	HRESULT Initialize( DWORD dwEnumFlags = BCS_ENUM_DISPLAY_DEFAULTS )
	{
		DEV_BROADCAST_DEVICEINTERFACE NotificationFilter = {0};

		ZeroMemory( &NotificationFilter, sizeof(NotificationFilter) );
		NotificationFilter.dbcc_size = sizeof(DEV_BROADCAST_DEVICEINTERFACE);
		NotificationFilter.dbcc_devicetype = DBT_DEVTYP_DEVICEINTERFACE;

//		May need to resort back to this in order not to miss any events
//		hDevNotify = RegisterDeviceNotification( m_hWnd, &NotificationFilter,
//			DEVICE_NOTIFY_WINDOW_HANDLE | DEVICE_NOTIFY_ALL_INTERFACE_CLASSES
//		);

#ifndef _WIN32_WCE
		NotificationFilter.dbcc_classguid = GUID_BTHPORT_DEVICE_INTERFACE;
		m_hDevNotifyBthPort = RegisterDeviceNotification( m_hWnd, &NotificationFilter, DEVICE_NOTIFY_WINDOW_HANDLE );
		NotificationFilter.dbcc_classguid = GUID_DEVINTERFACE_HID;
		m_hDevNotifyHID = RegisterDeviceNotification( m_hWnd, &NotificationFilter, DEVICE_NOTIFY_WINDOW_HANDLE );

#ifdef USE_LISTVIEW
		InsertColumn ( 0, _T("Port"),			LVCFMT_LEFT,   69, 0 );
		InsertColumn ( 1, _T("Description"),	LVCFMT_LEFT,  172, 1 );

		// Set list view image lists
		m_ImageListData.cbSize = sizeof(m_ImageListData);
		SetupDiGetClassImageList( &m_ImageListData );

		// Get the Bluetooth icon index
//		if (m_nBluetoothIconIndex == m_nSerialPortIconIndex)
//			SetupDiGetClassBitmapIndex( &GUID_DEVCLASS_BLUETOOTH, &m_nBluetoothIconIndex );

		//m_hImageList = sImageListData.ImageList;
		SetImageList( m_ImageListData.ImageList, LVSIL_SMALL );
#endif
#endif

		return EnumConnections(dwEnumFlags);
	}

	HRESULT	GetLastResult()					{ return m_hr; }

	void DeleteAllConnections()
	{
		// Disconnect and delete any connection objects we have allocated.
		// Each control (list box or list view) item contains a pointer to the associated connection object.

		const int nCount = GetCount();

		if (nCount) {
			for (int i = 0; i < nCount; i++) {	// Loop through items in the control
				CConnection *pConnection = GetConnection( i );

				ATLASSERT( pConnection );
				delete pConnection;				// Delete the connection object
			}

			ResetContent();						// Remove all of the items from the control
		}
	}

	HRESULT EnumConnections( DWORD dwFlags = BCS_ENUM_DISPLAY_DEFAULTS )
	{
		m_dwEnumFlags = dwFlags & ~(BCS_ENUM_DISCOVER | BCS_ENUM_SELECT);

#ifdef USE_LISTVIEW
		//DeleteAllItems();	// BUGBUG!!! Need to delete the item data
#else
		DeleteAllConnections();
#endif

		m_hr = BcsEnumConnections(
			NULL,
			m_hWnd,
			ConnectionCallback,						//  Will be called for each connection
			this,									//  Passed to pfnConnectionCallback as the pvParam.
			dwFlags
			);

#if SHOW_LOOPBACK_PORTS

		// Add entries for the loopback serial ports
		DWORD requiredSize = 0;
		BOOL bGuidClass = SetupDiClassGuidsFromNameA("VSPorts", 0, 0, &requiredSize);

		if (requiredSize > 0) {
			_GUID *pGuidClass = (_GUID*) malloc(requiredSize * sizeof(GUID));

			//get GUID of the ports class:
			bGuidClass = SetupDiClassGuidsFromNameA("VSPorts", pGuidClass, requiredSize * sizeof(GUID), &requiredSize);

			if (bGuidClass) {
				BcsEnumConnections(
					pGuidClass,
					m_hWnd,
					ConnectionCallback,
					this,
					DIGCF_PRESENT /*|	DIGCF_DEVICEINTERFACE*/ | BCS_ENUM_FRIENDLYNAME
					);

				free( pGuidClass );
			}
		}
#endif

		if (SUCCEEDED(m_hr)) {
			if (GetCount()) {
				m_hr = S_OK;
				SetCurSel( 0 );					// Set the default selection
				EnableWindow( true );
			}
			else {
				m_hr = BCS_MSG_NO_CONNECTIONS;
				EnableWindow( false	);
			}
		}

		BcsDevBroadcast sDeviceChangeEvent = {sizeof sDeviceChangeEvent, DBT_USERDEFINED};
		StringCchCopyA( sDeviceChangeEvent.dbud_szName, sizeof sDeviceChangeEvent.dbud_szName, "Microvision\\EnumConnections" );
		::SendMessage( GetParent(), WM_DEVICECHANGE, DBT_USERDEFINED, (LPARAM)&sDeviceChangeEvent );

		return m_hr;
	}

	HRESULT Discover()
	{
#ifdef _WIN32_WCE
		return EnumConnections( m_dwEnumFlags | BCS_ENUM_DISCOVER );
#else
		return EnumConnections( m_dwEnumFlags | BCS_ENUM_SELECT );
#endif
	}

	LRESULT AddConnection( BCS_CONNECTION_INFO *pConnectionInfo )
	{
		USES_CONVERSION;

		// Adds a string to the list box and if successful uses a copy constructor
		// to allocate a new CConnection object and associate it as item data.

#ifndef USE_LISTVIEW
		int nSel = AddString( W2T(pConnectionInfo->szName) );

		switch (nSel) {
		case LB_ERR:		return GetLastError();
		case LB_ERRSPACE:	return LB_ERRSPACE;
		}

		ATLASSERT( nSel >= 0 );				// AddString succeeded

		CConnection *pConnection = new CConnection( GetParent() );
		pConnection->m_ConnectionInfo = *(CConnectionInfo *)pConnectionInfo;

		if (pConnection == NULL || SetItemData(nSel, reinterpret_cast<DWORD_PTR>(pConnection)) == LB_ERR) {
			DeleteConnection( nSel );		// Deletes the newly added string
			delete pConnection;
			return pConnection ? E_FAIL : E_OUTOFMEMORY;
		}

		SetCurSel( nSel );					// Set as current selection
#else
		const GUID *pClassGuid = connection.m_ConnectionInfo.ulClassofDevice == BCS_COD_SERIAL_PORT
			? &GUID_DEVINTERFACE_SERENUM_BUS_ENUMERATOR
			: &GUID_DEVCLASS_BLUETOOTH;
		int nImage = -1;
		SetupDiGetClassImageIndex( &m_ImageListData, pClassGuid, &nImage );

		int nSel = GetItemCount();
//		if (connection.m_ConnectionInfo.ulClassofDevice == COD_SERIAL_PORT) {
//			InsertItem(  nSel, pConnection->GetName(), nImage );
//			InsertItem(  nSel, (PCTSTR)connection.m_ConnectionInfo.Address.rgBytes, nImage );
			switch ( connection.Mode() ) {
			case SERIAL_TETHERED:				// These modes are currently supported
			{
				InsertItem(  nSel, (PCTSTR)connection.m_ConnectionInfo.PortName(), nImage );
				break;
			}
			case BLUETOOTH_TETHERED:
			{
				//char caddr[24];
				//sprintf( caddr, "%I64x", connection.m_ConnectionInfo.Address.ullLong );
				InsertItem(  nSel, /*caddr*/_T("Bluetooth"), nImage );
				break;
			}
			}

			SetI ext( nSe ection.m_sz onId );
			SetItemText( nSel, 1, connection.m_ConnectionInfo.szName );
//		}

		CConnection *pConnection = new CConnection(connection);

		if (pConnection == NULL || SetItemData(nSel, reinterpret_cast<DWORD_PTR>(pConnection)) == FALSE) {
			DeleteConnection( nSel );		// Deletes the newly added string
			delete pConnection;
			return pConnection ? E_FAIL : E_OUTOFMEMORY;
		}
#endif

		return ERROR_SUCCESS;				// Successfully added connection to list box
	}

#ifdef USE_LISTVIEW
	int GetCurSel() const		{ return GetSelectedIndex(); }
	int SetCurSel( int nSel )	{ return SelectItem(nSel) ? nSel : LB_ERR; }
	int GetCount() const		{ return GetItemCount(); }
	void ResetContent()			{ /*DeleteAllItems();*/::SendMessage(m_hWnd, LVM_DELETEALLITEMS, 0, 0L); }	//BUGBUG!!!!
#endif

	CConnection *GetConnection() { return GetConnection( GetCurSel() ); }

	CConnection *GetConnection( int nIndex )
	{
		// Returns the CConnection object associated with the specified list box item.

		DWORD_PTR dwItemData = GetItemData( nIndex );

		if (dwItemData == LB_ERR)			// nIndex out of range?
			return NULL;

		CConnection *pConnection = reinterpret_cast<CConnection *>( dwItemData );

		ATLASSERT( pConnection );			// Should always have a valid connection object
											// associated with the list box item
		return pConnection;
	}

	HRESULT DeleteConnection( int nIndex )
	{
		// Deletes the specified connection control string and associated connection object.
		// Sets the new selection.

		DWORD_PTR dwItemData = GetItemData( nIndex );

		if (dwItemData == LB_ERR)			// nIndex out of range
			return E_INVALIDARG;

		CConnection *pConnection = reinterpret_cast<CConnection *>( dwItemData );

		if (pConnection == NULL)			// No item data
			return HRESULT_FROM_WIN32( ERROR_INVALID_DATA );

		HRESULT hr = BcsRemoveConnection( &pConnection->m_ConnectionInfo, 0 );

#ifndef _WIN32_WCE							// Will get an error attempting to delete an unauthenticated device
		if (FAILED(hr))
			return hr;						// Return BCS SDK error
#endif

		delete pConnection;					// Delete the connection object

		int nSel = GetCurSel();				// Save the current selection

#ifndef USE_LISTVIEW
		int nCount = DeleteString( nSel );	// Delete string from list box
#else
		BOOL bRet = DeleteItem( nIndex );	// Delete string from control
		const int nCount = GetCount();
#endif

		if (nCount) {
			if (nIndex == nSel) {			// Need to restore the current selection
				if (nSel >= nCount)			// Selection is out of range
					nSel = nCount - 1;		// Set selection to last item in the list
				SetCurSel( nSel );			// Set new selection
			}
		}

		return S_OK;						// Successfully deleted the entry
	}

	// COwnerDraw

	void MeasureItem( MEASUREITEMSTRUCT *pMeasureItemStruct )
	{
		CClientDC dc(m_hWnd);
		TEXTMETRIC tm;
		dc.GetTextMetrics(&tm);
		pMeasureItemStruct->itemHeight = tm.tmHeight + 1;
	}

	void DrawItem( DRAWITEMSTRUCT *pDrawItemStruct )
	{
		// Set up state variables
		BOOL bSelected		= pDrawItemStruct->itemState & ODS_SELECTED;		// 0x0001
		BOOL bDisabled		= pDrawItemStruct->itemState & ODS_DISABLED;		// 0x0004
		BOOL bHaveFocus		= pDrawItemStruct->itemState & ODS_FOCUS;			// 0x0010

		// Fill the background
		CDCHandle dc = pDrawItemStruct->hDC;
		RECT rcItem = pDrawItemStruct->rcItem;
		COLORREF crBackground = bSelected ? COLOR_HIGHLIGHT : COLOR_WINDOW;
		dc.FillRect( &rcItem, ::GetSysColorBrush(crBackground) );

		// Check for empty list box
		if (pDrawItemStruct->itemID == (UINT)-1) {
			if (bHaveFocus)					// Draw the focus rectangle
				dc.DrawFocusRect( &rcItem );

			return;							// List box is empty, nothing else required
		}

		// Get a pointer to the connection data to use in distinguishing between Bluetooth and Serial ports
		CConnection *pConnection = reinterpret_cast<CConnection *>( pDrawItemStruct->itemData );

#ifndef _WIN32_WCE
		// Draw the icon
		rcItem.left += 2;					// Add a 2 pixel left margin to the left of the icon
		rcItem.left = SetupDiDrawMiniIcon( dc, rcItem,
			pConnection->m_ConnectionInfo.IsSerialPort() ? m_nSerialPortIconIndex : m_nBluetoothIconIndex,
			MAKELONG( DMI_BKCOLOR, crBackground )  
			) + 2;							// Add another 2 pixels to right of the icon
#endif

		// Draw the text
		TCHAR szItem[64] = { 0 };
		ATLASSERT( GetTextLen(pDrawItemStruct->itemID) < 64 );
		GetText( pDrawItemStruct->itemID, szItem );
		dc.SetBkMode( TRANSPARENT );
		dc.SetTextColor( ::GetSysColor(bDisabled ? COLOR_GRAYTEXT : (bSelected ? COLOR_HIGHLIGHTTEXT : COLOR_WINDOWTEXT)) );
		dc.DrawText( szItem, -1, &rcItem, DT_SINGLELINE | DT_NOPREFIX | DT_VCENTER );

		// May need to draw a focus rectangle around the text
		if (!bSelected && bHaveFocus) {
			RECT rcFocus = rcItem;
			dc.DrawText(szItem, -1, &rcFocus, DT_SINGLELINE | DT_NOPREFIX | DT_VCENTER | DT_CALCRECT);
			rcFocus.bottom += 1;
			::InflateRect(&rcFocus, 2, 0);
			dc.DrawFocusRect(&rcFocus);
      }
   }
};

class CConnectionListCtrl : public CConnectionListImpl<CConnectionListCtrl, CListBox, CWinTraitsOR<LBS_OWNERDRAWVARIABLE|LBS_HASSTRINGS> >
{
   DECLARE_WND_SUPERCLASS(_T("WTL_ConnectionListBox"), GetWndClassName())  
};

#if 0
class CConnectionComboCtrl : public CConnectionListImpl< CConnectionComboCtrl, CComboBox, CWinTraitsOR<CBS_DROPDOWN> >
{
   DECLARE_WND_SUPERCLASS(_T("WTL_ConnectionComboBox"), GetWndClassName())  
};
#endif

#ifdef USE_LISTVIEW
class CConnectionListViewCtrl : public CConnectionListImpl< CConnectionListViewCtrl, CListViewCtrl >
{
   DECLARE_WND_SUPERCLASS(_T("WTL_ConnectionListView"), GetWndClassName())  
};
#endif

static HRESULT WINAPI ConnectionCallback( void *pvParam, BCS_CONNECTION_INFO *pConnectionInfo )
{
	USES_CONVERSION;

	CConnectionListCtrl* pConnectionListCtrl = reinterpret_cast<CConnectionListCtrl *>( pvParam );

//	int nPortFilter = pConnectionListCtrl->nPortFilter;

//	if (nPortFilter && nPortFilter != pConnectionInfo->Address.ullLong)
//		return S_OK;	// Keep going

	// Adds a string to the control and if successful uses a copy constructor
	// to allocate a new CConnection object and associate it as item data.
	// Display the name or address of the device

	LRESULT lRes = pConnectionListCtrl->AddConnection( pConnectionInfo );

//	if (nPortFilter && nPortFilter =!= pConnectionInfo->Address.ullLong)
//		return S_OK;	// All done, found the one we were looking for

	// Map lRes to an HRESULT
	switch (lRes) {
	case ERROR_SUCCESS:				return S_OK;
	case LB_ERRSPACE:				return E_OUTOFMEMORY;
	}

	return AtlHresultFromWin32( lRes );
}

/// Trace Control.
/// Displays in SendKeys format.
#if 0
// uMapType values for MapVirtualKey
enum {
	eVirtualKeyToScanCode	   = 0,	// uCode is a virtual-key code and is translated into a scan code. If it is a virtual-key code that does not distinguish between left- and right-hand keys, the left-hand scan code is returned. If there is no translation, the function returns 0. 
	eScanCodeToVirtualKey	   = 1,	// uCode is a scan code and is translated into a virtual-key code that does not distinguish between left- and right-hand keys. If there is no translation, the function returns 0. 
	eVirtualKeyToUnshiftedChar = 2,	// uCode is a virtual-key code and is translated into an unshifted character value in the low-order word of the return value. Dead keys (diacritics) are indicated by setting the top bit of the return value. If there is no translation, the function returns 0. 
	eScanCodeToVirtualKeyLR	   = 3	// uCode is a scan code and is translated into a virtual-key code that distinguishes between left- and right-hand keys. If there is no translation, the function returns 0. 
};
#endif
static PCTSTR AsciiName[34] = {
_T("<NUL>"),	// ^@	  0	0x0
_T("<SOH>"),	// ^A	  1	0x1
_T("<STX>"),	// ^B	  2	0x2
_T("<ETX>"),	// ^C	  3	0x3
_T("<EOT>"),	// ^D	  4	0x4
_T("<ENQ>"),	// ^E	  5	0x5
_T("<ACK>"),	// ^F	  6	0x6
_T("<BEL>"),	// ^G	  7	0x7
_T("<BS>"),		// ^H	  8	0x8
_T("<HT>"),		// ^I	  9	0x9
_T("<LF>"),		// ^J	 10	0xA
_T("<VT>"),		// ^K	 11	0xB
_T("<FF>"),		// ^L	 12	0xC
_T("<CR>"),		// ^M	 13	0xD
_T("<SO>"),		// ^N	 14	0xE
_T("<SI>"),		// ^O	 15	0xF
_T("<DLE>"),	// ^P	 16	0x10
_T("<DC1>"),	// ^Q	 17	0x11
_T("<DC2>"),	// ^R	 18	0x12
_T("<DC3>"),	// ^S	 19	0x13
_T("<DC4>"),	// ^T	 20	0x14
_T("<NAK>"),	// ^U	 21	0x15
_T("<SYN>"),	// ^V	 22	0x16
_T("<ETB>"),	// ^W	 23	0x17
_T("<CAN>"),	// ^X	 24	0x18
_T("<EM>"),		// ^Y	 25	0x19
_T("<SUB>"),	// ^Z	 26	0x1A
_T("<ESC>"),	// ^[	 27	0x1B
_T("<FS>"),		// ^\	 28	0x1C
_T("<GS>"),		// ^]	 29	0x1D
_T("<RS>"),		// ^^	 30	0x1E
_T("<US>"),		// ^_	 31	0x1F
_T("<SP>"),		// SP	 32	0x20
_T("<DEL>")		// DEL	127	0x7F
};

#ifndef _WIN32_WCE
template< class T, class TBase = CRichEditCtrl, class TWinTraits = CControlWinTraits >
#else
template< class T, class TBase = CEdit, class TWinTraits = CControlWinTraits >
#endif
class ATL_NO_VTABLE CTraceImpl : public CWindowImpl< T, TBase, TWinTraits >
{
	DECLARE_WND_SUPERCLASS(NULL, TBase::GetWndClassName())

	BOOL	m_bBorder;						// Original WS_BORDER setting
	BOOL	m_bClientEdge;					// Original WS_EX_CLIENTEDGE setting
	bool	m_bThemedBorder;
	RECT	m_rcClientPos;
#ifndef _WIN32_WCE
	CTheme *m_pTheme;
#endif

	PCTSTR GetVirtualKeyName( int nVk, bool bNonCharFormingOnly = false )
	{
		switch ( nVk ) {	// Switch on the virtual key code
		case VK_PAUSE:		return _T("{BREAK}");
		case VK_CAPITAL:	return _T("{CAPSLOCK}");
		case VK_DELETE:		return _T("{DELETE}");
		case VK_DOWN:		return _T("{DOWN}");
		case VK_END:		return _T("{END}");
		case VK_F1:			return _T("{F1}");
		case VK_F10:		return _T("{F10}");
		case VK_F11:		return _T("{F11}");
		case VK_F12:		return _T("{F12}");
		case VK_F13:		return _T("{F13}");
		case VK_F14:		return _T("{F14}");
		case VK_F15:		return _T("{F15}");
		case VK_F16:		return _T("{F16}");
		case VK_F2:			return _T("{F2}");
		case VK_F3:			return _T("{F3}");
		case VK_F4:			return _T("{F4}");
		case VK_F5:			return _T("{F5}");
		case VK_F6:			return _T("{F6}");
		case VK_F7:			return _T("{F7}");
		case VK_F8:			return _T("{F8}");
		case VK_F9:			return _T("{F9}");
		case VK_HELP:		return _T("{HELP}");
		case VK_HOME:		return _T("{HOME}");
		case VK_INSERT:		return _T("{INSERT}");
		case VK_LEFT:		return _T("{LEFT}");
		case VK_NUMLOCK:	return _T("{NUMLOCK}");
		case VK_NEXT:		return _T("{PGDN}");
		case VK_PRIOR:		return _T("{PGUP}");
		case VK_SNAPSHOT:	return _T("{PRTSC}");
		case VK_RIGHT:		return _T("{RIGHT}");
		case VK_SCROLL:		return _T("{SCROLLLOCK}");
		case VK_UP:			return _T("{UP}");
		}

		if (bNonCharFormingOnly == false) {
			switch ( nVk ) {
			case VK_CANCEL:		return _T("{BREAK}");
			case VK_BACK:		return _T("{BACKSPACE}");
			case VK_TAB:		return _T("{TAB}");
			case VK_RETURN:		return _T("{ENTER}\n");
			case VK_ESCAPE:		return _T("{ESC}");
			case VK_SPACE:		return _T("{SPACE}");
			}
		}

		return NULL;		// No match
	}

	// Message map and handlers
	BEGIN_MSG_MAP_EX(CTraceImpl)
		MSG_WM_CREATE(			OnCreate		)	// 0x0001
		MSG_WM_DESTROY(			OnDestroy		)	// 0x0002
#ifndef _WIN32_WCE
		MSG_WM_STYLECHANGED(	OnStyleChanged	)	// 0x007D
		MSG_WM_NCCALCSIZE(		OnNcCalcSize	)	// 0x0083
		MSG_WM_NCPAINT(			OnNcPaint		)	// 0x0085
#endif

		MESSAGE_RANGE_HANDLER_EX( WM_KEYFIRST, WM_KEYLAST, OnKeyMessage )	// 0x0100 - 0x0109

#ifndef _WIN32_WCE
		MSG_WM_THEMECHANGED(	OnThemeChanged	)	// 0x031A
#endif
	END_MSG_MAP()

    // Message handlers
	LRESULT OnCreate(CREATESTRUCT *pCreateStruct)
	{
		//MessageBox( _T("OnCreate"), _T("OnCreate") );
		LRESULT lRes = DefWindowProc();
		ATLASSERT( ::IsWindow(m_hWnd) );
		return lRes;
	}
	void OnDestroy()
	{
#ifndef _WIN32_WCE
		if (m_pTheme)
			m_pTheme->CloseThemeData();

		delete m_pTheme;
#endif
	}

#ifndef _WIN32_WCE
	void OnStyleChanged( int nStyleType, STYLESTRUCT *pStyleStruct )
	{
		// uStyleType specifies whether the window's styles or extended window styles have changed. This parameter can be one or more of the following values. 
		// GWL_EXSTYLE:		// The extended window styles have changed.
		// GWL_STYLE:		// The window styles have changed.

		// pStyleStruct is a pointer to a STYLESTRUCT structure that contains the new styles for the window.
		// An application can examine the styles, but cannot change them.

		bool bCurrentThemedBorder = m_bThemedBorder;
		m_bThemedBorder = false;	//true;	//false;

		//LONG lStyle		= GetWindowLong( GWL_STYLE );
		//LONG lExStyle	= GetWindowLong( GWL_EXSTYLE );

		//First, check if the control is supposed to have a border
		if ((m_pTheme && bCurrentThemedBorder)
	//		|| (GetWindowLong(GWL_STYLE) & WS_BORDER || GetWindowLong(GWL_EXSTYLE) & WS_EX_CLIENTEDGE))
			|| m_bBorder|| m_bClientEdge)
		{
			//Check if a theme is presently active
			if (IsThemeActive())
				m_bThemedBorder = true;
		}

		//Recalculate the NC area and repaint the window
		SetWindowPos( NULL, NULL, NULL, NULL, NULL, SWP_NOMOVE|SWP_NOSIZE|SWP_NOZORDER|SWP_NOACTIVATE|SWP_FRAMECHANGED );
		RedrawWindow( NULL, NULL, RDW_INVALIDATE|RDW_NOCHILDREN|RDW_UPDATENOW|RDW_FRAME );
	}

	//LRESULT OnNcCalcSize(BOOL bCalcValidRects, NCCALCSIZE_PARAMS *pncsp)		// Should be
	LRESULT OnNcCalcSize( BOOL bCalcValidRects, LPARAM lParam )					// Actual
	{
		//If wParam is FALSE, we don't need to make any calculation
		if (bCalcValidRects == FALSE)
			return 0;
					//Ask the control to first calculate the space it needs
					//LRESULT nOriginalReturn = CallWindowProc(m_pOriginalWndProc, hwnd, uMsg, wParam, lParam);
					//LRESULT lRes = DefWindowProc();

		//m_bMsgHandled = false;					// Let the control factor in its own non-client elements (scrollbars, etc.)
		LRESULT lRes = DefWindowProc();

		try
		{
			//Here, we indicate to Windows that the non-client area of the richedit control is not what it thinks it should be
			//This gives us the necessary space to draw the special border later on
			if (m_bThemedBorder)
			{
				NCCALCSIZE_PARAMS *pncsp = (NCCALCSIZE_PARAMS*)lParam;

				if (m_pTheme)
				{
					//Get the size required by the current theme to be displayed properly
					RECT rcClient; ZeroMemory(&rcClient, sizeof(RECT));
					HDC hdc = ::GetDC(GetParent());
					if (m_pTheme->GetThemeBackgroundContentRect(hdc, EP_EDITTEXT, ETS_NORMAL, &pncsp->rgrc[0], &rcClient) == S_OK)
					{
						//Add a pixel to every edge so that the client area is not too close to the border drawn by the theme (thus simulating a native edit box)
						InflateRect(&rcClient, -1, -1);

						m_rcClientPos.left		= rcClient.left			- pncsp->rgrc[0].left;
						m_rcClientPos.top		= rcClient.top			- pncsp->rgrc[0].top;
						m_rcClientPos.right		= pncsp->rgrc[0].right	- rcClient.right;
						m_rcClientPos.bottom	= pncsp->rgrc[0].bottom	- rcClient.bottom;

						memcpy(&pncsp->rgrc[0], &rcClient, sizeof(RECT));
					}
					::ReleaseDC(GetParent(), hdc);

					return 0;	//WVR_REDRAW;
				}
			}

		}
		catch(...)
		{
			//No exception is supposed to be thrown here but you can never be too safe
			//Trace an error here with your favorite TRACE macro
			//MessageBox( _T("OnNcCalcSize"), _T("catch") );
		}
		return 0;;
	}

	void OnNcPaint( HRGN hRgn )
	{
//		m_bMsgHandled = false;				// Let the control paint its own non-client elements (scrollbars, etc.)
		LRESULT lRes = DefWindowProc();
	#if 1
		try
		{
			if (m_bThemedBorder)
			{
				if (m_pTheme)
				{
					HDC hdc = GetWindowDC();

					//Clip the DC so that we only draw on the non-client area
					RECT rcBorder;
					GetWindowRect(&rcBorder);
					rcBorder.right -= rcBorder.left; rcBorder.bottom -= rcBorder.top;
					rcBorder.left = rcBorder.top = 0;

					RECT rcClient; memcpy(&rcClient, &rcBorder, sizeof(RECT));
					rcClient.left	+= m_rcClientPos.left;
					rcClient.top	+= m_rcClientPos.top;
					rcClient.right	-= m_rcClientPos.right;
					rcClient.bottom	-= m_rcClientPos.bottom;
					ExcludeClipRect(hdc, rcClient.left, rcClient.top, rcClient.right, rcClient.bottom);

					//Make sure the background is in a proper state
					if (m_pTheme->IsThemeBackgroundPartiallyTransparent(EP_EDITTEXT, ETS_NORMAL))
						::DrawThemeParentBackground( m_hWnd, hdc, &rcBorder );

					//Draw the border of the edit box
					int nState;
					if (!IsWindowEnabled())
						nState = ETS_DISABLED;
					else if (GetOptions() & ECO_READONLY)
						nState = ETS_READONLY;
					else
						nState = ETS_NORMAL;

					HRESULT hr = m_pTheme->DrawThemeBackground( hdc, EP_EDITTEXT, nState, &rcBorder );
					ATLASSERT( SUCCEEDED(hr) );
					//AtlDrawThemeClientEdge(m_pTheme->m_hTheme, m_hWnd, /*hRgnUpdate*/NULL, NULL, EP_EDITTEXT, nState);

					ReleaseDC( hdc );
				}
			}
		}
		catch(...)
		{
			//No exception is supposed to be thrown here but you can never be too safe
			//Trace an error here with your favorite TRACE macro
		}
	#else
		if (m_pTheme && m_pTheme->m_hTheme) {
			//LRESULT lRet = 0;
	//		if (/*IsThemingSupported() &&*/ ((GetExStyle() & WS_EX_CLIENTEDGE) != 0))
			{
	//			if((m_dwExtendedStyle & THEME_EX_3DCLIENTEDGE) != 0)
	//			{
	//				lRet = ::DefWindowProc(pT->m_hWnd, uMsg, wParam, lParam);
	//				bHandled = TRUE;
	//			}
	//			/*else*/ if (/*(m_hTheme != NULL) &&*/ ((m_dwExtendedStyle & THEME_EX_THEMECLIENTEDGE) != 0))
				{
					//HRGN hRgn = (wParam != 1) ? (HRGN)wParam : NULL;
					try
					{
					/*bool bRet =*/ AtlDrawThemeClientEdge(m_pTheme->m_hTheme, m_hWnd, /*hRgnUpdate*/NULL, NULL, EP_EDITTEXT, ETS_NORMAL);
					}
					catch(...)
					{
						_RPTF0( _CRT_ERROR, "Exception in CTraceImpl::OnNcPaint" );
					}

				}
			}
		}
	#endif
	}
#endif

	LRESULT OnKeyMessage(UINT uMsg, WPARAM wParam, LPARAM lParam)
	{
		const int cchKeyName = 16;			// Longest keyname is "^{SCROLLLOCK}"
		TCHAR	szKeyName[cchKeyName] = {0};
		PCTSTR	pszKeyName = szKeyName;		// Pointer to NUL terminated string that will be displayed in the Trace window

		switch ( uMsg ) {
		case WM_KEYDOWN:					// 0x0100
			pszKeyName = GetVirtualKeyName( wParam, true );
			if (pszKeyName)
				break;						// This virtual key does not result in a WM_CHAR message, process WM_KEYDOWN

			return 0;						// Process WM_CHAR instead

		case WM_KEYUP:						// 0x0101
			return 0;

		case WM_CHAR:						// 0x0102
			if (wParam > VK_SPACE && wParam != 127)
				// Windows should provide a character that is in the range of TCHAR (Unicode or ANSI)
				szKeyName[0] = (TCHAR)wParam;
			else {									// Non-printable character (wParam <= VKSPACE || wParam == 127)
				long lScanCodeBits = lParam & 0x01ff0000;

				if (lScanCodeBits == 0)				// WM_CHAR message was not generated via the keyboard.  Use ASCII notation.
					pszKeyName = AsciiName[wParam <= VK_SPACE ? wParam : sizeof(AsciiName)/sizeof(PCSTR)-1];
				else {								// Have scan code, now determine whether to use SendKeys or control key notation
					long lScanCode = lScanCodeBits >> 16;
					UINT uVk = MapVirtualKey( lScanCode, eScanCodeToVirtualKeyLR );

													// ^A - ^Z
					if (uVk >= 'A' &&  uVk <= 'Z' && wParam == uVk - 0x40) {
						szKeyName[0] = '^';			// Indicate that this is a control character
						szKeyName[1] = uVk;			// 'A' - 'Z'
					}
					else {
						PCTSTR pszVirtualKeyName = GetVirtualKeyName( uVk ? uVk : wParam );

						if (pszVirtualKeyName) {	// {BACKSPACE}, {TAB}, {ENTER}, {ESC}, {SPACE}
							if (uVk == wParam)
								pszKeyName = pszVirtualKeyName;
							else {					// uVk != wParam, Ctrl key must have been down
								HRESULT hr = StringCchPrintf ( szKeyName, cchKeyName, _T("^%s"), pszVirtualKeyName );
								ATLASSERT( SUCCEEDED(hr) );
							}
						}
						else {						// ^@, ^[, ^\, ^], ^^, ^_
							szKeyName[0] = '^';
							szKeyName[1] = wParam + 0x40;
						}
					}
				}
			}
			break;

		case WM_DEADCHAR:					// 0x0103
				ATLTRACE( _T("Unprocessed WM_DEADCHAR message( wParam: %x lParam: %x )\n"), wParam, lParam );
				return 0;

		case WM_SYSKEYDOWN:					// 0x0104
				//ATLTRACE( _T("Unprocessed WM_SYSKEYDOWN message( wParam: %x lParam: %x )\n"), wParam, lParam );
				return 0;

		case WM_SYSKEYUP:					// 0x0105
				return 0;

		case WM_SYSCHAR:					// 0x0106
				if (wParam > VK_SPACE && wParam != 127) {
					// Windows should provide a character that is in the range of TCHAR (Unicode or ANSI)
					szKeyName[0] = '%';		// Symbol for Alt
					szKeyName[1] = (TCHAR)wParam;
					break;
				}
				ATLTRACE( _T("Unprocessed WM_SYSCHAR message( wParam: %x lParam: %x )\n"), uMsg, wParam, lParam );
				return 0;

		case WM_SYSDEADCHAR:				// 0x0107
				ATLTRACE( _T("Unprocessed WM_SYSDEADCHAR message( wParam: %x lParam: %x )\n"), uMsg, wParam, lParam );
				return 0;

#ifndef _WIN32_WCE
		case WM_UNICHAR:					// 0x0109
			szKeyName[0] = wParam;
			break;
#endif

		default:
#ifndef _WIN32_WCE
			_RPTF3( _CRT_ERROR, "Unhandled message uMsg: %x wParam: %x lParam: %x\n", uMsg, wParam, lParam );
#else
			ATLASSERT( uMsg >= WM_KEYFIRST && uMsg < WM_KEYLAST );
#endif
		}

		ReplaceSel( pszKeyName );			// Add the string to the RichEdit control

		return 0;
	}

	void OnThemeChanged()
	{
		if (m_pTheme) {
			m_pTheme->CloseThemeData();		// Close the previous theme

			//Load the theme associated with edit boxes
			if (m_pTheme->OpenThemeData(m_hWnd, L"Edit") == NULL) {
				m_pTheme->CloseThemeData();
				delete m_pTheme;
				m_pTheme = NULL;
			}
		}

		bool bCurrentThemedBorder = m_bThemedBorder;
		m_bThemedBorder = false;	//true;	//false;

		//First, check if the control is supposed to have a border
		if ((m_pTheme && bCurrentThemedBorder)
	//		|| (GetWindowLong(GWL_STYLE) & WS_BORDER || GetWindowLong(GWL_EXSTYLE) & WS_EX_CLIENTEDGE))
			|| m_bBorder|| m_bClientEdge)
		{
			DWORD dwStyle	= GetStyle();
			DWORD dwExStyle = GetExStyle();

			if (IsThemeActive()) {				// Remove the border style, we don't want the control to draw its own border
				m_bThemedBorder = true;
				if (dwStyle & WS_BORDER)
					SetWindowLong( GWL_STYLE, dwStyle ^ WS_BORDER);
				if (dwExStyle & WS_EX_CLIENTEDGE)
					SetWindowLong( GWL_EXSTYLE, dwExStyle ^ WS_EX_CLIENTEDGE);
			}
			else {								// Restore the border style, we want the control to draw its own border
				if (m_bBorder && (dwStyle & WS_BORDER) == 0)
					SetWindowLong( GWL_STYLE, dwStyle | WS_BORDER );
				if (m_bClientEdge && (dwExStyle & WS_EX_CLIENTEDGE) == 0)
					SetWindowLong( GWL_EXSTYLE, dwExStyle | WS_EX_CLIENTEDGE );
			}
		}

		//Recalculate the NC area and repaint the window
		SetWindowPos( NULL, NULL, NULL, NULL, NULL, SWP_NOMOVE | SWP_NOSIZE | SWP_NOZORDER | SWP_NOACTIVATE | SWP_FRAMECHANGED );
		RedrawWindow( NULL, NULL, RDW_INVALIDATE | RDW_NOCHILDREN | RDW_UPDATENOW | RDW_FRAME );
	}

public:
// Constructors
	CTraceImpl() :
		m_bBorder(			false ),
		m_bClientEdge(		false ),
#ifndef _WIN32_WCE
		m_bThemedBorder(	false ),
		m_pTheme(			NULL  )
#else
		m_bThemedBorder(	false )
#endif
	{
		ZeroMemory( &m_rcClientPos, sizeof(m_rcClientPos) );
	}

	BOOL SubclassWindow(HWND hWnd)
	{
		ATLASSERT(m_hWnd==NULL);
		ATLASSERT(::IsWindow(hWnd));	//It is your responsibility to ensure that the handle parameter is, indeed, a richedit window

		m_bBorder	 = ::GetWindowLong(hWnd, GWL_STYLE) & WS_BORDER;
		m_bClientEdge = ::GetWindowLong(hWnd, GWL_EXSTYLE) & WS_EX_CLIENTEDGE;

#ifndef _WIN32_WCE
		m_pTheme = new CTheme();

		// Consider RichEdit IMF_AUTOFONT option for EM_SETLANGOPTIONS and EM_GETLANGOPTIONS.
		// Also consider Font Binding
		// Rich Edit 3.0 will assign a charset to plain-text characters depending on their context. Some examples are: 

		// Hangul symbols get HANGUL_CHARSET 
		// Non-neutral ANSI characters get ANSI_CHARSET in any event 
		// Chinese characters get SHIFTJIS_CHARSET if kana characters are found nearby or GB2312_CHARSET if no kana are found nearby. 
		// Greek characters get GREEK_CHARSET 
#endif

		CFont font = ::CreateFont( -12, 0, 0, 0, 0, 0, 0, 0, DEFAULT_CHARSET, 0, 0, 0, 0, _T("Courier New") );
		bool bRedraw = true;
		::SendMessage(hWnd, WM_SETFONT, (WPARAM)(HFONT)font, MAKELPARAM(bRedraw, 0));
		//::SendMessage(hWnd, WM_SETFONT, (WPARAM)(HFONT)GetStockObject(ANSI_FIXED_FONT), MAKELPARAM(bRedraw, 0));

#ifndef _WIN32_WCE
		if (CTheme::IsThemingSupported()) {
			//MessageBox( _T("posting WM_THEMECHANGED"), _T("CTraceImpl::SubclassWindow") );

			//Check the current state of the richedit control
			::PostMessage( hWnd, WM_THEMECHANGED, 0, 0 );
		}
#endif

	//Recalculate the NC area and repaint the window
//	SetWindowPos(NULL, NULL, NULL, NULL, NULL, SWP_NOMOVE|SWP_NOSIZE|SWP_NOZORDER|SWP_NOACTIVATE|SWP_FRAMECHANGED);
//	RedrawWindow(NULL, NULL, RDW_INVALIDATE|RDW_NOCHILDREN|RDW_UPDATENOW|RDW_FRAME);

		return CWindowImpl< T, TBase, TWinTraits >::SubclassWindow(hWnd);
	}

// Operations
	LRESULT Trace( PCTSTR pszBarCode, int nLen )
	{
		ATLASSERT(::IsWindow(m_hWnd));

		const int cbBuf = BCS_MAX_BARCODE_SIZE * 5 + 7;
		TCHAR szBuf[cbBuf];		szBuf[0] = '\0';
		TCHAR *pBuf = szBuf;
		TCHAR *pBufEnd = szBuf + cbBuf - 7;

		while (nLen-- > 0) {
			TCHAR c = *pszBarCode++;

			// Use current code page to print 33 - 126, 128 - 255.
			// Use ASCII codes for other characters.
			PCTSTR pszKeyName = NULL;
			if ( c >= 0 && c <= VK_SPACE )
				pszKeyName = AsciiName[c];
			else if ( c == 0x7F ) 
				pszKeyName = AsciiName[sizeof(AsciiName)/sizeof(PCSTR)-1];

			if ( pszKeyName ) {				// Copy key name
				while (*pBuf = *pszKeyName++)
					pBuf++;
			}
			else
				*pBuf++ = c;

			if (pBuf > pBufEnd)				// nLen parameter is too large
				return ERROR_INVALID_PARAMETER;
		}

		*pBuf++ = '\r';						// carriage return
		*pBuf++ = '\n';						// newline
		*pBuf++ = 0;						// NUL terminator
		ReplaceSel( szBuf );

		return ERROR_SUCCESS;
	}
};

//#ifndef _WIN32_WCE
class CTraceCtrl : public CTraceImpl<CTraceCtrl>
{
   DECLARE_WND_SUPERCLASS(_T("WTL_TraceCtrl"), GetWndClassName())  
};
//#else
#ifdef _WIN32_WCE
//const int VK_Base = 0xE000;	// Start of Unicode Private Use Area
#endif

#if 0
const int VK_Base = 0xE000;	// Start of Unicode Private Use Area

/// Synthesizes keystrokes.
/// Callback allows another class to format and send the keystrokes


typedef LRESULT (WINAPI *PFN_SEND_KEYSTROKES_CALLBACK)( void *pvParam, const char *pszBarCode, int nLength );

class CKeystrokes
{
	PFN_SEND_KEYSTROKES_CALLBACK m_pfnCallback;
	void *m_pvParam;

public:
	CKeystrokes() :
		m_pfnCallback(0),
		m_pvParam(0)
	{}

	LRESULT RegisterCallback( PFN_SEND_KEYSTROKES_CALLBACK pfnCallback, void *pvParam )
	{
		ATLASSERT( m_pfnCallback == NULL );

		m_pfnCallback	= pfnCallback;
		m_pvParam		= pvParam;

		return ERROR_SUCCESS;
	}

	LRESULT SendChar( wchar_t wc, WORD wShiftState = 0 )
	{
		BYTE	byVkCode;										//The virtual key code

		if (wc >= 0xE000 && wc <= 0xF8FF)						// Encoded virtual key
			byVkCode = (wc - 0xE000) & 0xff;					// Isolate the virtual key
		else {
			SHORT wVkCode = VkKeyScanW( wc );					// Get virtual-key code and shift state for the current keyboard
			byVkCode		 = LOBYTE( wVkCode );				// Isolate the virtual key
			wShiftState	|= HIBYTE( wVkCode );					// Bitwise or in the shift state bits
		}

		if (byVkCode) {
			if (wShiftState & 1)	keybd_event( VK_SHIFT,		0, 0, 0 );
			if (wShiftState & 2)	keybd_event( VK_CONTROL,	0, 0, 0 );
			if (wShiftState & 4)	keybd_event( VK_MENU,		0, 0, 0 );

			BYTE byScanCode = LOBYTE( MapVirtualKey(byVkCode, eVirtualKeyToScanCode) );
			keybd_event( byVkCode, byScanCode, 0, 0 );			// Simulate a key press and release
			keybd_event( byVkCode, byScanCode, KEYEVENTF_KEYUP, 0 );

			if (wShiftState & 1)	keybd_event( VK_SHIFT,		0, KEYEVENTF_KEYUP, 0);
			if (wShiftState & 2)	keybd_event( VK_CONTROL,	0, KEYEVENTF_KEYUP, 0);
			if (wShiftState & 4)	keybd_event( VK_MENU,		0, KEYEVENTF_KEYUP, 0);
		}

		return ERROR_SUCCESS;
	}

	LRESULT Send( const char *pszBarCode, int nLength = -1 )
	{
		ATLASSERT( pszBarCode );

		if (nLength < 0)										// Need to find the length
			nLength = strlen( pszBarCode );

		if (m_pfnCallback)										// Call the registered callback
			return (*m_pfnCallback)( m_pvParam, pszBarCode, nLength );

		while (nLength-- > 0 && *pszBarCode)
			SendChar( *pszBarCode++ );

		return ERROR_SUCCESS;
	}
};
#endif

/// Allows selection of a trim amount.
/// Incorporates spinner.

template< class T, class TBase = CEdit, class TWinTraits = CControlWinTraits >
class ATL_NO_VTABLE CTrimImpl : public CWindowImpl< T, TBase, TWinTraits >
{
	DECLARE_WND_SUPERCLASS(NULL, TBase::GetWndClassName())

	// Dialog controls
	CUpDownCtrl		udTrim;		// Provides a spinner for the edit box

	// Member variables
	unsigned int	m_nTrim;	// Number of character to trim

	// Private methods
	int SetWindowTextValue( unsigned int uValue )
	{
		// Performs a range check and writes the result to edit field.

		if ( uValue > BCS_MAX_BARCODE_SIZE )
			uValue = BCS_MAX_BARCODE_SIZE;

		TCHAR szMax[16];

#if defined(_CRT_NONSTDC_DEPRECATE)
		_ultot_s( uValue, szMax, (sizeof szMax/sizeof TCHAR), 10 );
#else
		_ultot( uValue, szMax, 10 );
#endif

		BOOL bRet = SetWindowText( szMax );
		return bRet ? uValue : -1;
	}

// Message map and handlers
	BEGIN_MSG_MAP_EX(CTrimImpl)
		MSG_WM_CREATE(		OnCreate	)
		MSG_OCM_COMMAND(	OnCommand	)
	END_MSG_MAP()

    // Message handlers
	LRESULT OnCreate(CREATESTRUCT *pCreateStruct)
	{
		LRESULT lRes = DefWindowProc();
		ATLASSERT( ::IsWindow(m_hWnd) );
		return lRes;
	}

	void OnCommand( UINT uNotifyCode, int nID, HWND /*hWndCtrl*/ )
	{
		switch (uNotifyCode) {
		case EN_UPDATE:						// 0x0400
		//case EN_ERRSPACE:					// 0x0500
		//case EN_MAXTEXT:					// 0x0501
			m_nTrim = GetValue();
			SetValue( m_nTrim );			// Gets rid of leading zeros
			break;
		}
		SetMsgHandled( false );
	}

public:
// Constructors
	CTrimImpl() :
		m_nTrim(0)
	{}

	BOOL SubclassWindow(HWND hWnd)
	{
		ATLASSERT(m_hWnd==NULL);
		ATLASSERT(::IsWindow(hWnd));
		return CWindowImpl< T, TBase, TWinTraits >::SubclassWindow(hWnd);
	}

// Operations
	LRESULT Initialize( HWND hWndBuddy )
	{
		udTrim	= hWndBuddy;

		const int nDigits = 2;

		// EM_LIMITTEXT: For edit controls and Microsoft Rich Edit 1.0, bytes are used.
		// For Rich Edit 2.0 and later, characters are used.

		const int nBytes = nDigits * sizeof(TCHAR);

		LimitText(  nBytes );	// Simple edit control, use bytes to restrict the number of digits

		udTrim	= hWndBuddy;

		// We set the buddy here in order to avoid initial EN_CHANGE message that is sent
		// when the updown control is created if the UDS_AUTOBUDDY style bit is set.

		udTrim.SetBuddy( m_hWnd );

		// Set the spinner range to be from 0 to the maximum bar code length.

		udTrim.SetRange(  0, BCS_MAX_BARCODE_SIZE );

		// Setting the position automatically sets the edit text since the updown has the
		// UDS_SETBUDDYINT style bit set.

		udTrim.SetPos(  m_nTrim );

		return ERROR_SUCCESS;
	}

	unsigned int GetValue()
	{
		// Reads a numeric edit field and converts the string to an integer.
		// Performs a range check and writes the modified result back to edit field.

		if (m_hWnd) {
			TCHAR szNum[4];
			GetWindowText( szNum, 4 );
			//GetLine( 1, szNum, 4 );
			int i = _ttoi( szNum );
			if ( i > BCS_MAX_BARCODE_SIZE )
				return SetWindowTextValue( BCS_MAX_BARCODE_SIZE );
			if ( i < 0 )
				return SetWindowText( _T("0") ), i = 0;
			return i;
		}
		
		return m_nTrim;
	}

	LRESULT SetValue( unsigned int nTrim )
	{
		if (m_hWnd) {
			if ( nTrim > BCS_MAX_BARCODE_SIZE )
				return ERROR_BAD_ARGUMENTS;

			if (SetWindowTextValue(nTrim) < 0)
				return -1;
		}

		m_nTrim = nTrim;

		return ERROR_SUCCESS;
	}

	LRESULT WriteToRegistry( CRegKey &rk, PCTSTR pszValueName )
	{
		// Write current value to registry
#if (_ATL_VER >= 0x0700)
		return rk.SetDWORDValue( pszValueName, GetValue() );
#else // !(_ATL_VER >= 0x0700)
		return rk.SetValue( GetValue(), pszValueName);
#endif // !(_ATL_VER >= 0x0700)
	}

	LRESULT ReadFromRegistry( CRegKey &rk, PCTSTR pszValueName )
	{
		DWORD dwValue = 0;

		// Override the default settings if saved values are available
#if (_ATL_VER >= 0x0700)
		LONG lRet = rk.QueryDWORDValue( pszValueName, dwValue );
#else // !(_ATL_VER >= 0x0700)
		LONG lRet = rk.QueryValue( dwValue, pszValueName);
#endif // !(_ATL_VER >= 0x0700)
		//REG ATLASSERT( lRet == ERROR_SUCCESS );

		if (lRet == ERROR_SUCCESS) {
			if (SetValue(dwValue) < 0)
				return -1;
		}

		return lRet;
	}
};

class CTrimCtrl : public CTrimImpl<CTrimCtrl>
{
   DECLARE_WND_SUPERCLASS(_T("WTL_TrimCtrl"), GetWndClassName())  
};

/// Limits contents to a single character.
/// Ensures that strings representing virtual keys cannot be edited

template< class T, class TBase = CEdit, class TWinTraits = CControlWinTraits >
class ATL_NO_VTABLE CKeyEditImpl : public CWindowImpl< T, TBase, TWinTraits >
{
	DECLARE_WND_SUPERCLASS(NULL, TBase::GetWndClassName())

	BEGIN_MSG_MAP_EX(CKeyEditCtrl)
		MESSAGE_HANDLER(WM_CREATE, OnCreate)
		MSG_WM_MOUSEMOVE(OnMouseMove)
		MESSAGE_HANDLER(WM_LBUTTONDOWN, OnLButtonDown)
	END_MSG_MAP()

   // Message Handlers

   LRESULT OnCreate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
   {
      LRESULT lRes = DefWindowProc(uMsg, wParam, lParam);
      _Init();
      return lRes;
   }

	void OnMouseMove( UINT nFlags, CPoint /*point*/ )
	{
		// This override maintains the selection over the entire string for strings longer than one
		// character.  

	   if (!(nFlags & MK_LBUTTON && LineLength() > 1))
			SetMsgHandled( false );
	}

	LRESULT OnLButtonDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
	{
		// This override prevents the user from changing the selection to an insertion point if the
		// selection is more than one character.  The strings representing virtual characters are 
		// more than one character long and should not be edited, only replaced.

		LRESULT lResult = DefWindowProc(uMsg, wParam, lParam);

		int nLen = LineLength();
		if (nLen > 1)
			SetSelAll();

		return lResult;
	}

public:
   // Operations

   BOOL SubclassWindow(HWND hWnd)
   {
      ATLASSERT(m_hWnd == NULL);
      ATLASSERT(::IsWindow(hWnd));
      BOOL bRet = CWindowImpl< CKeyEditCtrl, CEdit, CControlWinTraits >::SubclassWindow(hWnd);
      if ( bRet ) _Init();
      return bRet;
   }

   // Implementation

   void _Init()
   {
      ATLASSERT(::IsWindow(m_hWnd));

	  SetLimitText( 1 );					// Limit text input to one character
   }
};

class CKeyEditCtrl : public CKeyEditImpl<CKeyEditCtrl>
{
   DECLARE_WND_SUPERCLASS(_T("WTL_KeyEditCtrl"), GetWndClassName())  
};

/// Allows selection of a single character or virtual key.
/// Incorporates Ctrl check box

const wchar_t rgwVirtualKeys[] = {
	VK_Base,
	VK_Base + VK_RETURN,
	VK_Base + VK_TAB,
	VK_Base + VK_SPACE,
	VK_Base + VK_LEFT,
	VK_Base + VK_RIGHT,
	VK_Base + VK_UP,
	VK_Base + VK_DOWN
};

const int nItems = sizeof(rgwVirtualKeys)/sizeof(wchar_t);		// Number of items in combo box

static const TCHAR *rgszVirtualKeys[nItems] = {
	_T("None"),								// No character sent
	_T("{ENTER}"),							// Enter key
	_T("{TAB}"),							// Tab key
	_T("{SPACE}"),							// Space key
	_T("{LEFT}"),							// Left arrow key
	_T("{RIGHT}"),							// Right arrow key
	_T("{UP}"),								// Up arrow key
	_T("{DOWN}")							// Down arrow key
};

template< class T, class TBase = CComboBox, class TWinTraits = CControlWinTraits >
class ATL_NO_VTABLE CKeyImpl : public CWindowImpl< T, TBase, TWinTraits >
{
	DECLARE_WND_SUPERCLASS(NULL, TBase::GetWndClassName())

	// Dialog controls
	CButton			m_chkCtrl;
	CKeyEditCtrl	m_ctlEdit;

	// Member variables
	WORD	m_wState;
	wchar_t	m_wChar;

// Message map and handlers
	BEGIN_MSG_MAP_EX(CKeyImpl)
		MSG_WM_CREATE(OnCreate)
		COMMAND_CODE_HANDLER_EX( CBN_SELCHANGE,	OnSelChange	)
		COMMAND_CODE_HANDLER_EX( EN_CHANGE,		OnChange	)
	END_MSG_MAP()

	LRESULT OnCreate(CREATESTRUCT *pCreateStruct)
	{
		LRESULT lRes = DefWindowProc();
		ATLASSERT( ::IsWindow(m_hWnd) );
		return lRes;
	}

	void OnSelChange( UINT /*uNotifyCode*/, int /*nID*/, CWindow /*wndCtrl*/ )
	{
		// This override allows us to capture virtual keys selected via the drop down list.

		m_wChar = rgwVirtualKeys[GetCurSel()];
		SetMsgHandled( false );
	}

	void OnChange( UINT /*uNotifyCode*/, int /*nID*/, CEdit edCtrl )
	{
		if (edCtrl.LineLength() == 0) {		// User has deleted entry
			SetCurSel( 0 );					// Display "None"
			return;
		}

		// The following SetSelAll changes the behavior of the edit box portion of this combo box so
		// that it is always selected.  Pressing a suitable key changes the contents of the edit box
		// to the new key and selects it so that pressing another key causes the contents to be
		// replaced with the contents of the new key.

		// This behavior matches selecting an entry from the drop down.  The default behavior for
		// a drop down combo box is to set the selection to cover the entire string inserted into
		// the edit box.

		edCtrl.SetSelAll();

		// There should never be more than one character displayed in the edit box when we get to
		// this point.

		ATLASSERT( edCtrl.LineLength() == 1 );

		// Assign the first (and only) character in edit box to a member variable.

		wchar_t szBuf[2] = {0};
		if (::GetWindowTextW(edCtrl, szBuf, 2) == 1) {
			m_wChar = szBuf[0];
		}
		SetMsgHandled( false );
	}

public:
// Constructors
	CKeyImpl() :
		m_wChar(VK_Base)
	{}

// Operations
	BOOL SubclassWindow(HWND hWnd)
	{
		ATLASSERT(m_hWnd==NULL);
		ATLASSERT(::IsWindow(hWnd));
		return CWindowImpl< T, TBase, TWinTraits >::SubclassWindow(hWnd);
	}

	LRESULT Initialize( HWND hWndCtrl = NULL )
	{
		m_chkCtrl = hWndCtrl;

		COMBOBOXINFO cbInfo = {sizeof(cbInfo)};
		GetComboBoxInfo(&cbInfo);
		m_ctlEdit.SubclassWindow(cbInfo.hwndItem);

		for (int i = 0; i < nItems; i++) {
			AddString(rgszVirtualKeys[i]);
		}

		LRESULT lRes = SetChar( m_wChar );
		if (lRes == ERROR_SUCCESS)
			return SetState( m_wState );

		return lRes;
	}

// Attributes
	wchar_t GetChar()
	{
		return m_wChar;
	}

	LRESULT SetChar( wchar_t wChar )
	{
		if (wChar >= 0xE000 && wChar <= 0xF8FF) {				// Encoded virtual key
			// Search for virtual key
			int i = 0;
			while (i < nItems) {
				if (wChar == rgwVirtualKeys[i]) {				// Found a match
					if (m_hWnd)
						return SetCurSel(i) >= 0 ? ERROR_SUCCESS : CB_ERR;

					m_wChar = wChar;

					return ERROR_SUCCESS;
				}
				i++;
			}
			return ERROR_BAD_ARGUMENTS;
		}

		m_wChar = wChar;

		if (m_ctlEdit) {
			wchar_t szBuf[2] = {0};
			szBuf[0] = m_wChar;
			::SetWindowTextW( m_ctlEdit, szBuf );
		}

		return ERROR_SUCCESS;
	}

	WORD GetState()
	{
		if (m_chkCtrl)
			m_wState = m_chkCtrl.GetCheck() ? 2 : 0;

		return m_wState;
	}

	LRESULT SetState( WORD wState )
	{
		switch (wState) {
		case 0:	if (m_chkCtrl) m_chkCtrl.SetCheck(false);	break;
		case 2:	if (m_chkCtrl) m_chkCtrl.SetCheck(true);	break;
		default:	return  ERROR_BAD_ARGUMENTS;
		}
		m_wState = wState;

		return ERROR_SUCCESS;
	}

	LRESULT WriteToRegistry( CRegKey &rk, PCTSTR pszValueName )
	{
		// Write current value to registry
		DWORD dwValue = MAKELONG( m_wChar, GetState() );
#if (_ATL_VER >= 0x0700)
		return rk.SetDWORDValue( pszValueName, dwValue );
#else // !(_ATL_VER >= 0x0700)
		return rk.SetValue( dwValue, pszValueName );
#endif // !(_ATL_VER >= 0x0700)
	}

	LRESULT ReadFromRegistry( CRegKey &rk, PCTSTR pszValueName )
	{
		DWORD dwValue = 0;

		// Override the default settings if saved values are available
#if (_ATL_VER >= 0x0700)
		LONG lRet = rk.QueryDWORDValue( pszValueName, dwValue );
#else // !(_ATL_VER >= 0x0700)
		LONG lRet = rk.QueryValue( dwValue, pszValueName );
#endif // !(_ATL_VER >= 0x0700)
		//REG ATLASSERT( lRet == ERROR_SUCCESS );

		if (lRet == ERROR_SUCCESS) {
			LRESULT lRes = SetChar( LOWORD(dwValue) );
			if (lRes == ERROR_SUCCESS)
				return SetState( HIWORD(dwValue) );
		}

		return ERROR_SUCCESS;
	}
};

class CKeyCtrl : public CKeyImpl<CKeyCtrl>
{
   DECLARE_WND_SUPERCLASS(_T("WTL_KeyCtrl"), GetWndClassName())  
};

#endif // __SCLCTRLS_H__
