// Scanner Class Library - SCL version 0.01
// Copyright (C) Microvision, Inc. All rights reserved.

// scledit.h : interface of the CSEditView class

#pragma once

class CSEditView : public CWindowImpl<		CSEditView, CRichEditCtrl>,
					public CEditCommands<	CSEditView>
{
public:
	DECLARE_WND_SUPERCLASS(NULL, CRichEditCtrl::GetWndClassName())

	BEGIN_MSG_MAP_EX(CSEditView)
		MSG_WM_COMMAND( OnCommand )
        CHAIN_MSG_MAP_ALT(CEditCommands<CSEditView>, 1)
	END_MSG_MAP()

	void OnCommand(  UINT /*uNotifyCode*/, int nID, HWND /*hWnd*/ )
	{
		switch (nID) {
		case ID_EDIT_UNDO:		Undo();		break;
		case ID_EDIT_CUT:		Cut();		break;
		case ID_EDIT_COPY:		Copy();		break;
#ifndef _WIN32_WCE
		case ID_EDIT_PASTE:		PasteSpecial( CF_TEXT );	break;
#else // _WIN32_WCE
		case ID_EDIT_PASTE:		Paste();	break;
#endif // _WIN32_WCE
		}
	}

	bool LoadFile( PCTSTR pstrFilePath )
	{
		_ASSERTE( pstrFilePath != NULL );

#ifndef _WIN32_WCE
		HANDLE hFile = ::CreateFile( pstrFilePath, GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL | FILE_FLAG_SEQUENTIAL_SCAN, NULL );
		if (hFile == INVALID_HANDLE_VALUE)
			return false;

		EDITSTREAM es = {(DWORD)hFile, 0, StreamReadCallback};
		StreamIn( SF_TEXT, es );
		_ASSERTE( es.dwError == ERROR_SUCCESS );

		::CloseHandle( hFile );
		return es.dwError == ERROR_SUCCESS;

#else // _WIN32_WCE
		_ASSERTE( FALSE );
		return false;
#endif // _WIN32_WCE
	}

	static DWORD CALLBACK StreamReadCallback( DWORD dwCookie, BYTE *pbBuff, LONG cb, LONG *pcb )
	{
		_ASSERTE( dwCookie != 0 );
		_ASSERTE( pcb != NULL );

		if (::ReadFile((HANDLE)dwCookie, pbBuff, cb, (DWORD *)pcb, NULL))
			return ERROR_SUCCESS;

		return ::GetLastError();
	}

	bool SaveFile( PTSTR pstrFilePath )
	{
		_ASSERTE( pstrFilePath != NULL );

#ifndef _WIN32_WCE
		HANDLE hFile = ::CreateFile( pstrFilePath, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_ARCHIVE | FILE_FLAG_SEQUENTIAL_SCAN, NULL );
		if (hFile == INVALID_HANDLE_VALUE)
			return false;

		EDITSTREAM es = {(DWORD)hFile, 0, StreamWriteCallback};
		StreamOut( SF_TEXT, es );
		_ASSERTE( es.dwError == ERROR_SUCCESS );

		::CloseHandle( hFile );
		return es.dwError == ERROR_SUCCESS;

#else // _WIN32_WCE
		_ASSERTE( FALSE );
		return false;
#endif // _WIN32_WCE
	}

	static DWORD CALLBACK StreamWriteCallback( DWORD dwCookie, BYTE *pbBuff, LONG cb, LONG *pcb )
	{
		_ASSERTE( dwCookie != 0 );
		_ASSERTE( pcb != NULL );

		if (::WriteFile((HANDLE)dwCookie, pbBuff, cb, (DWORD *)pcb, NULL))
			return ERROR_SUCCESS;

		return ::GetLastError();
	}
};
