// Scanner Class Library - SCL version 0.01
// Copyright (C) Microvision, Inc. All rights reserved.

#ifndef __SCLREG_H__
#define __SCLREG_H__

#pragma once

#ifndef __cplusplus
  #error FTL requires C++ compilation (use a .cpp suffix)
#endif

#ifndef _WIN32_WCE
#pragma comment(lib, "shlwapi.lib")
#endif // _WIN32_WCE

///////////////////////////////////////////////////////////////////////////////
// Classes in this file:
//
// CRegistryHive
// CConnection

/// Manages the registry hive.
/// Switches to HKEY_CURRENT_USER if needed to provide writable registry hive.

// Global registry key string contants
static const TCHAR *pcszRegKeyRun			= _T("Software\\Microsoft\\Windows\\CurrentVersion\\Run");

class CRegistryHive
{
	const TCHAR	  *	m_pcszRegKeyApp;
	const TCHAR	  *	m_pcszAppName;
	HKEY			m_hkHive;
	bool			m_bWritableHiveSet;

	// Support functions
	LRESULT CopyKey( const TCHAR *pcszRegKey )
	{
		CRegKey rk;
		LRESULT lRes = rk.Create( HKEY_CURRENT_USER, pcszRegKey );
		if (lRes != ERROR_SUCCESS)
			return lRes;

		return ::SHCopyKey( HKEY_LOCAL_MACHINE, pcszRegKey, rk, 0 );
	}

public:
	CRegistryHive( const TCHAR *pcszRegKeyApp, const TCHAR *pcszAppName = NULL, bool bInitializeAsWriteable = true ) {
		// Initialize member variables
		m_pcszRegKeyApp	= pcszRegKeyApp;
		m_pcszAppName	= pcszAppName;

		// Establish access to the registry
		HKEY hKey = NULL;

		// Check HKEY_CURRENT_USER first.  If pcszRegKeyApp does not exist, start out with HKEY_LOCAL_MACHINE
		if (RegOpenKeyEx(HKEY_CURRENT_USER, pcszRegKeyApp, 0, KEY_WRITE | KEY_READ, &hKey) == ERROR_SUCCESS) {
			RegCloseKey( hKey );
			m_hkHive = HKEY_CURRENT_USER;
			m_bWritableHiveSet = true;
			//::MessageBox( 0, _T("Opened HKEY_CURRENT_USER"), pcszRegKeyApp, MB_OK );
		}
		else {
			m_hkHive = HKEY_LOCAL_MACHINE;
			m_bWritableHiveSet = false;
			if (bInitializeAsWriteable)
				GetWritableHive();
		}
	}

// Attributes
	operator HKEY() const	{ return m_hkHive;	}

// Operations
	HKEY GetWritableHive()
	{
		if (m_bWritableHiveSet)				// Writable hive has already been set
			return m_hkHive;

		m_bWritableHiveSet = true;			// This method only gets called once

		// Can we write to m_hkHive?
		CRegKey rkApp;
		LRESULT lRes = rkApp.Create( m_hkHive, m_pcszRegKeyApp );
		HKEY hKey = NULL;
		if ( lRes != ERROR_SUCCESS ) {
			//::MessageBox( 0, _T("Switching to HKEY_CURRENT_USER"), m_pcszRegKeyApp, MB_OK );

			ATLASSERT( lRes == ERROR_ACCESS_DENIED );

			if (m_pcszAppName) {			// Copy the Microsoft\\Windows\\CurrentVersion\\Run data from m_hkHive to HKEY_CURRENT_USER
				CRegKey rkRun;
				LRESULT lRes = rkRun.Open( HKEY_LOCAL_MACHINE, pcszRegKeyRun );
				if (lRes == ERROR_SUCCESS) {
					// Check for application name entry
					TCHAR szValue[_MAX_PATH];
					ULONG nChars = _MAX_PATH;
					lRes = rkRun.QueryStringValue( m_pcszAppName, szValue, &nChars );
					rkRun.Close();
					if (lRes == ERROR_SUCCESS) {
						// Add the application name entry
						if (rkRun.Open(HKEY_CURRENT_USER, pcszRegKeyRun) == ERROR_SUCCESS) {
							lRes = rkRun.SetStringValue( m_pcszAppName, szValue );
							rkRun.Close();
						}
					}
				}
			}

			lRes = CopyKey( m_pcszRegKeyApp );			// Copy the Scanner Wedge data from m_hkHive to HKEY_CURRENT_USER
			ATLASSERT( lRes == 0 );

			m_hkHive = HKEY_CURRENT_USER;
		}

		return m_hkHive;
	}
};

#endif // __SCLREG_H__
