#ifndef VERSION
#define VERSION

#pragma comment(lib, "version.lib")

#include <exception>

struct sLanguage {
	WORD wLanguage;
	WORD wCodePage;
};

class CVersion
{
	void *pVersionData;
	sLanguage *pLanguageInformation;

	PCTSTR GetValue( PCSTR pcszKey ) const
	{
		TCHAR *pszValue = _T("");

		if (pVersionData) {
			// Build the query string
			TCHAR szSpecifier[48];
			HRESULT hr = StringCbPrintf( szSpecifier, sizeof szSpecifier, _T("\\StringFileInfo\\%04x%04x\\%hs"),
				pLanguageInformation->wLanguage,
				pLanguageInformation->wCodePage,
				pcszKey
				);

			// Query for the requested value
			UINT uLen = 0;
			BOOL b = ::VerQueryValue( pVersionData, szSpecifier, (void **)&pszValue, &uLen );
			ATLASSERT( b );
		}

		return pszValue;
	}

public:
	CVersion( PCTSTR pcszModuleFileName )
	{
		// Get version info
		DWORD h = 0;
		DWORD cbVerInfo = ::GetFileVersionInfoSize( pcszModuleFileName, &h );

		if (cbVerInfo) {
			pVersionData = new BYTE[cbVerInfo];
			if (pVersionData) {
				if (::GetFileVersionInfo(pcszModuleFileName, 0, cbVerInfo, pVersionData)) {

					// Get language information
					UINT uLen = 0;
					if (::VerQueryValue(pVersionData, _T("\\VarFileInfo\\Translation"),
										reinterpret_cast<void **>(&pLanguageInformation), &uLen) == FALSE)
						throw std::exception("Requested localized version information not available");
				}
				else {
					char szException[80];
					HRESULT hr = StringCbPrintfA( szException, sizeof szException, "Could not get version information: %d", ::GetLastError() );
					throw std::exception( szException );
				}
			}
			else
				throw std::bad_alloc();
		}
		else {
			char szException[80];
			HRESULT hr = StringCbPrintfA( szException, sizeof szException, "No version information found: %d", ::GetLastError() );
			throw std::exception( szException );
		}

		VS_FIXEDFILEINFO *pFfi = 0;
		UINT uLen = 0;
		::VerQueryValue( pVersionData, _T("\\"), reinterpret_cast<void **>(&pFfi), &uLen );
	}

	~CVersion() { delete [] pVersionData; }

	PCTSTR ProductName()		const { return GetValue( "ProductName"		); }
	PCTSTR InternalName()		const { return GetValue( "InternalName"		); }
	PCTSTR ProductVersion()		const { return GetValue( "ProductVersion"	); }
	PCTSTR SpecialBuild()		const { return GetValue( "SpecialBuild"		); }
	PCTSTR PrivateBuild()		const { return GetValue( "PrivateBuild"		); }
	PCTSTR Copyright()			const { return GetValue( "LegalCopyright"	); }
	PCTSTR Trademarks()			const { return GetValue( "LegalTrademarks"	); }
	PCTSTR Comments()			const { return GetValue( "Comments"			); }
	PCTSTR CompanyName()		const { return GetValue( "CompanyName"		); }
	PCTSTR FileVersion()		const { return GetValue( "FileVersion"		); }
	PCTSTR FileDescription()	const { return GetValue( "FileDescription"	); }
};

#endif
