// MainDlg.h : interface of the CMainDlg class
//
/////////////////////////////////////////////////////////////////////////////

#pragma once

/// Main property sheet dialog implementation.
/// Handles data available notification.

class CMainDlg : public CPropertySheetImpl<CMainDlg>
{
	NOTIFYICONDATA	m_nid;
	UINT			WM_TRAYICON;
	BOOL			m_bNotifyIconInstalled;
	enum {			m_nDefault = ID_FILE_OPEN };

	void Reset()
	{
		NMHDR nmhdr = { m_hWnd, 0, PSN_RESET };

		// Send the notify message to each page
		for (int i = 0; i < GetPageCount(); i++) {
			HWND hWnd = IndexToHwnd(i);
			if (hWnd) ::SendMessage( hWnd, WM_NOTIFY, 0, (LPARAM)&nmhdr );
		}
	}

    // Maps
	BEGIN_MSG_MAP_EX(CMainDlg)
		COMMAND_HANDLER_EX(		IDOK,			BN_CLICKED,	OnOk				)
		COMMAND_HANDLER_EX(		IDCANCEL,		BN_CLICKED,	OnCancel			)

		COMMAND_ID_HANDLER_EX(	ID_APP_ABOUT,				OnFileOpen			)
		COMMAND_ID_HANDLER_EX(	ID_APP_EXIT,				OnAppExit			)
		COMMAND_ID_HANDLER_EX(	ID_FILE_OPEN,				OnFileOpen			)

		MESSAGE_HANDLER_EX(		WM_SYSCOMMAND,				OnSysCommand		)
		MESSAGE_HANDLER_EX(		WM_TRAYICON,				OnTrayIcon			)

		MESSAGE_HANDLER_EX(		SW_SETACTIVEPAGE,			OnSetActivePage		)

        CHAIN_MSG_MAP(CPropertySheetImpl<CMainDlg>)
	END_MSG_MAP()

	// Message handlers
	void OnOk( UINT /*uNotifyCode*/, int /*nID*/, HWND /*hWndCtrl*/ )
	{
		Apply();
		ShowWindow( SW_HIDE );
	}

	void OnCancel( UINT /*uNotifyCode*/, int /*nID*/, HWND /*hWndCtrl*/ )
	{
		Reset();
		ShowWindow( SW_HIDE );
	}

	void OnFileOpen( UINT /*uNotifyCode*/, int nID, HWND /*hWndCtrl*/ )
	{
		// Show dialog
		if (!IsWindowVisible())
			ShowWindow( SW_SHOW );

		// Restore if window if minimized
		if (IsIconic())
			ShowWindow( SW_RESTORE );
		else
			BringWindowToTop();

		// Make this the active window
		::SetForegroundWindow( m_hWnd );

		if ( nID == ID_APP_ABOUT )
			SetActivePage( 0 );
	}

	void OnAppExit( UINT /*uNotifyCode*/, int /*nID*/, HWND /*hWndCtrl*/ )
	{
		// Remove the icon from the task bar
		if ( m_bNotifyIconInstalled ) {
			m_bNotifyIconInstalled = false;
			Shell_NotifyIcon( NIM_DELETE, &m_nid );
		}

		// Use DestroyWindow to close a property sheet regardless of whether it is modal or modeless
		DestroyWindow();
	}

	LRESULT OnSysCommand( UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/ )
	{
		if((wParam & 0xFFF0) == SC_CLOSE)
			SendMessage( WM_CLOSE );
		else
			SetMsgHandled( false );

		return 0;
	}

	LRESULT OnTrayIcon( UINT /*uMsg*/, WPARAM wParam, LPARAM lParam )
	{
		if (wParam != m_nid.uID)			// Is this the ID we want?
			return 0;

		if (LOWORD(lParam) == WM_RBUTTONUP)	// Was the right-button clicked?
		{
			CMenu menu;
			if (!menu.LoadMenu(m_nid.uID))	// Load the menu
				return 0;

			// Get the sub-menu
			CMenuHandle oPopup(menu.GetSubMenu(0));

			CPoint pos;
			GetCursorPos(&pos);				// Get the menu position

			SetForegroundWindow( m_hWnd );	// Make app window the foreground window (MSDN ID: 135788, http://support.microsoft.com/kb/135788)
			oPopup.SetMenuDefaultItem( m_nDefault );
			oPopup.TrackPopupMenu( TPM_LEFTALIGN, pos.x, pos.y, m_hWnd );
			PostMessage( WM_NULL );			// Post a benign message to the window or thread (MSDN ID: 135788)
			menu.DestroyMenu();
		}
		else if (LOWORD(lParam) == WM_LBUTTONDBLCLK)
			SendMessage( WM_COMMAND, m_nDefault, 0 );

		return 0;
	}

	LRESULT OnSetActivePage( UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/ )
	{
		int nIndex = IdToIndex( wParam );
		if ( GetActiveIndex() != nIndex )
			SetActivePage( nIndex );

		return 0;
	}

public:
	CMainDlg()								// Constructor
	{
		memset( &m_nid, 0, sizeof(m_nid) );
		m_nid.cbSize = sizeof( NOTIFYICONDATA );

		m_bNotifyIconInstalled = false;
		WM_TRAYICON = ::RegisterWindowMessage(_T("WM_TRAYICON"));
	}

	static int CALLBACK PropSheetCallback(HWND hWnd, UINT uMsg, LPARAM lParam)
	{
		if (uMsg == PSCB_PRECREATE) {
			// Remove the DS_CONTEXTHELP style from the dialog box template
			if (((DLGTEMPLATEEX *)lParam)->signature == 0xFFFF) {
				((DLGTEMPLATEEX *)lParam)->style &= ~DS_CONTEXTHELP;
				if (pgOptions.StartHidden())
					((DLGTEMPLATEEX *)lParam)->style &= ~WS_VISIBLE;
			}
			else {
				((DLGTEMPLATE *)lParam)->style &= ~DS_CONTEXTHELP;
				if (pgOptions.StartHidden())
					((DLGTEMPLATE *)lParam)->style &= ~WS_VISIBLE;
			}
		}

		return CPropertySheetImpl<CMainDlg>::PropSheetCallback( hWnd, uMsg, lParam);
	}

	void OnSheetInitialized()
	{
		// Set icons
		HICON hIcon =		AtlLoadIconImage( IDR_MAINFRAME );
		SetIcon( hIcon );
		HICON hIconSmall =	AtlLoadIconImage( IDR_MAINFRAME, LR_DEFAULTCOLOR, ::GetSystemMetrics(SM_CXSMICON), ::GetSystemMetrics(SM_CYSMICON) );
		SetIcon( hIconSmall, FALSE );

		m_nid.hWnd = m_hWnd;
		m_nid.uID = IDR_POPUP;
		m_nid.uFlags = NIF_MESSAGE | NIF_ICON | NIF_TIP;
		m_nid.uCallbackMessage = WM_TRAYICON;
		m_nid.hIcon = hIconSmall;
		StringCbCopy( m_nid.szTip, sizeof m_nid.szTip, pcszAppName );

		m_bNotifyIconInstalled = Shell_NotifyIcon( NIM_ADD, &m_nid );
	}
};
