// Wedge.cpp : main source file for Wedge.exe
//

#include "stdafx.h"

#include "resource.h"
#include "VersionInfo.h"					// COMPANY_NAME, PRODUCT_NAME, PRODUCT_VERSION

// Contants
static const TCHAR *pcszAppName		= _T(PRODUCT_NAME);
static const TCHAR *pcszAppRegKey	= _T("Software\\") _T(COMPANY_NAME) _T("\\") _T(PRODUCT_NAME);

CRegistryHive regHive( pcszAppRegKey );		// Handles registry policy

enum										// Connection Messages
{
	SW_GETBARCODES = WM_APP + 0x100,		// 0x8100
	SW_SETACTIVEPAGE,						// 0x8101
	SW_SENDBARCODE,							// 0x8102
	SW_SENDDATA,							// 0x8103
	SW_DISPLAYPACKET						// 0x8104
};

bool bM2M;									// Use M2M
bool bData;									// Send/display Data

TCHAR szInitialConnectionId[BCS_MAX_NAME_SIZE];

CKeystrokes		  g_Keystrokes;				// Sends bar code as keystrokes
HANDLE			  hDataHeap;				// Private heap for transferring data between threads

bool SendBarCode(	const TCHAR *pszBarCode, int cchBarCode );
bool SendData(		const TCHAR *pszData,	 int cchData );
bool DisplayPacket(	const TCHAR *pszData,	 int cchData );

#include "PageDlgs.h"						// Property sheet page dialogs

CAboutPage		  pgAbout;					// About page object instance
CConnectionsPage  pgConnections;			// Connections page object instance
CTracePage		  pgTrace;					// Trace page object instance
CFormatPage		  pgFormat;					// Format page object instance
CSettingsPage	  pgSettings;				// Settings page object instance
COptionsPage	  pgOptions;				// Options page object instance

#include "MainDlg.h"						// Main property sheet dialog

CMainDlg dlgMain;							// Main property sheet dialog object instance

bool SendBarCode( const TCHAR *pszBarCode, int cbBarCode )
{
	bool bIsWedgeForegroundWindow = ::GetForegroundWindow() == dlgMain;

	// Send data to capture window or send keystrokes
	if ( bIsWedgeForegroundWindow )
		dlgMain.SendMessage( SW_SETACTIVEPAGE, IDD_TRACE );

	return pgTrace.SendBarCode( bIsWedgeForegroundWindow, pszBarCode, cbBarCode );
}

bool SendData( const TCHAR *pszData, int cbData )
{
	bool bIsWedgeForegroundWindow = ::GetForegroundWindow() == dlgMain;

	// Send data to capture window or send keystrokes
	if ( bIsWedgeForegroundWindow )
		dlgMain.SendMessage( SW_SETACTIVEPAGE, IDD_TRACE );

	return pgTrace.SendData( bIsWedgeForegroundWindow, pszData, cbData );
}

bool DisplayPacket( const TCHAR *pszData, int cbData )
{
	return pgTrace.DisplayPacket( dlgMain, pszData, cbData );
}

CAppModule _Module;

int WINAPI _tWinMain(HINSTANCE hInstance, HINSTANCE /*hPrevInstance*/, LPTSTR lpstrCmdLine, int nCmdShow)
{
	::CoInitialize( NULL );

    // NOTE: See the ActivatePreviousInstance method (atlwince.h) for a much more sophisticated
    // method of activating the previous version.

    // Allow only one instance of the application.
	HANDLE hMutex = CreateMutex( NULL, true, pcszAppName );

	// Check for previous instance of the application
	if (ERROR_ALREADY_EXISTS == GetLastError()) {
		HWND hWndPrev = FindWindow( WC_DIALOG, pcszAppName );
		if (hWndPrev) {
			SetForegroundWindow( hWndPrev );
			ShowWindow( hWndPrev, SW_SHOW );	// May be hidden
			return 0;
		}
	}

	// Initialize WTL
	HRESULT hRes = _Module.Init(NULL, hInstance);
	ATLASSERT(SUCCEEDED(hRes));

	// Parse command line, register or unregister or run the server
	TCHAR szTokens[] = _T("-/");

	PCTSTR pszToken = _Module.FindOneOf( ::GetCommandLine(), szTokens );
	while (pszToken != NULL) {
		if (lstrcmpi(pszToken, _T("M2M")) == 0) {
			bM2M = true;
			break;
		}
		else if (lstrcmpi(pszToken, _T("Data")) == 0) {
			bM2M = true;
			bData = true;
			break;
		}
		pszToken = _Module.FindOneOf( pszToken, szTokens );
	}

	HINSTANCE hInstRich = ::LoadLibrary(CRichEditCtrl::GetLibraryName());
	ATLASSERT(hInstRich);

//ImportBarCodes();

//---------------------------------------------------------------------------------------

	hDataHeap = HeapCreate( 0, 0, 0x10000 );	// Limit heap size to 65536 bytes

    // The block around the CMainDlg variable is important because CMainDlg may have members that
    // use ATL and WTL features. Those members may also use ATL/WTL features in their destructors.
    // If the block were not present, the CMainDlg destructor (and the destructors of the members)
    // would run after the call to _Module.Term() (which uninitializes ATL/WTL) and try to use
    // ATL/WTL features, which could cause a hard-to-diagnose crash.

	int nRet = 0;
	// BLOCK: Run application
	{
	    dlgMain.SetTitle( pcszAppName );	// Set the dialog caption

	    dlgMain.AddPage ( pgAbout );		// Add the pages
        dlgMain.AddPage ( pgConnections );
		dlgMain.AddPage ( pgTrace );
		dlgMain.AddPage ( pgFormat );
		dlgMain.AddPage ( pgSettings );
		dlgMain.AddPage ( pgOptions );

		if (pgOptions.AutoActivate()) {		// Attempt to automatically activate the scanner
			dlgMain.SetActivePage(1);		// Start with connections page
			CRegKey rkApp;					// Get the initial Connection Id value
			LRESULT lRes = rkApp.Open( regHive, pcszAppRegKey, KEY_READ );
			ATLASSERT( lRes == ERROR_SUCCESS );
			if (lRes == ERROR_SUCCESS) {
				ULONG nChars = BCS_MAX_NAME_SIZE;
				lRes = rkApp.QueryStringValue( _T("ConnectionId"), szInitialConnectionId, &nChars);
				rkApp.Close();
			}
		}

		nRet = dlgMain.DoModal();			// Display the main dialog
	}

// Free heap ???

	::FreeLibrary(hInstRich);

	_Module.Term();

	ReleaseMutex( hMutex );

	::CoUninitialize();

	return nRet;
}
